@@ -20,7 +20,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgacqctl.c,v 1.65 2002/02/28 19:09:03 tom Exp tom $
+ *  $Id: epgacqctl.c,v 1.78 2003/04/09 20:44:17 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGCTL
@@ -33,21 +33,22 @@
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
 
+#include "epgvbi/btdrv.h"
+#include "epgvbi/ttxdecode.h"
 #include "epgdb/epgblock.h"
 #include "epgdb/epgdbif.h"
+#include "epgdb/epgstream.h"
 #include "epgdb/epgqueue.h"
 #include "epgdb/epgtscqueue.h"
 #include "epgdb/epgdbsav.h"
 #include "epgdb/epgdbmgmt.h"
 #include "epgdb/epgdbmerge.h"
-#include "epgdb/epgdbacq.h"
 
 #include "epgui/uictrl.h"
 #include "epgui/epgmain.h"
 #include "epgctl/epgctxmerge.h"
 #include "epgctl/epgctxctl.h"
 #include "epgctl/epgscan.h"
-#include "epgvbi/btdrv.h"
 #include "epgctl/epgacqctl.h"
 #include "epgctl/epgacqsrv.h"
 #include "epgctl/epgacqclnt.h"
@@ -133,11 +134,11 @@ static void EpgAcqCtl_StatisticsReset( void )
 //
 static void EpgAcqCtl_StatisticsUpdate( void )
 {
-   ulong total, obsolete;
    time_t acqMinTime[2], aiDiff;
    time_t aiAcqTimestamp;
    time_t now = time(NULL);
    EPGDB_HIST * pHist;
+   uint total, obsolete;
    uint stream, idx;
 
    acqStatsUpdate = FALSE;
@@ -254,7 +255,8 @@ const EPGDB_STATS * EpgAcqCtl_GetAcqStats( void )
 
       if (acqCtl.mode != ACQMODE_NETWORK)
       {  // retrieve additional data from TTX packet decoder
-         EpgDbAcqGetStatistics(&acqStats.ttx.ttxPkgCount, &acqStats.ttx.epgPkgCount, &acqStats.ttx.epgPagCount);
+         TtxDecode_GetStatistics(&acqStats.ttx.ttxPkgCount,
+                                 &acqStats.ttx.epgPkgCount, &acqStats.ttx.epgPagCount);
       }
 
       pAcqStats = &acqStats;
@@ -271,14 +273,33 @@ const EPGDB_STATS * EpgAcqCtl_GetAcqStats( void )
 //   it allows to see what channel is tuned in instead of the acq provider's
 // - in network acq mode the VPS CNI is forwarded by the remote acq server
 //
-const EPGDB_ACQ_VPS_PDC * EpgAcqCtl_GetVpsPdc( void )
+const EPGDB_ACQ_VPS_PDC * EpgAcqCtl_GetVpsPdc( VPSPDC_REQ_ID clientId )
 {
-   if (acqCtl.state != ACQSTATE_OFF)
+   static bool updateVect[VPSPDC_REQ_COUNT] = {FALSE};
+   EPGDB_ACQ_VPS_PDC * result = NULL;
+
+   if (clientId < VPSPDC_REQ_COUNT)
    {
-      return &acqStats.vpsPdc;
+      if (acqCtl.state != ACQSTATE_OFF)
+      {
+         // poll for new VPS/PDC data
+         if (EpgAcqCtl_ProcessVps())
+         {
+            memset(updateVect, FALSE, sizeof(updateVect));
+         }
+
+         // if there are reults which have not been given yet to the client, return them
+         if ( (updateVect[clientId] == FALSE) || (clientId == VPSPDC_REQ_DAEMON) )
+         {
+            updateVect[clientId] = TRUE;
+            result = &acqStats.vpsPdc;
+         }
+      }
    }
    else
-      return NULL;
+      debug1("EpgAcqCtl-GetVpsPdc: illegal client id %d", clientId);
+
+   return result;
 }
 
 #ifdef USE_DAEMON
@@ -397,8 +418,9 @@ void EpgAcqCtl_ResetVpsPdc( void )
       acqStats.vpsPdc.cni = 0;
 
       if (acqCtl.mode != ACQMODE_NETWORK)
-      {  // reset VPS/PDC acq state machine
-         EpgDbAcqResetVpsPdc();
+      {  // reset EPG and ttx decoder state machine
+         EpgAcqCtl_ChannelChange(FALSE);
+         EpgAcqCtl_StatisticsUpdate();
       }
       else
       {  // tell server to send the next newly received VPS/PDC CNI & PIL
@@ -409,6 +431,68 @@ void EpgAcqCtl_ResetVpsPdc( void )
    }
 }
 
+// ---------------------------------------------------------------------------
+// Start teletext decoding and initialize the EPG stream decoder
+//
+static void EpgAcqCtl_TtxStart( EPGDB_CONTEXT *dbc, EPGDB_QUEUE * pQueue, uint pageNo, uint appId )
+{
+   uint epgPageNo;
+   uint epgAppId;
+   bool bWaitForBiAi;
+
+   if (pageNo != EPG_ILLEGAL_PAGENO)
+      epgPageNo = dbc->pageNo = pageNo;
+   else if ((dbc->pageNo != EPG_ILLEGAL_PAGENO) && VALID_EPG_PAGENO(dbc->pageNo))
+      epgPageNo = dbc->pageNo;
+   else
+      epgPageNo = dbc->pageNo = EPG_DEFAULT_PAGENO;
+
+   if (appId != EPG_ILLEGAL_APPID)
+      epgAppId = appId;
+   else if (dbc->appId != EPG_ILLEGAL_APPID)
+      epgAppId = dbc->appId;
+   else
+      epgAppId = EPG_DEFAULT_APPID;
+
+   if (dbc->pAiBlock != NULL)
+   {  // provider already known
+      // set up a list of alphabets for string decoding
+      EpgBlockSetAlphabets(&dbc->pAiBlock->blk.ai);
+      // since alphabets are known PI can be collected right from the start
+      bWaitForBiAi = FALSE;
+   }
+   else
+   {  // unknown provider -> wait for AI before allowing PI
+      bWaitForBiAi = TRUE;
+   }
+
+   // initialize the state of the streams decoder
+   EpgStreamInit(pQueue, bWaitForBiAi, epgAppId, epgPageNo);
+
+   TtxDecode_StartEpgAcq(epgPageNo, FALSE);
+}
+
+// ---------------------------------------------------------------------------
+// Stop teletext decoding and clear the EPG stream and its queue
+//
+static void EpgAcqCtl_TtxStop( void )
+{
+   TtxDecode_StopAcq();
+   EpgStreamClear();
+}
+
+// ---------------------------------------------------------------------------
+// Stop and Re-Start teletext acquisition
+// - called after change of channel or internal parameters
+//
+static void EpgAcqCtl_TtxReset( EPGDB_CONTEXT *dbc, EPGDB_QUEUE * pQueue, uint pageNo, uint appId )
+{
+   // discard remaining blocks in the scratch buffer
+   EpgStreamClear();
+
+   EpgAcqCtl_TtxStart(dbc, pQueue, pageNo, appId);
+}
+
 // ---------------------------------------------------------------------------
 // Close the acq database
 // - automatically chooses the right close function for the acq database
@@ -531,22 +615,19 @@ bool EpgAcqCtl_Start( void )
       else
       {
          // initialize teletext decoder
-         EpgDbAcqInit();
          EpgDbQueue_Init(&acqDbQueue);
          EpgTscQueue_Init(&acqTsc);
 
          acqCtl.chanChangeTime = 0;
          EpgAcqCtl_InitCycle();
-         EpgAcqCtl_UpdateProvider(FALSE);
 
-         EpgDbAcqStart(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
-
-         if (BtDriver_StartAcq() == FALSE)
-         {  // VBI slave process/thread does not exist or failed to start acq -> hold acq
-            EpgDbAcqStop();
-         }
-         else
+         if (BtDriver_StartAcq())
+         {
+            // set input source and tuner frequency (also detect if device is busy)
+            EpgAcqCtl_UpdateProvider(FALSE);
+            EpgAcqCtl_TtxStart(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
             result = TRUE;
+         }
       }
 
       if (result)
@@ -598,7 +679,7 @@ void EpgAcqCtl_Stop( void )
       else
       {
          BtDriver_StopAcq();
-         EpgDbAcqStop();
+         EpgAcqCtl_TtxStop();
          EpgTscQueue_Clear(&acqTsc);
       }
 
@@ -611,7 +692,7 @@ void EpgAcqCtl_Stop( void )
 }
 
 // ---------------------------------------------------------------------------
-// Enable or disable BI/AI callback handling for EPG scan
+// Stop acquisition for the EPG scan but keep the device open/driver loaded
 //
 void EpgAcqCtl_Suspend( bool suspend )
 {
@@ -622,12 +703,25 @@ void EpgAcqCtl_Suspend( bool suspend )
    else
    {
       EpgAcqCtl_CloseDb();
+      EpgStreamClear();
       EpgTscQueue_Clear(&acqTsc);
 
       acqCtl.state = ACQSTATE_OFF;
    }
 }
 
+// ---------------------------------------------------------------------------
+// Query cause for the last acquisition failure
+// - must only be called when acq start failed
+//
+const char * EpgAcqCtl_GetLastError( void )
+{
+   if (acqCtl.mode != ACQMODE_NETWORK)
+      return BtDriver_GetLastError();
+   else
+      return NULL;
+}
+
 // ---------------------------------------------------------------------------
 // Initialize the module for daemon mode
 //
@@ -641,7 +735,7 @@ void EpgAcqCtl_InitDaemon( void )
 // Set input source and tuner frequency for a provider
 // - errors are reported to the user interface
 //
-static bool EpgAcqCtl_TuneProvider( ulong freq, uint cni )
+static bool EpgAcqCtl_TuneProvider( uint freq, uint cni )
 {
    bool isTuner;
    bool result = FALSE;
@@ -654,31 +748,20 @@ static bool EpgAcqCtl_TuneProvider( ulong freq, uint cni )
    // remember time of channel change
    acqCtl.chanChangeTime = time(NULL);
 
+   // tune onto the provider's channel (before starting acq, to avoid catching false data)
    // always set the input source - may have been changed externally (since we don't hog the video device permanently)
-   if ( BtDriver_SetInputSource(acqCtl.inputSource, TRUE, &isTuner) )
+   if ( BtDriver_TuneChannel(acqCtl.inputSource, freq, FALSE, &isTuner) )
    {
       if (isTuner)
       {
          if (freq != 0)
          {
-            // tune onto the provider's channel (before starting acq, to avoid catching false data)
-            if (BtDriver_TuneChannel(freq, FALSE))
-            {
-               BtDriver_CloseDevice();
-               result = TRUE;
-            }
-            else
-            {
-               dprintf0("EpgAcqCtl-TuneProv: failed to tune channel -> force to passive mode\n");
-               acqCtl.mode = ACQMODE_FORCED_PASSIVE;
-               acqCtl.passiveReason = ACQPASSIVE_ACCESS_DEVICE;
-            }
+            result = TRUE;
          }
          else
          {
             dprintf0("EpgAcqCtl-TuneProv: no freq in db\n");
             // close the device, which was kept open after setting the input source
-            BtDriver_CloseDevice();
             if (cni != 0)
             {  // inform the user that acquisition will not be possible
                UiControlMsg_MissingTunerFreq(cni);
@@ -755,7 +838,7 @@ static uint EpgAcqCtl_GetProvider( void )
 static bool EpgAcqCtl_UpdateProvider( bool changeDb )
 {
    EPGDB_CONTEXT * pPeek;
-   ulong freq = 0;
+   uint freq = 0;
    uint cni;
    bool warnInputError;
    bool dbChanged, inputChanged, dbInitial;
@@ -769,7 +852,7 @@ static bool EpgAcqCtl_UpdateProvider( bool changeDb )
       // determine current CNI depending on acq mode and index
       cni = EpgAcqCtl_GetProvider();
 
-      // for acq start: open the requested db (or any database for passive mode)
+      // for acq start: open the requested db (or empty database for passive or network mode)
       if (pAcqDbContext == NULL)
       {
          dbInitial = TRUE;
@@ -837,7 +920,7 @@ static bool EpgAcqCtl_UpdateProvider( bool changeDb )
       {  // no provider to be tuned onto -> set at least the input source
          if (acqCtl.mode != ACQMODE_PASSIVE)
          {
-            result = EpgAcqCtl_TuneProvider(0L, 0);
+            result = EpgAcqCtl_TuneProvider(0, 0);
             inputChanged = ((acqCtl.mode != ACQMODE_FORCED_PASSIVE) || (acqCtl.passiveReason != ACQPASSIVE_ACCESS_DEVICE));
          }
       }
@@ -1067,6 +1150,10 @@ bool EpgAcqCtl_CheckDeviceAccess( void )
         (EpgScan_IsActive() == FALSE) )
    {
       result = EpgAcqCtl_UpdateProvider(FALSE);
+
+      if (result == FALSE)
+         UiControlMsg_AcqEvent(ACQ_EVENT_STATS_UPDATE);
+
       dprintf1("EpgAcqCtl-CheckDeviceAccess: device is now %s\n", result ? "free" : "busy");
    }
    else
@@ -1261,11 +1348,13 @@ void EpgAcqCtl_DescribeAcqState( EPGACQ_DESCR * pAcqState )
 //
 static bool EpgAcqCtl_AiCallback( const AI_BLOCK *pNewAi )
 {
+   EPGACQ_STATE   oldState;
    const AI_BLOCK *pOldAi;
-   ulong oldTunerFreq;
+   uint ai_cni;
    bool accept = FALSE;
 
    assert(EpgScan_IsActive() == FALSE);  // EPG scan uses a separate callback
+   oldState = acqCtl.state;
 
    if (AI_GET_CNI(pNewAi) == 0)
    {
@@ -1283,75 +1372,51 @@ static bool EpgAcqCtl_AiCallback( const AI_BLOCK *pNewAi )
          dprintf3("EpgCtl: AI found, CNI=0x%04X version %d/%d\n", AI_GET_CNI(pNewAi), pNewAi->version, pNewAi->version_swo);
       EpgDbLockDatabase(pAcqDbContext, TRUE);
       pOldAi = EpgDbGetAi(pAcqDbContext);
+      ai_cni = AI_GET_CNI(pNewAi);
 
       if (pOldAi == NULL)
       {  // the current db is empty
-         oldTunerFreq = pAcqDbContext->tunerFreq;
          EpgDbLockDatabase(pAcqDbContext, FALSE);
 
-         EpgAcqCtl_SwitchDb(AI_GET_CNI(pNewAi));
-         dprintf2("EpgAcqCtl: empty acq db, AI found: CNI 0x%04X (%s)\n", AI_GET_CNI(pNewAi), ((pAcqDbContext->pAiBlock == NULL) ? "new" : "reload ok"));
+         // this is just like a provider change...
+
+         EpgAcqCtl_SwitchDb(ai_cni);
+         dprintf2("EpgAcqCtl: empty acq db, AI found: CNI 0x%04X (%s)\n", ai_cni, ((EpgDbContextGetCni(pAcqDbContext) == 0) ? "new" : "reload ok"));
 
          #ifdef USE_DAEMON
          // inform server module about the current provider
-         EpgAcqServer_SetProvider(AI_GET_CNI(pNewAi));
+         EpgAcqServer_SetProvider(ai_cni);
          #endif
 
-         EpgDbLockDatabase(pAcqDbContext, TRUE);
-         // if this is not a new db, handle it like a provider change below
-         pOldAi = EpgDbGetAi(pAcqDbContext);
-         if (pOldAi == NULL)
+         // store the tuner frequency if none is known yet
+         if ( (pAcqDbContext->tunerFreq == 0) && (acqCtl.mode != ACQMODE_NETWORK) )
          {
-            if (acqCtl.mode != ACQMODE_NETWORK)
-            {
-               if (oldTunerFreq != 0)
-               {
-                  dprintf1("EpgAcqCtl: setting tuner freq: %.2f\n", (double)oldTunerFreq/16);
-                  pAcqDbContext->tunerFreq = oldTunerFreq;
-               }
-               else
-               {  // freq for db unknown -> try to obtain it from the driver
-                  pAcqDbContext->tunerFreq = BtDriver_QueryChannel();
-                  debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)pAcqDbContext->tunerFreq/16, AI_GET_CNI(pNewAi));
-               }
-
-               if (pAcqDbContext->tunerFreq != 0)
-               {  // store the provider channel frequency in the rc/ini file
-                  UiControlMsg_NewProvFreq(AI_GET_CNI(pNewAi), pAcqDbContext->tunerFreq);
-               }
+            // try to obtain the frequency from the driver (not supported by all drivers)
+            pAcqDbContext->tunerFreq = BtDriver_QueryChannel();
+            if (pAcqDbContext->tunerFreq != 0)
+            {  // store the provider channel frequency in the rc/ini file
+               debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)(pAcqDbContext->tunerFreq & 0xffffff)/16, ai_cni);
+               UiControlMsg_NewProvFreq(ai_cni, pAcqDbContext->tunerFreq);
             }
+         }
 
-            // update the ui netwop list if neccessary
-            UiControlMsg_AcqEvent(ACQ_EVENT_PROV_CHANGE);
-
-            acqCtl.state = ACQSTATE_RUNNING;
-            EpgAcqCtl_StatisticsReset();
-
-            // new provider -> dump asap
+         // new provider -> dump asap
+         if (EpgDbContextGetCni(pAcqDbContext) == 0)
             acqCtl.dumpTime = 0;
 
-            accept = TRUE;
-         }
+         acqCtl.state = ACQSTATE_RUNNING;
+         acqStatsUpdate = TRUE;
+
+         // update the ui netwop list and acq stats output if neccessary
+         UiControlMsg_AcqEvent(ACQ_EVENT_PROV_CHANGE);
+
+         accept = TRUE;
       }
+      else
+      {  // already an AI in the database
 
-      if (pOldAi != NULL)
-      {
          if (AI_GET_CNI(pOldAi) == AI_GET_CNI(pNewAi))
          {
-            if ( (acqCtl.state == ACQSTATE_WAIT_AI) &&
-                 (acqCtl.mode != ACQMODE_NETWORK) )
-            {
-               if (pAcqDbContext->tunerFreq == 0)
-               {  // freq for db unknown -> try to obtain it from the driver
-                  pAcqDbContext->tunerFreq = BtDriver_QueryChannel();
-                  debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)pAcqDbContext->tunerFreq/16, AI_GET_CNI(pNewAi));
-               }
-               else
-               {  // store the provider channel frequency in the rc/ini file
-                  UiControlMsg_NewProvFreq(AI_GET_CNI(pNewAi), pAcqDbContext->tunerFreq);
-               }
-            }
-
             if ( (pOldAi->version != pNewAi->version) ||
                  (pOldAi->version_swo != pNewAi->version_swo) )
             {
@@ -1388,6 +1453,7 @@ static bool EpgAcqCtl_AiCallback( const AI_BLOCK *pNewAi )
                if (netwop >= pNewAi->netwopCount)
                   UiControlMsg_AcqEvent(ACQ_EVENT_STATS_UPDATE);
             }
+            EpgDbLockDatabase(pAcqDbContext, FALSE);
 
             acqStatsUpdate = TRUE;
             acqCtl.state = ACQSTATE_RUNNING;
@@ -1395,42 +1461,67 @@ static bool EpgAcqCtl_AiCallback( const AI_BLOCK *pNewAi )
          }
          else
          {  // wrong provider -> switch acq database
-            dprintf2("EpgAcqCtl: switching acq db from %04X to %04X\n", AI_GET_CNI(pOldAi), AI_GET_CNI(pNewAi));
+            dprintf2("EpgAcqCtl: switching acq db from %04X to %04X\n", AI_GET_CNI(pOldAi), ai_cni);
             EpgDbLockDatabase(pAcqDbContext, FALSE);
 
-            EpgAcqCtl_SwitchDb(AI_GET_CNI(pNewAi));
+            EpgAcqCtl_SwitchDb(ai_cni);
 
             if (acqCtl.mode != ACQMODE_NETWORK)
             {
-               EpgDbAcqReset(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
+               if (acqCtl.state == ACQSTATE_RUNNING)
+               {  // should normally not happen, because channel changes are detected by TTX page header supervision
+                  debug1("EpgAcqCtl: unexpected prov change to CNI %04X - resetting EPG Acq", ai_cni);
+                  EpgAcqCtl_TtxReset(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
 
-               // if db is new, dump asap
-               if (EpgDbContextGetCni(pAcqDbContext) == 0)
-                  acqCtl.dumpTime = 0;
+                  // if db is new, dump asap
+                  if (EpgDbContextGetCni(pAcqDbContext) == 0)
+                     acqCtl.dumpTime = 0;
+                  else
+                     acqCtl.dumpTime = time(NULL);
+
+                  acqCtl.state = ACQSTATE_WAIT_BI;
+               }
                else
-                  acqCtl.dumpTime = time(NULL);
-
-               acqCtl.state = ACQSTATE_WAIT_BI;
+                  acqCtl.state = ACQSTATE_RUNNING;
             }
 
             #ifdef USE_DAEMON
             // inform server module about the current provider
-            EpgAcqServer_SetProvider(AI_GET_CNI(pNewAi));
+            EpgAcqServer_SetProvider(ai_cni);
             #endif
 
-            EpgAcqCtl_StatisticsReset();
+            if (acqCtl.state != ACQSTATE_RUNNING)
+               EpgAcqCtl_StatisticsReset();
+            else
+               acqStatsUpdate = TRUE;
             UiControlMsg_AcqEvent(ACQ_EVENT_PROV_CHANGE);
 
-            EpgDbLockDatabase(pAcqDbContext, TRUE);
             accept = TRUE;
          }
-      }
 
-      EpgDbLockDatabase(pAcqDbContext, FALSE);
+         // Store the TV tuner frequency if none is known yet
+         // (Note: the tuner frequency in the db is never overwritten because we can
+         // not be 100% sure that we know the current TV tuner freq here, esp. if it's
+         // coming from a remote TV application; only the EPG scan updates the freq.)
+         if ( (pAcqDbContext->tunerFreq == 0) &&
+              (oldState == ACQSTATE_WAIT_AI) && (accept) &&
+              (acqCtl.state == ACQSTATE_RUNNING) &&
+              (acqCtl.mode != ACQMODE_NETWORK) )
+         {
+            // try to obtain the frequency from the driver (not supported by all drivers)
+            pAcqDbContext->tunerFreq = BtDriver_QueryChannel();
+            if (pAcqDbContext->tunerFreq != 0)
+            {  // store the provider channel frequency in the rc/ini file
+               debug2("EpgAcqCtl: setting current tuner frequency %.2f for CNI 0x%04X", (double)(pAcqDbContext->tunerFreq & 0xffffff)/16, ai_cni);
+               UiControlMsg_NewProvFreq(ai_cni, pAcqDbContext->tunerFreq);
+            }
+         }
+      }
    }
    else
       assert(acqCtl.state != ACQSTATE_OFF);
 
+   assert(EpgDbIsLocked(pAcqDbContext) == FALSE);
    return accept;
 }
 
@@ -1456,7 +1547,8 @@ static bool EpgAcqCtl_BiCallback( const BI_BLOCK *pNewBi )
          if (pAcqDbContext->appId != pNewBi->app_id)
          {  // not the default id
             dprintf2("EpgCtl: app-ID changed from %d to %d\n", pAcqDbContext->appId, pAcqDbContext->appId);
-            EpgDbAcqReset(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, pNewBi->app_id);
+            EpgStreamClear();
+            EpgStreamInit(&acqDbQueue, TRUE, pNewBi->app_id, pAcqDbContext->pageNo);
          }
       }
       else
@@ -1564,7 +1656,7 @@ static void EpgAcqCtl_ChannelChange( bool changeDb )
             UiControlMsg_AcqEvent(ACQ_EVENT_STATS_UPDATE);
       }
 
-      EpgDbAcqReset(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
+      EpgAcqCtl_TtxReset(pAcqDbContext, &acqDbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
       acqCtl.state = ACQSTATE_WAIT_BI;
       EpgAcqCtl_StatisticsReset();
    }
@@ -1572,18 +1664,20 @@ static void EpgAcqCtl_ChannelChange( bool changeDb )
 
 // ----------------------------------------------------------------------------
 // Poll VPS/PDC for channel changes
-// - called every 250 ms from main event loop
-// - if CNI or PIL changes, the user configured action is launched
+// - invoked when local client requests VPS/PDC data
+// - called every 250 ms from daemon main loop
+// - if CNI or PIL changes, it's forwarded to connected clients (if requested)
 //
 bool EpgAcqCtl_ProcessVps( void )
 {
    uint newCni, newPil;
    bool change = FALSE;
+   bool update = FALSE;
 
    if ( (acqCtl.state != ACQSTATE_OFF) && (EpgScan_IsActive() == FALSE) &&
         (acqCtl.mode != ACQMODE_NETWORK) )
    {
-      if (EpgDbAcqGetCniAndPil(&newCni, &newPil))
+      if (TtxDecode_GetCniAndPil(&newCni, &newPil, NULL))
       {
          if ( (acqStats.vpsPdc.cni != newCni) ||
               ((newPil != acqStats.vpsPdc.pil) && VPS_PIL_IS_VALID(newPil)) )
@@ -1601,9 +1695,11 @@ bool EpgAcqCtl_ProcessVps( void )
          // inform server module about the current CNI and PIL (even if no change)
          EpgAcqServer_SetVpsPdc(change);
          #endif
+
+         update = TRUE;
       }
    }
-   return change;
+   return update;
 }
 
 // ---------------------------------------------------------------------------
@@ -1679,9 +1775,9 @@ bool EpgAcqCtl_ProcessPackets( void )
       if (acqCtl.mode != ACQMODE_NETWORK)
       {
          // check if new data is available in the VBI ringbuffer
-         if (EpgDbAcqCheckForPackets(&stopped))
+         if (TtxDecode_CheckForPackets(&stopped))
          {  // assemble VBI lines, i.e. TTX packets to EPG blocks and put them in the queue
-            if (EpgDbAcqProcessPackets() == FALSE)
+            if (EpgStreamProcessPackets() == FALSE)
             {  // channel change detected -> restart acq for the current provider
                dprintf0("EpgAcqCtl: uncontrolled channel change detected\n");
                EpgAcqCtl_ChannelChange(TRUE);
@@ -1693,14 +1789,14 @@ bool EpgAcqCtl_ProcessPackets( void )
          if (stopped == FALSE)
          {
             // check the MIP if EPG is transmitted on a different page
-            pageNo = EpgDbAcqGetMipPageNo();
+            pageNo = TtxDecode_GetMipPageNo();
             if ((pageNo != EPG_ILLEGAL_PAGENO) && (pageNo != pAcqDbContext->pageNo))
             {  // found a different page number in MIP
                dprintf2("EpgAcqCtl: non-default MIP page no for EPG: %03X (was %03X) -> restart acq\n", pageNo, pAcqDbContext->pageNo);
 
                // XXX the pageno should not be saved before RUNNING since this might be the wrong db
                pAcqDbContext->pageNo = pageNo;
-               EpgDbAcqReset(pAcqDbContext, &acqDbQueue, pageNo, EPG_ILLEGAL_APPID);
+               EpgAcqCtl_TtxReset(pAcqDbContext, &acqDbQueue, pageNo, EPG_ILLEGAL_APPID);
             }
          }
          else
