@@ -20,27 +20,30 @@
  *  Author:
  *          Tom Zoerner
  *
- *  $Id: epgacqsrv.c,v 1.5 2002/02/28 19:11:09 tom Exp tom $
+ *  $Id: epgacqsrv.c,v 1.12 2003/10/05 19:19:37 tom Exp $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGCTL
 #define DPRINTF_OFF
 
+#ifdef USE_DAEMON
+
 #include <stdio.h>
 #include <string.h>
-#include <signal.h>
 #include <time.h>
 
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
 #include "epgctl/epgversion.h"
 #include "epgdb/epgblock.h"
+#include "epgdb/epgswap.h"
 #include "epgdb/epgqueue.h"
 #include "epgdb/epgtscqueue.h"
 #include "epgdb/epgdbmgmt.h"
 #include "epgdb/epgdbmerge.h"
 #include "epgdb/epgdbsav.h"
 #include "epgdb/epgnetio.h"
+#include "epgui/uictrl.h"
 #include "epgctl/epgacqctl.h"
 #include "epgctl/epgacqsrv.h"
 
@@ -72,6 +75,7 @@ typedef struct REQUEST_struct
 
    SRV_REQ_STATE   state;
    EPGNETIO_STATE  io;
+   bool            endianSwap;
 
    EPGNETIO_DUMP   dump;
    uint            dumpProvIdx;
@@ -231,7 +235,7 @@ static void EpgAcqServer_BuildVpsPdcMsg( EPGDBSRV_STATE * req )
 {
    const EPGDB_ACQ_VPS_PDC  * pVpsPdc;
 
-   pVpsPdc = EpgAcqCtl_GetVpsPdc();
+   pVpsPdc = EpgAcqCtl_GetVpsPdc(VPSPDC_REQ_DAEMON);
    if (pVpsPdc != NULL)
    {
       dprintf5("EpgAcqServer-BuildVpsPdcMsg: %02d.%02d. %02d:%02d (0x%04X)\n", (pVpsPdc->pil >> 15) & 0x1F, (pVpsPdc->pil >> 11) & 0x0F, (pVpsPdc->pil >>  6) & 0x1F, (pVpsPdc->pil      ) & 0x3F, pVpsPdc->cni);
@@ -359,7 +363,7 @@ static void EpgAcqServer_EnableBlockFwd( EPGDBSRV_STATE * req )
 static void EpgAcqServer_Close( EPGDBSRV_STATE * req, bool closeAll )
 {
    dprintf1("EpgAcqServer-Close: fd %d\n", req->io.sock_fd);
-   EpgNetIo_Logger(LOG_INFO, req->io.sock_fd, "closing connection", NULL);
+   EpgNetIo_Logger(LOG_INFO, req->io.sock_fd, 0, "closing connection", NULL);
 
    EpgNetIo_CloseIO(&req->io);
 
@@ -387,6 +391,8 @@ static void EpgAcqServer_AddConnection( int listen_fd )
    sock_fd = EpgNetIo_AcceptConnection(listen_fd);
    if (sock_fd != -1)
    {
+      dprintf1("EpgAcqServer-AddConnection: fd %d\n", sock_fd);
+
       req = xmalloc(sizeof(EPGDBSRV_STATE));
       memset(req, 0, sizeof(EPGDBSRV_STATE));
 
@@ -404,26 +410,67 @@ static void EpgAcqServer_AddConnection( int listen_fd )
 // ----------------------------------------------------------------------------
 // Checks the size of a message from client to server
 //
-static bool EpgAcqServer_CheckMsg( uint len, EPGNETIO_MSG_HEADER * pHead, EPGDBSRV_MSG_BODY * pBody )
+static bool EpgAcqServer_CheckMsg( uint len, EPGNETIO_MSG_HEADER * pHead, EPGDBSRV_MSG_BODY * pBody, bool * pEndianSwap )
 {
-   bool result;
+   uint idx;
+   bool result = FALSE;
 
    switch (pHead->type)
    {
       case MSG_TYPE_CONNECT_REQ:
-         result = (len == sizeof(EPGNETIO_MSG_HEADER) + sizeof(pBody->con_req)) &&
-                  (memcmp(pBody->con_req.magic, MAGIC_STR, MAGIC_STR_LEN) == 0);
+         if ( (len == sizeof(EPGNETIO_MSG_HEADER) + sizeof(pBody->con_req)) &&
+              (memcmp(pBody->con_req.magic, MAGIC_STR, MAGIC_STR_LEN) == 0) )
+         {
+            if (pBody->con_req.endianMagic == PROTOCOL_ENDIAN_MAGIC)
+            {
+               *pEndianSwap = FALSE;
+               result       = TRUE;
+            }
+            else if (pBody->con_req.endianMagic == PROTOCOL_WRONG_ENDIAN)
+            {
+               *pEndianSwap = TRUE;
+               result       = TRUE;
+            }
+         }
          break;
+
       case MSG_TYPE_FORWARD_REQ:
-         result = (len == sizeof(EPGNETIO_MSG_HEADER) + sizeof(pBody->fwd_req)) &&
-                  (pBody->fwd_req.cniCount <= MAX_MERGED_DB_COUNT);
+         if (len == sizeof(EPGNETIO_MSG_HEADER) + sizeof(pBody->fwd_req))
+         {
+            if (*pEndianSwap)
+            {
+               swap32(&pBody->fwd_req.statsReqBits);
+               swap32(&pBody->fwd_req.cniCount);
+               if (pBody->fwd_req.cniCount <= MAX_MERGED_DB_COUNT)
+               {
+                  for (idx=0; idx < pBody->fwd_req.cniCount; idx++)
+                  {
+                     swap32(&pBody->fwd_req.dumpStartTimes[idx]);
+                     swap32(&pBody->fwd_req.provCnis[idx]);
+                  }
+                  result = TRUE;
+               }
+            }
+            else if (pBody->fwd_req.cniCount <= MAX_MERGED_DB_COUNT)
+            {
+               result = TRUE;
+            }
+         }
          break;
+
       case MSG_TYPE_STATS_REQ:
-         result = (len == sizeof(EPGNETIO_MSG_HEADER) + sizeof(pBody->stats_req));
+         if (len == sizeof(EPGNETIO_MSG_HEADER) + sizeof(pBody->stats_req))
+         {
+            if (*pEndianSwap)
+               swap32(&pBody->stats_req.statsReqBits);
+            result = TRUE;
+         }
          break;
+
       case MSG_TYPE_CLOSE_IND:
          result = (len == sizeof(EPGNETIO_MSG_HEADER));
          break;
+
       case MSG_TYPE_CONNECT_CNF:
       case MSG_TYPE_FORWARD_CNF:
       case MSG_TYPE_FORWARD_IND:
@@ -589,26 +636,26 @@ static bool EpgAcqServer_TakeMessage( EPGDBSRV_STATE *req, EPGDBSRV_MSG_BODY * p
 // ----------------------------------------------------------------------------
 // Set bits for all active sockets in fd_set for select syscall
 //
-uint EpgAcqServer_GetFdSet( fd_set * rd, fd_set * wr )
+sint EpgAcqServer_GetFdSet( fd_set * rd, fd_set * wr )
 {
    EPGDBSRV_STATE    *req;
-   uint              max;
+   sint              max_fd;
 
    // add TCP/IP and UNIX-domain listening sockets
-   max = 0;
+   max_fd = 0;
    if ((srvState.max_conn == 0) || (srvState.conCount < srvState.max_conn))
    {
       if (srvState.tcp_ip_fd != -1)
       {
          FD_SET(srvState.tcp_ip_fd, rd);
-         if (srvState.tcp_ip_fd > max)
-             max = srvState.tcp_ip_fd;
+         if (srvState.tcp_ip_fd > max_fd)
+             max_fd = srvState.tcp_ip_fd;
       }
       if (srvState.pipe_fd != -1)
       {
          FD_SET(srvState.pipe_fd, rd);
-         if (srvState.pipe_fd > max)
-             max = srvState.pipe_fd;
+         if (srvState.pipe_fd > max_fd)
+             max_fd = srvState.pipe_fd;
       }
    }
 
@@ -631,11 +678,11 @@ uint EpgAcqServer_GetFdSet( fd_set * rd, fd_set * wr )
       else
          FD_SET(req->io.sock_fd, rd);
 
-      if (req->io.sock_fd > max)
-          max = req->io.sock_fd;
+      if (req->io.sock_fd > max_fd)
+          max_fd = req->io.sock_fd;
    }
 
-   return max;
+   return max_fd;
 }
 
 // ----------------------------------------------------------------------------
@@ -685,7 +732,7 @@ void EpgAcqServer_HandleSockets( fd_set * rd, fd_set * wr )
             // check for finished read -> process request
             if ( (req->io.readLen != 0) && (req->io.readLen == req->io.readOff) )
             {
-               if (EpgAcqServer_CheckMsg(req->io.readLen, &req->io.readHeader, (EPGDBSRV_MSG_BODY *) req->io.pReadBuf))
+               if (EpgAcqServer_CheckMsg(req->io.readLen, &req->io.readHeader, (EPGDBSRV_MSG_BODY *) req->io.pReadBuf, &req->endianSwap))
                {
                   req->io.readLen  = 0;
 
@@ -699,7 +746,6 @@ void EpgAcqServer_HandleSockets( fd_set * rd, fd_set * wr )
                }
                else
                {  // message has illegal size or content
-                  req->io.pReadBuf = NULL;
                   EpgAcqServer_Close(req, FALSE);
                }
             }
@@ -735,19 +781,26 @@ void EpgAcqServer_HandleSockets( fd_set * rd, fd_set * wr )
          }
          else if (req->state == SRV_STATE_DUMP_ACQ)
          {  // Perform dump of AI and OI#0 of non-requested db
-            if ( (EpgNetIo_DumpAiOi(&req->io, pAcqDbContext, &req->dump) == FALSE) && (req->io.sock_fd != -1) )
+            if (pAcqDbContext != NULL)
             {
-               if ( (req->statsReqBits & STATS_REQ_BITS_TSC_REQ) &&
-                    (req->statsReqBits & STATS_REQ_BITS_TSC_ALL) &&
-                    (req->enableAllBlocks == FALSE) )
-               {  // if this is a non-forwarded provider but acq timescales are open,
-                  // generate full timescale info
-                  EpgTscQueue_AddAll(&req->tscQueue, pAcqDbContext);
-               }
-               req->msgBuf.fwd_ind.cni = srvState.lastFwdCni;
-               EpgNetIo_WriteMsg(&req->io, MSG_TYPE_FORWARD_IND, sizeof(req->msgBuf.fwd_ind), &req->msgBuf.fwd_ind, FALSE);
+               if ( (EpgNetIo_DumpAiOi(&req->io, pAcqDbContext, &req->dump) == FALSE) && (req->io.sock_fd != -1) )
+               {
+                  if ( (req->statsReqBits & STATS_REQ_BITS_TSC_REQ) &&
+                       (req->statsReqBits & STATS_REQ_BITS_TSC_ALL) &&
+                       (req->enableAllBlocks == FALSE) )
+                  {  // if this is a non-forwarded provider but acq timescales are open,
+                     // generate full timescale info
+                     EpgTscQueue_AddAll(&req->tscQueue, pAcqDbContext);
+                  }
+                  req->msgBuf.fwd_ind.cni = srvState.lastFwdCni;
+                  EpgNetIo_WriteMsg(&req->io, MSG_TYPE_FORWARD_IND, sizeof(req->msgBuf.fwd_ind), &req->msgBuf.fwd_ind, FALSE);
 
-               req->state = SRV_STATE_FORWARD;
+                  req->state = SRV_STATE_FORWARD;
+               }
+            }
+            else
+            {  // acquisition has been stopped (can only occur during server shutdown)
+               EpgAcqServer_Close(req, TRUE);
             }
          }
          else if ( (req->doVpsPdc) &&
@@ -858,7 +911,7 @@ void EpgAcqServer_Destroy( void )
    if (srvState.pipe_fd != -1)
    {
       EpgNetIo_StopListen(FALSE, srvState.pipe_fd);
-      EpgNetIo_Logger(LOG_NOTICE, -1, "shutting down", NULL);
+      EpgNetIo_Logger(LOG_NOTICE, -1, 0, "shutting down", NULL);
    }
    if (srvState.tcp_ip_fd != -1)
    {
@@ -869,9 +922,7 @@ void EpgAcqServer_Destroy( void )
    EpgAcqServer_SetAddress(FALSE, NULL, NULL);
    EpgNetIo_SetLogging(0, 0, NULL);
 
-   #ifdef WIN32
-   WSACleanup();
-   #endif
+   EpgNetIo_Destroy();
 }
 
 // ----------------------------------------------------------------------------
@@ -879,20 +930,7 @@ void EpgAcqServer_Destroy( void )
 //
 void EpgAcqServer_Init( bool have_tty )
 {
-   #ifndef WIN32
-   struct sigaction   act;
-
-   // setup signal handler to ignore SIGPIPE from socket I/O
-   memset(&act, 0, sizeof(act));
-   sigemptyset(&act.sa_mask);
-   act.sa_handler = SIG_IGN;
-   sigaction(SIGPIPE, &act, NULL);
-
-   #else // WIN32
-   WSADATA stWSAData;
-
-   WSAStartup(0x202, &stWSAData);
-   #endif
+   EpgNetIo_Init(NULL);
 
    // initialize state struct
    memset(&srvState, 0, sizeof(srvState));
@@ -911,8 +949,12 @@ bool EpgAcqServer_Listen( void )
    if (EpgNetIo_CheckConnect() == FALSE)
    {
       // create named socket in /tmp for listening to local clients
+      #ifndef WIN32
       srvState.pipe_fd = EpgNetIo_ListenSocket(FALSE, NULL, NULL);
       if (srvState.pipe_fd != -1)
+      #else
+      srvState.pipe_fd = -1;
+      #endif
       {
          if (srvState.do_tcp_ip)
          {
@@ -920,19 +962,27 @@ bool EpgAcqServer_Listen( void )
             srvState.tcp_ip_fd = EpgNetIo_ListenSocket(TRUE, srvState.listen_ip, srvState.listen_port);
             if (srvState.tcp_ip_fd != -1)
             {
-               EpgNetIo_Logger(LOG_NOTICE, -1, "started listening on local and TCP/IP socket", NULL);
+               #ifndef WIN32
+               EpgNetIo_Logger(LOG_NOTICE, -1, 0, "started listening on local and TCP/IP socket, port ", srvState.listen_port, NULL);
+               #else
+               EpgNetIo_Logger(LOG_NOTICE, -1, 0, "started listening on TCP/IP socket, port ", srvState.listen_port, NULL);
+               #endif
                result = TRUE;
             }
          }
          else
          {  // no TCP/IP socket requested
-            EpgNetIo_Logger(LOG_NOTICE, -1, "started listening on local socket", NULL);
+            #ifndef WIN32
+            EpgNetIo_Logger(LOG_NOTICE, -1, 0, "started listening on local socket", NULL);
             result = TRUE;
+            #else
+            EpgNetIo_Logger(LOG_ERR, -1, 0, "TCP/IP not enabled - only TCP/IP supported for Win32", NULL);
+            #endif
          }
       }
    }
    else
-      EpgNetIo_Logger(LOG_ERR, -1, "a nxtvepg daemon is already running", NULL);
+      EpgNetIo_Logger(LOG_ERR, -1, 0, "a nxtvepg daemon is already running", NULL);
 
    return result;
 }
@@ -1151,3 +1201,4 @@ void EpgAcqServer_AddBlock( EPGDB_CONTEXT * dbc, EPGDB_BLOCK * pNewBlock )
    }
 }
 
+#endif  // USE_DAEMON
