@@ -31,13 +31,15 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgctxctl.c,v 1.18 2002/02/13 21:04:05 tom Exp tom $
+ *  $Id: epgctxctl.c,v 1.24 2003/10/05 19:14:28 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGCTL
 #define DPRINTF_OFF
 
 #include <time.h>
+#include <string.h>
+#include <errno.h>
 
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
@@ -848,13 +850,37 @@ time_t EpgContextCtl_GetAiUpdateTime( uint cni )
    return lastAiUpdate;
 }
 
+// ---------------------------------------------------------------------------
+// Set PI "cut-off" time, i.e. time offset after which expired PI are removed from db
+// - assigned to all open databases (already removed PI are not recoverable though)
+// - also passed to reload module (where it's stored internally for future loads)
+//
+void EpgContextCtl_SetPiExpireDelay( time_t expireDelay )
+{
+   CTX_CACHE  * pWalk;
+
+   EpgDbSavSetPiExpireDelay(expireDelay);
+
+   pWalk = pContextCache;
+   while (pWalk != NULL)
+   {
+      if (pWalk->pDbContext != NULL)
+      {
+         pWalk->pDbContext->expireDelayPi = expireDelay;
+
+         EpgDbExpire(pWalk->pDbContext);
+      }
+      pWalk = pWalk->pNext;
+   }
+}
+
 // ---------------------------------------------------------------------------
 // Update tuner frequency for a database
 // - allows to change the frequency without loading the complete db
 // - used by EPG scan for already known providers (the provider might have been
 //   found in passive mode when the channel freq. could not be determined)
 //
-bool EpgContextCtl_UpdateFreq( uint cni, ulong freq )
+bool EpgContextCtl_UpdateFreq( uint cni, uint freq )
 {
    CTX_CACHE * pContext;
    bool result;
@@ -864,7 +890,7 @@ bool EpgContextCtl_UpdateFreq( uint cni, ulong freq )
    pContext = EpgContextCtl_SearchCni(cni);
    if ((pContext != NULL) && (pContext->state == CTX_CACHE_OPEN))
    {  // db is open -> just set the frequency there
-      dprintf3("EpgContextCtl-UpdateFreq: update freq for opened db 0x%04X: %lu -> %lu\n", cni, pContext->pDbContext->tunerFreq, freq);
+      dprintf3("EpgContextCtl-UpdateFreq: update freq for opened db 0x%04X: %u -> %u\n", cni, pContext->pDbContext->tunerFreq, freq);
 
       // set the flag that the db needs to be saved
       if (pContext->pDbContext->tunerFreq != freq)
@@ -875,7 +901,7 @@ bool EpgContextCtl_UpdateFreq( uint cni, ulong freq )
    }
    else
    {  // db is not open -> write the frequency into the file header
-      debug2("EpgContextCtl-UpdateFreq: update freq for non-open db 0x%04X: %lu", cni, freq);
+      debug2("EpgContextCtl-UpdateFreq: update freq for non-open db 0x%04X: %u", cni, freq);
       result = EpgDbDumpUpdateHeader(cni, freq);
    }
    return result;
@@ -956,11 +982,11 @@ const uint * EpgContextCtl_GetProvList( uint * pCount )
 // - for databases which are not cached because they are defective or have and
 //   incompatible version, the tuner frequency is read from the header
 //
-uint EpgContextCtl_GetFreqList( uint ** ppProvList, ulong ** ppFreqList )
+uint EpgContextCtl_GetFreqList( uint ** ppProvList, uint ** ppFreqList )
 {
    CTX_CACHE * pContext;
    uint maxCount, idx;
-   ulong freq;
+   uint freq;
 
    // determine the number of providers, including "error" contexts
    maxCount = 0;
@@ -974,8 +1000,8 @@ uint EpgContextCtl_GetFreqList( uint ** ppProvList, ulong ** ppFreqList )
    if (maxCount > 0)
    {
       // allocate memory for the result lists
-      *ppProvList = xmalloc(maxCount * sizeof(uint));
-      *ppFreqList = xmalloc(maxCount * sizeof(uint));
+      *ppProvList = xmalloc(maxCount * sizeof(**ppProvList));
+      *ppFreqList = xmalloc(maxCount * sizeof(**ppFreqList));
 
       // build the lists
       idx = 0;
@@ -1022,6 +1048,57 @@ uint EpgContextCtl_GetFreqList( uint ** ppProvList, ulong ** ppFreqList )
    return maxCount;
 }
 
+// ---------------------------------------------------------------------------
+// Remove a provider from the cache and remove the database
+// - fails if the database is still opened
+// - returns 0 on success, EBUSY if db open, or POSIX error code from file unlink
+//
+uint EpgContextCtl_Remove( uint cni )
+{
+   CTX_CACHE * pContext;
+   uint result = 0;
+
+   // search for the context in the cache
+   pContext = pContextCache;
+   while (pContext != NULL)
+   {
+      if (pContext->provCni == cni)
+         break;
+      pContext = pContext->pNext;
+   }
+
+   if (pContext != NULL)
+   {
+      if ( (pContext->openRefCount == 0) &&
+           (pContext->peekRefCount == 0) )
+      {
+         if (pContext->pDbContext != NULL)
+         {
+            EpgDbDestroy(pContext->pDbContext, FALSE);
+            pContext->pDbContext = NULL;
+         }
+
+         pContext->state     = CTX_CACHE_ERROR;
+         pContext->reloadErr = EPGDB_RELOAD_EXIST;
+
+         result = EpgDbRemoveDatabaseFile(cni);
+      }
+      else
+      {  // db still opened -> abort
+         debug3("EpgContextCtl-Remove: cannot remove open db 0x%04X: ref=%d+%d", cni, pContext->openRefCount, pContext->peekRefCount);
+         result = EBUSY;
+      }
+   }
+   else
+      result = EpgDbRemoveDatabaseFile(cni);
+
+   // a missing database file is no error
+   if (result == ENOENT)
+      result = 0;
+
+   return result;
+}
+
 // ---------------------------------------------------------------------------
 // Free all cached contexts
 // - called upon program shutdown to free all resources
