@@ -34,7 +34,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgscan.c,v 1.18 2002/01/02 17:06:54 tom Exp tom $
+ *  $Id: epgscan.c,v 1.35 2003/06/27 20:27:40 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGCTL
@@ -46,21 +46,21 @@
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
 
+#include "epgvbi/cni_tables.h"
+#include "epgvbi/vbidecode.h"
+#include "epgvbi/tvchan.h"
+#include "epgvbi/btdrv.h"
+#include "epgvbi/ttxdecode.h"
 #include "epgdb/epgblock.h"
 #include "epgdb/epgdbif.h"
 #include "epgdb/epgdbsav.h"
 #include "epgdb/epgstream.h"
 #include "epgdb/epgqueue.h"
-#include "epgdb/epgdbacq.h"
 #include "epgdb/epgdbmgmt.h"
 
 #include "epgui/uictrl.h"
 #include "epgctl/epgctxctl.h"
 #include "epgctl/epgacqctl.h"
-#include "epgvbi/cni_tables.h"
-#include "epgvbi/vbidecode.h"
-#include "epgvbi/tvchan.h"
-#include "epgvbi/btdrv.h"
 
 #include "epgctl/epgscan.h"
 
@@ -72,22 +72,25 @@ typedef struct
 {
    EPGSCAN_STATE    state;            // state machine
    bool             acqWasEnabled;    // flag if acq was active before start of scan
+   uint             inputSrc;         // driver input source (only TV tuner is allowed)
    uint             channel;          // currently scanned channel
    uint             channelIdx;       // index of channel currently checked
    uint             signalFound;      // number of channels with signal detected
    uint             newProvCount;     // number of providers found
    uint             badProvCount;     // number of defective providers found (refresh mode)
    time_t           startTime;        // start of acq for the current channel
+   time_t           extraWait;        // extra seconds to wait until timeout
    bool             doSlow;           // user option: do not skip channels w/o reception
    bool             doRefresh;        // user option: check already known providers only
    bool             useXawtv;         // user option: work on predefined channel list
    uint             provFreqCount;    // number of known frequencies (xawtv and refresh mode)
-   ulong          * provFreqTab;      // list of known frequencies (xawtv and refresh mode)
+   uint           * provFreqTab;      // list of known frequencies (xawtv and refresh mode)
    uint           * provCniTab;       // list of known provider CNIs (refresh mode)
    EPGDB_CONTEXT  * pDbContext;       // database context for BI/AI search and reload
    EPGDB_QUEUE      dbQueue;          // queue for incoming EPG blocks
    bool             foundBi;          // TRUE when BI found on current channel
-   void          (* MsgCallback)( const char * pMsg );
+   EPGSCAN_MSGCB  * MsgCallback;      // callback function to print messages
+   EPGSCAN_DELCB  * DelCallback;      // callback function to print messages
 } EPGSCANCTL_STATE;
 
 static EPGSCANCTL_STATE scanCtl = {SCAN_STATE_OFF};
@@ -103,7 +106,7 @@ static const EPGDB_ADD_CB epgScanCb =
 // ----------------------------------------------------------------------------
 // Tune the next channel
 //
-static bool EpgScan_NextChannel( ulong * pFreq )
+static bool EpgScan_NextChannel( uint * pFreq )
 {
    bool result = FALSE;
 
@@ -134,12 +137,14 @@ static bool EpgScan_NextChannel( ulong * pFreq )
 static bool EpgScan_AiCallback( const AI_BLOCK *pNewAi )
 {
    const AI_BLOCK *pOldAi;
-   ulong oldTunerFreq;
+   uint oldTunerFreq;
    uint oldAppId;
    uchar msgbuf[80];
    bool accept = FALSE;
 
-   if ((scanCtl.state >= SCAN_STATE_WAIT_EPG) && (AI_GET_CNI(pNewAi) != 0))
+   if ( ( (scanCtl.state == SCAN_STATE_WAIT_NI_OR_EPG) ||
+          (scanCtl.state == SCAN_STATE_WAIT_EPG) ) &&
+        (AI_GET_CNI(pNewAi) != 0))
    {
       dprintf3("EpgScan: AI found, CNI=0x%04X version %d/%d\n", AI_GET_CNI(pNewAi), pNewAi->version, pNewAi->version_swo);
       EpgDbLockDatabase(scanCtl.pDbContext, TRUE);
@@ -158,14 +163,20 @@ static bool EpgScan_AiCallback( const AI_BLOCK *pNewAi )
          if (EpgDbContextGetCni(scanCtl.pDbContext) == 0)
          {  // new provider -> nothing to do here, just add the AI and keep all params
             sprintf(msgbuf, "Found %sprovider: %s", (scanCtl.doRefresh ? "" : "new "), AI_GET_NETWOP_NAME(pNewAi, pNewAi->thisNetwop));
-            scanCtl.MsgCallback(msgbuf);
+            scanCtl.MsgCallback(msgbuf, TRUE);
             // count the number of newly found providers
             scanCtl.newProvCount += 1;
          }
          else
          {
-            sprintf(msgbuf, "Provider already available: %s", AI_GET_NETWOP_NAME(pNewAi, pNewAi->thisNetwop));
-            scanCtl.MsgCallback(msgbuf);
+            sprintf(msgbuf, "Provider '%s' is %s.",
+                            AI_GET_NETWOP_NAME(pNewAi, pNewAi->thisNetwop),
+                            scanCtl.doRefresh ? "ok" : "already known - skipping");
+            scanCtl.MsgCallback(msgbuf, TRUE);
+
+            // in refresh mode count this provider as done (used in the summary)
+            if (scanCtl.doRefresh)
+               scanCtl.newProvCount += 1;
          }
 
          // update parameters with the new ones, if neccessary
@@ -173,13 +184,13 @@ static bool EpgScan_AiCallback( const AI_BLOCK *pNewAi )
 
          if (scanCtl.pDbContext->tunerFreq != oldTunerFreq)
          {
-            dprintf2("EpgScan: updating tuner freq: %.2f -> %.2f\n", (double)scanCtl.pDbContext->tunerFreq/16, (double)oldTunerFreq/16);
+            debug3("EpgScan-AiCallback: CNI 0x%04X: updating tuner freq: 0x%x -> 0x%x", AI_GET_CNI(pNewAi), scanCtl.pDbContext->tunerFreq, oldTunerFreq);
             scanCtl.pDbContext->modified = TRUE;
             scanCtl.pDbContext->tunerFreq = oldTunerFreq;
 
             if (EpgDbContextGetCni(scanCtl.pDbContext) != 0)
             {
-               scanCtl.MsgCallback("storing provider's tuner frequency");
+               scanCtl.MsgCallback("storing provider's tuner frequency", TRUE);
                EpgContextCtl_UpdateFreq(AI_GET_CNI(pNewAi), scanCtl.pDbContext->tunerFreq);
             }
 
@@ -210,7 +221,8 @@ static bool EpgScan_AiCallback( const AI_BLOCK *pNewAi )
 //
 static bool EpgScan_BiCallback( const BI_BLOCK *pNewBi )
 {
-   if (scanCtl.state >= SCAN_STATE_WAIT_EPG)
+   if ( (scanCtl.state == SCAN_STATE_WAIT_NI_OR_EPG) ||
+        (scanCtl.state == SCAN_STATE_WAIT_EPG) )
    {
       if (pNewBi->app_id == EPG_ILLEGAL_APPID)
       {
@@ -226,7 +238,8 @@ static bool EpgScan_BiCallback( const BI_BLOCK *pNewBi )
             if (scanCtl.pDbContext->appId != pNewBi->app_id)
             {  // not the default id
                dprintf2("EpgCtl: app-ID changed from %d to %d\n", scanCtl.pDbContext->appId, scanCtl.pDbContext->appId);
-               EpgDbAcqReset(scanCtl.pDbContext, &scanCtl.dbQueue, EPG_ILLEGAL_PAGENO, pNewBi->app_id);
+               EpgStreamClear();
+               EpgStreamInit(&scanCtl.dbQueue, TRUE, pNewBi->app_id, scanCtl.pDbContext->pageNo);
             }
          }
          else
@@ -238,55 +251,6 @@ static bool EpgScan_BiCallback( const BI_BLOCK *pNewBi )
    return FALSE;
 }
 
-// ----------------------------------------------------------------------------
-// Process all available lines from VBI and check for BI and AI blocks
-// 
-void EpgScan_ProcessPackets( void )
-{
-   bool stopped;
-   uint pageNo;
-
-   if (scanCtl.pDbContext != NULL)
-   {
-      // the check function must be called even if EPG acq is not yet enabled
-      // because it also handles channel changes and watches over the slave's life
-      if (EpgDbAcqCheckForPackets(&stopped))
-      {
-         if (scanCtl.state >= SCAN_STATE_WAIT_EPG)
-         {
-            if (EpgDbAcqProcessPackets() == FALSE)
-            {
-               // Notification from acquisition about channel change: should not be reached, because
-               // the video and vbi devices are kepty busy, so no external channel changes can occur
-               fatal0("EpgScan-ProcessPackets: uncontrolled channel change detected");
-            }
-            EpgDbProcessQueueByType(&scanCtl.pDbContext, &scanCtl.dbQueue, BLOCK_TYPE_BI, &epgScanCb);
-            EpgDbProcessQueueByType(&scanCtl.pDbContext, &scanCtl.dbQueue, BLOCK_TYPE_AI, &epgScanCb);
-
-            // accept OI block #0 because its message is displayed in the prov selection dialog
-            if (EpgDbContextGetCni(scanCtl.pDbContext) != 0)
-               EpgDbProcessQueueByType(&scanCtl.pDbContext, &scanCtl.dbQueue, BLOCK_TYPE_OI, &epgScanCb);
-         }
-      }
-
-      if (stopped == FALSE)
-      {
-         pageNo = EpgDbAcqGetMipPageNo();
-         if ((pageNo != EPG_ILLEGAL_PAGENO) && (pageNo != scanCtl.pDbContext->pageNo))
-         {  // found a different page number in MIP
-            dprintf2("EpgScan-ProcessPackets: non-default MIP page no for EPG: %03X (was %03X) -> restart acq\n", pageNo, scanCtl.pDbContext->pageNo);
-
-            scanCtl.pDbContext->pageNo = pageNo;
-            EpgDbAcqReset(scanCtl.pDbContext, &scanCtl.dbQueue, pageNo, EPG_ILLEGAL_APPID);
-         }
-      }
-      else
-      {
-         EpgScan_Stop();
-      }
-   }
-}
-
 // ----------------------------------------------------------------------------
 // EPG scan timer event handler - called every 250ms
 // 
@@ -296,31 +260,40 @@ uint EpgScan_EvHandler( void )
    const AI_BLOCK * pAi;
    const char * pName, * pCountry;
    time_t now = time(NULL);
-   uchar chanName[10], msgbuf[300];
-   ulong freq;
-   ulong ttxPkgCount, epgPkgCount, epgPageCount;
+   uchar chanName[10], msgbuf[300], dispText[PDC_TEXT_LEN + 1];
+   uint  freq;
+   uint32_t ttxPkgCount, epgPkgCount, epgPageCount;
    uint cni, dataPageCnt;
+   uint pageNo;
    time_t delay;
    uint rescheduleMs;
+   bool isTuner;
    bool niWait;
+   bool stopped;
 
+   // Process all available lines from VBI and check for BI and AI blocks
+   // (note: the check function must be called even if EPG acq is not yet enabled
+   // because it also handles channel changes and watches over the slave's life)
+   TtxDecode_CheckForPackets(&stopped);
+   if ( stopped || (scanCtl.pDbContext == NULL) )
+      EpgScan_Stop();
+
+   dispText[0] = 0;
    rescheduleMs = 0;
    if (scanCtl.state != SCAN_STATE_OFF)
    {
       if (scanCtl.state == SCAN_STATE_RESET)
       {  // reset state again 50ms after channel change
-         // make sure all data of the previous channel is removed from the ring buffer
-         EpgDbAcqInitScan();
          scanCtl.state = SCAN_STATE_WAIT_SIGNAL;
          dprintf1("WAIT_SIGNAL channel %d\n", scanCtl.channel);
       }
       else if (scanCtl.state == SCAN_STATE_WAIT_SIGNAL)
       {  // skip this channel if there's no stable signal
-         EpgDbAcqGetStatistics(&ttxPkgCount, &epgPkgCount, &epgPageCount);
+         TtxDecode_GetStatistics(&ttxPkgCount, &epgPkgCount, &epgPageCount);
          if ( scanCtl.doSlow || scanCtl.useXawtv || scanCtl.doRefresh ||
               BtDriver_IsVideoPresent() || (ttxPkgCount > 0) )
          {
-            dprintf2("WAIT for data on channel %d (%ld ttx pkgs)\n", scanCtl.channel, ttxPkgCount);
+            dprintf2("WAIT for data on channel %d (%d ttx pkgs)\n", scanCtl.channel, ttxPkgCount);
             if (BtDriver_IsVideoPresent())
                scanCtl.signalFound += 1;
             scanCtl.state = SCAN_STATE_WAIT_ANY;
@@ -333,10 +306,37 @@ uint EpgScan_EvHandler( void )
       }
       else
       {
-         EpgDbAcqGetScanResults(&cni, &niWait, &dataPageCnt);
-
-         if (scanCtl.state == SCAN_STATE_WAIT_EPG)
+         if ( (scanCtl.state == SCAN_STATE_WAIT_NI_OR_EPG) ||
+              (scanCtl.state == SCAN_STATE_WAIT_EPG) )
          {
+            pageNo = TtxDecode_GetMipPageNo();
+            if ((pageNo != EPG_ILLEGAL_PAGENO) && (pageNo != scanCtl.pDbContext->pageNo))
+            {  // found a different page number in MIP
+               dprintf2("EpgScan-ProcessPackets: non-default MIP page no for EPG: %03X (was %03X) -> restart acq\n", pageNo, scanCtl.pDbContext->pageNo);
+               scanCtl.MsgCallback("Found non-default EPG ttx page in MIP - restarting", TRUE);
+
+               scanCtl.pDbContext->pageNo = pageNo;
+               EpgStreamClear();
+               EpgStreamInit(&scanCtl.dbQueue, TRUE, EPG_DEFAULT_APPID, pageNo);
+               TtxDecode_StartEpgAcq(pageNo, TRUE);
+
+               // set effective start time to "now"
+               scanCtl.extraWait = now - scanCtl.startTime;
+            }
+
+            if (EpgStreamProcessPackets() == FALSE)
+            {
+               // Notification from acquisition about channel change: should not be reached, because
+               // the video and vbi devices are kepty busy, so no external channel changes can occur
+               fatal0("EpgScan-ProcessPackets: uncontrolled channel change detected");
+            }
+            EpgDbProcessQueueByType(&scanCtl.pDbContext, &scanCtl.dbQueue, BLOCK_TYPE_BI, &epgScanCb);
+            EpgDbProcessQueueByType(&scanCtl.pDbContext, &scanCtl.dbQueue, BLOCK_TYPE_AI, &epgScanCb);
+
+            // accept OI block #0 because its message is displayed in the prov selection dialog
+            if (EpgDbContextGetCni(scanCtl.pDbContext) != 0)
+               EpgDbProcessQueueByType(&scanCtl.pDbContext, &scanCtl.dbQueue, BLOCK_TYPE_OI, &epgScanCb);
+
             EpgDbLockDatabase(scanCtl.pDbContext, TRUE);
             pAi = EpgDbGetAi(scanCtl.pDbContext);
             if (pAi != NULL)
@@ -347,6 +347,9 @@ uint EpgScan_EvHandler( void )
             }
             EpgDbLockDatabase(scanCtl.pDbContext, FALSE);
          }
+
+         TtxDecode_GetScanResults(&cni, &niWait, &dataPageCnt, dispText, sizeof(dispText));
+
          if ((cni != 0) && ((scanCtl.state <= SCAN_STATE_WAIT_NI) || (scanCtl.state == SCAN_STATE_WAIT_NI_OR_EPG)))
          {
             dprintf2("Found VPS/PDC/NI 0x%04X on channel %d\n", cni, scanCtl.channel);
@@ -357,16 +360,23 @@ uint EpgScan_EvHandler( void )
                TvChannels_GetName(scanCtl.channel, chanName, sizeof(chanName));
             else
                sprintf(chanName, "#%d", scanCtl.channel);
-            // append country if available
-            sprintf(msgbuf, "Channel %s: CNI 0x%04X %s", chanName, cni, ((pName != NULL) ? pName : ""));
-            if ((pCountry != NULL) && (strstr(pName, pCountry) == NULL))
-               sprintf(msgbuf + strlen(msgbuf), " (%s)", pCountry);
-            scanCtl.MsgCallback(msgbuf);
+            if (pName != NULL)
+            {
+               sprintf(msgbuf, "Channel %s: CNI 0x%04X %s", chanName, cni, pName);
+               // append country if available
+               if ((pCountry != NULL) && (strstr(pName, pCountry) == NULL))
+                  sprintf(msgbuf + strlen(msgbuf), " (%s)", pCountry);
+            }
+            else if (dispText[0] != 0)
+               sprintf(msgbuf, "Channel %s: CNI 0x%04X \"%s\"", chanName, cni, dispText);
+            else
+               sprintf(msgbuf, "Channel %s: CNI 0x%04X", chanName, cni);
+            scanCtl.MsgCallback(msgbuf, FALSE);
 
             // check if there's already a database for this provider
             // note: must do a complete open, not only a peek, to make sure the db is intact
-            pDbContext = EpgContextCtl_Open(cni, CTX_FAIL_RET_NULL, CTX_RELOAD_ERR_NONE);
-            if (pDbContext != NULL)
+            if ( (scanCtl.doRefresh == FALSE) &&
+                 ((pDbContext = EpgContextCtl_Open(cni, CTX_FAIL_RET_NULL, CTX_RELOAD_ERR_NONE)) != NULL) )
             {  // database available
                EpgDbLockDatabase(pDbContext, TRUE);
                pAi = EpgDbGetAi(pDbContext);
@@ -374,14 +384,12 @@ uint EpgScan_EvHandler( void )
                {
                   sprintf(msgbuf, "Provider '%s' is already known - skipping.",
                                   AI_GET_NETWOP_NAME(pAi, pAi->thisNetwop));
-                  scanCtl.MsgCallback(msgbuf);
-                  // in refresh mode count this provider as done (used in the summary)
-                  if (scanCtl.doRefresh)
-                     scanCtl.newProvCount += 1;
+                  scanCtl.MsgCallback(msgbuf, TRUE);
                }
                if (pDbContext->tunerFreq != scanCtl.pDbContext->tunerFreq)
                {
-                  scanCtl.MsgCallback("storing provider's tuner frequency");
+                  debug3("EpgScan-EvHandler: CNI 0x%04X: updating tuner freq: 0x%x -> 0x%x", cni, pDbContext->tunerFreq, scanCtl.pDbContext->tunerFreq);
+                  scanCtl.MsgCallback("storing provider's tuner frequency", TRUE);
                   EpgContextCtl_UpdateFreq(cni, scanCtl.pDbContext->tunerFreq);
                }
                UiControlMsg_NewProvFreq(cni, scanCtl.pDbContext->tunerFreq);
@@ -393,16 +401,19 @@ uint EpgScan_EvHandler( void )
             }
             else
             {  // no database for this CNI yet: check if it's a known provider
-               if (CniIsKnownProvider(cni))
+               if (CniIsKnownProvider(cni) || scanCtl.doRefresh)
                {
                   // known provider -> wait for BI/AI
                   if (scanCtl.state <= SCAN_STATE_WAIT_NI)
-                     scanCtl.MsgCallback("checking for EPG transmission...");
+                     scanCtl.MsgCallback("checking for EPG transmission...", FALSE);
                   scanCtl.state = SCAN_STATE_WAIT_EPG;
                }
                else
                {  // CNI not known as provider -> keep checking for data page
-                  scanCtl.state = SCAN_STATE_WAIT_DATA;
+                  if (scanCtl.state <= SCAN_STATE_WAIT_NI)
+                     scanCtl.state = SCAN_STATE_WAIT_DATA;
+                  else if (scanCtl.state == SCAN_STATE_WAIT_NI_OR_EPG)
+                     scanCtl.state = SCAN_STATE_WAIT_EPG;
                }
             }
          }
@@ -421,8 +432,8 @@ uint EpgScan_EvHandler( void )
             {  // CNI already reported to the user -> just print the reason we start waiting
                sprintf(msgbuf, "Found some kind of data transmission on %d TTX page%s", dataPageCnt, ((dataPageCnt == 1) ? "" : "s"));
             }
-            scanCtl.MsgCallback(msgbuf);
-            scanCtl.MsgCallback("checking for EPG transmission...");
+            scanCtl.MsgCallback(msgbuf, FALSE);
+            scanCtl.MsgCallback("checking for EPG transmission...", FALSE);
 
             if (scanCtl.state <= SCAN_STATE_WAIT_NI)
                scanCtl.state = SCAN_STATE_WAIT_NI_OR_EPG;
@@ -441,7 +452,7 @@ uint EpgScan_EvHandler( void )
       {
          case SCAN_STATE_WAIT_SIGNAL:    delay =  2; break;
          case SCAN_STATE_WAIT_ANY:       delay =  2; break;
-         case SCAN_STATE_WAIT_NI:        delay =  4; break;
+         case SCAN_STATE_WAIT_NI:        delay =  6; break;
          case SCAN_STATE_WAIT_DATA:      delay =  2; break;
          case SCAN_STATE_WAIT_NI_OR_EPG: delay = 45; break;
          case SCAN_STATE_WAIT_EPG:       delay = 45; break;
@@ -452,7 +463,8 @@ uint EpgScan_EvHandler( void )
       if (scanCtl.doSlow)
          delay *= 2;
 
-      if ( (scanCtl.state == SCAN_STATE_DONE) || ((now - scanCtl.startTime) >= delay) )
+      if ( (scanCtl.state == SCAN_STATE_DONE) ||
+           ((now - scanCtl.startTime - scanCtl.extraWait) >= delay) )
       {  // max wait exceeded -> next channel
 
          // print a message why the search is aborted
@@ -462,13 +474,13 @@ uint EpgScan_EvHandler( void )
             if (scanCtl.foundBi)
             {
                if (EpgDbQueue_GetBlockCount(&scanCtl.dbQueue) > 0)
-                  scanCtl.MsgCallback("Nextview transmission found, but no application info received");
+                  scanCtl.MsgCallback("Nextview transmission found, but no application info received", TRUE);
                else if (scanCtl.foundBi)
-                  scanCtl.MsgCallback("Nextview announced in \"Bundle Inventory\", but no data received");
-               scanCtl.MsgCallback("Giving up, try again later.");
+                  scanCtl.MsgCallback("Nextview announced in \"Bundle Inventory\", but no data received", TRUE);
+               scanCtl.MsgCallback("Giving up, try again later.", TRUE);
             }
             else
-               scanCtl.MsgCallback("No EPG transmission found on this channel - giving up.");
+               scanCtl.MsgCallback("No EPG transmission found on this channel - giving up.", TRUE);
          }
          else if ( (scanCtl.useXawtv || scanCtl.doRefresh) &&
                    (scanCtl.state <= SCAN_STATE_WAIT_NI) )
@@ -477,12 +489,15 @@ uint EpgScan_EvHandler( void )
                TvChannels_GetName(scanCtl.channel, chanName, sizeof(chanName));
             else
                sprintf(chanName, "#%d", scanCtl.channel);
-            sprintf(msgbuf, "Channel %s: no CNI received - skipping", chanName);
-            scanCtl.MsgCallback(msgbuf);
+            if (dispText[0] != 0)
+               sprintf(msgbuf, "Channel %s: CNI 0x0000 \"%s\"", chanName, dispText);
+            else
+               sprintf(msgbuf, "Channel %s: no CNI received - skipping", chanName);
+            scanCtl.MsgCallback(msgbuf, FALSE);
          }
 
          if (scanCtl.doRefresh && (scanCtl.state != SCAN_STATE_DONE))
-         {  // refresh failed on one providr
+         {  // refresh failed on one provider (aborted by timeout)
             // load the database to check if it's defective
             bool isDefective = TRUE;
             pDbContext = EpgContextCtl_Open(scanCtl.provCniTab[scanCtl.channel - 1], CTX_FAIL_RET_NULL, CTX_RELOAD_ERR_NONE);
@@ -503,13 +518,13 @@ uint EpgScan_EvHandler( void )
                sprintf(msgbuf, "Defective database 0x%04X remains to be repaired.", scanCtl.provCniTab[scanCtl.channel - 1]);
                scanCtl.badProvCount += 1;
             }
-            scanCtl.MsgCallback(msgbuf);
+            scanCtl.MsgCallback(msgbuf, TRUE);
+            scanCtl.DelCallback(scanCtl.provCniTab[scanCtl.channel - 1]);
          }
 
          if ( EpgScan_NextChannel(&freq) )
          {
-            EpgDbAcqNotifyChannelChange();
-            if ( BtDriver_TuneChannel(freq, TRUE) )
+            if ( BtDriver_TuneChannel(scanCtl.inputSrc, freq, TRUE, &isTuner) )
             {
                // automatically dump db if provider was found, then free resources
                assert((scanCtl.pDbContext->pAiBlock == NULL) || (scanCtl.pDbContext->tunerFreq != 0));
@@ -517,13 +532,16 @@ uint EpgScan_EvHandler( void )
 
                scanCtl.pDbContext = EpgContextCtl_OpenDummy();
                scanCtl.pDbContext->tunerFreq = freq;
+               scanCtl.pDbContext->pageNo    = EPG_DEFAULT_PAGENO;
 
-               // Note: Reset initializes the app-ID and ttx page no, hence it must be called after the db switch
-               EpgDbAcqReset(scanCtl.pDbContext, &scanCtl.dbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
-               EpgDbAcqInitScan();
+               EpgStreamClear();
+               EpgStreamInit(&scanCtl.dbQueue, TRUE, EPG_DEFAULT_APPID, EPG_DEFAULT_PAGENO);
+
+               TtxDecode_StartEpgAcq(EPG_DEFAULT_PAGENO, TRUE);
 
                dprintf1("RESET channel %d\n", scanCtl.channel);
                scanCtl.startTime = now;
+               scanCtl.extraWait = 0;
                scanCtl.state = SCAN_STATE_RESET;
                scanCtl.foundBi = FALSE;
 
@@ -531,39 +549,47 @@ uint EpgScan_EvHandler( void )
             }
             else
             {
+               #ifndef WIN32
+               const char * pErrStr = BtDriver_GetLastError();
+               sprintf(msgbuf, "\nFatal error: channel change failed:\n%.200s.\n",
+                       ((pErrStr != NULL) ? pErrStr : "(unknown - internal error, please report)"));
+               scanCtl.MsgCallback(msgbuf, TRUE);
+               #else
+               scanCtl.MsgCallback("channel change failed (driver error) - abort.", TRUE);
+               #endif
+
                EpgScan_Stop();
-               scanCtl.MsgCallback("channel change failed - abort.");
             }
          }
          else
          {
             dprintf0("EPG scan finished\n");
             EpgScan_Stop();
-            scanCtl.MsgCallback("EPG scan finished.");
+            scanCtl.MsgCallback("EPG scan finished.", FALSE);
             if (scanCtl.signalFound == 0)
             {
                scanCtl.MsgCallback("No signal found on any channels!\n"
                                    "Please check your settings in the\n"
                                    "TV card input popup (Configure menu)\n"
-                                   "or check your antenna cable.");
+                                   "or check your antenna cable.", FALSE);
             }
             else if (scanCtl.doRefresh == FALSE)
             {
                if (scanCtl.newProvCount > 0)
                {
-                  sprintf(msgbuf, "Found %d new Nextview provider%s.", scanCtl.newProvCount, ((scanCtl.newProvCount == 1) ? "" : "s"));
-                  scanCtl.MsgCallback(msgbuf);
+                  sprintf(msgbuf, "\nFound %d new Nextview provider%s.", scanCtl.newProvCount, ((scanCtl.newProvCount == 1) ? "" : "s"));
+                  scanCtl.MsgCallback(msgbuf, FALSE);
                }
                else
                {
                   if (EpgContextCtl_GetProvCount() > 0)
-                     scanCtl.MsgCallback("\nSorry, no new providers found.");
+                     scanCtl.MsgCallback("\nSorry, no new providers found.", FALSE);
                   else
-                     scanCtl.MsgCallback("\nSorry, no providers found.");
+                     scanCtl.MsgCallback("\nSorry, no providers found.", FALSE);
 
                   scanCtl.MsgCallback("Please try again at a different time of day, as not all\n"
                                       "providers transmit at all hours (due to channel sharing\n"
-                                      "or technical difficulties). Press \"Help\" for more info.");
+                                      "or technical difficulties). Press \"Help\" for more info.", FALSE);
                }
             }
             else
@@ -575,15 +601,15 @@ uint EpgScan_EvHandler( void )
                                scanCtl.newProvCount, ((scanCtl.newProvCount == 1) ? "" : "s"),
                                scanCtl.provFreqCount - scanCtl.newProvCount, ((scanCtl.provFreqCount - scanCtl.newProvCount == 1) ? "" : "s"),
                                scanCtl.badProvCount, ((scanCtl.badProvCount == 1) ? "is" : "are"));
-               scanCtl.MsgCallback(msgbuf);
+               scanCtl.MsgCallback(msgbuf, FALSE);
                if (scanCtl.badProvCount > 0)
-                  scanCtl.MsgCallback("\nPlease remove remaining defective databases manually.");
+                  scanCtl.MsgCallback("\nYou may remove remaining defective databases\nwith the above buttons now.", FALSE);
             }
          }
       }
       else
       {  // continue scan on current channel
-         dprintf2("Continue waiting... waited %d of max. %d secs\n", (int)(now - scanCtl.startTime), (int)delay);
+         dprintf3("Continue waiting... state %d waited %d of max. %d secs\n", scanCtl.state, (int)(now - scanCtl.startTime - scanCtl.extraWait), (int)delay);
          if (scanCtl.state == SCAN_STATE_WAIT_SIGNAL)
             rescheduleMs = 100;
          else
@@ -611,15 +637,16 @@ uint EpgScan_EvHandler( void )
 //   doRefresh: work only on freqs of already known providers
 //
 EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv, bool doRefresh,
-                                    uint * cniTab, ulong *freqTab, uint freqCount,
-                                    uint * pRescheduleMs, void (* MsgCallback)(const char * pMsg) )
+                                    uint *cniTab, uint *freqTab, uint freqCount, uint * pRescheduleMs,
+                                    EPGSCAN_MSGCB * pMsgCallback, EPGSCAN_DELCB * pProvDelCallback  )
 {
-   ulong freq;
+   uint  freq;
    bool  isTuner;
    EPGSCAN_START_RESULT result;
-   uchar chanName[10], msgbuf[80];
+   uchar chanName[10], msgbuf[300];
    uint rescheduleMs;
 
+   scanCtl.inputSrc      = inputSource;
    scanCtl.doRefresh     = doRefresh;
    scanCtl.doSlow        = doSlow;
    scanCtl.useXawtv      = useXawtv;
@@ -627,7 +654,8 @@ EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv,
    scanCtl.provFreqCount = freqCount;
    scanCtl.provFreqTab   = freqTab;
    scanCtl.provCniTab    = cniTab;
-   scanCtl.MsgCallback   = MsgCallback;
+   scanCtl.MsgCallback   = pMsgCallback;
+   scanCtl.DelCallback   = pProvDelCallback;
    scanCtl.pDbContext    = NULL;
    result = EPGSCAN_OK;
 
@@ -635,10 +663,10 @@ EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv,
 
    if ( ((useXawtv || doRefresh) && ((freqTab == NULL) || (freqCount == 0))) ||
         (doRefresh && (cniTab == NULL)) ||
-        (MsgCallback == NULL) ||
+        (pMsgCallback == NULL) || (pProvDelCallback == NULL) ||
         (pRescheduleMs == NULL) )
    {
-      fatal7("EpgScan-Start: illegal NULL ptr param (doRefresh=%d, useXawtv=%d): freqCount=%d, cniTab=%ld, freqTab=%ld, msgCb=%ld, resched=%ld", doRefresh, useXawtv, freqCount, (long)cniTab, (long)freqTab, (long)MsgCallback, (long)pRescheduleMs);
+      fatal8("EpgScan-Start: illegal NULL ptr param (doRefresh=%d, useXawtv=%d): freqCount=%d, cniTab=%ld, freqTab=%ld, msgCb=%ld, provDelCb=%ld resched=%ld", doRefresh, useXawtv, freqCount, (long)cniTab, (long)freqTab, (long)pMsgCallback, (long)pProvDelCallback, (long)pRescheduleMs);
       result = EPGSCAN_INTERNAL;
    }
    else
@@ -646,11 +674,17 @@ EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv,
       scanCtl.pDbContext = EpgContextCtl_OpenDummy();
       if (scanCtl.acqWasEnabled == FALSE)
       {
-         EpgDbAcqInit();
          EpgDbQueue_Init(&scanCtl.dbQueue);
-         EpgDbAcqStart(scanCtl.pDbContext, &scanCtl.dbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
          if (BtDriver_StartAcq() == FALSE)
-            result = EPGSCAN_ACCESS_DEV_VIDEO;
+         {
+            #ifndef WIN32
+            const char * pErrStr = BtDriver_GetLastError();
+            sprintf(msgbuf, "\nFailed to open the VBI (i.e. teletext) device:\n%.200s.\n",
+                    ((pErrStr != NULL) ? pErrStr : "(unknown - internal error, please report)"));
+            scanCtl.MsgCallback(msgbuf, FALSE);
+            #endif
+            result = EPGSCAN_ACCESS_DEV_VBI;
+         }
       }
       else
          EpgAcqCtl_Suspend(TRUE);
@@ -658,24 +692,25 @@ EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv,
 
    if (result == EPGSCAN_OK)
    {
-      EpgDbAcqNotifyChannelChange();
-      if ( BtDriver_SetInputSource(inputSource, TRUE, &isTuner) )
+      scanCtl.channelIdx = 0;
+      scanCtl.channel = 0;
+      if (EpgScan_NextChannel(&freq))
       {
-         if (isTuner)
+         if ( BtDriver_TuneChannel(scanCtl.inputSrc, freq, TRUE, &isTuner) )
          {
-            scanCtl.channelIdx = 0;
-            scanCtl.channel = 0;
-            if ( EpgScan_NextChannel(&freq) &&
-                 BtDriver_TuneChannel(freq, TRUE) )
+            if (isTuner)
             {
                scanCtl.pDbContext->tunerFreq = freq;
+               scanCtl.pDbContext->pageNo    = EPG_DEFAULT_PAGENO;
 
-               EpgDbAcqReset(scanCtl.pDbContext, &scanCtl.dbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
-               EpgDbAcqInitScan();
+               EpgStreamInit(&scanCtl.dbQueue, TRUE, EPG_DEFAULT_APPID, EPG_DEFAULT_PAGENO);
+
+               TtxDecode_StartEpgAcq(EPG_DEFAULT_PAGENO, TRUE);
 
                dprintf1("RESET channel %d\n", scanCtl.channel);
                scanCtl.state = SCAN_STATE_RESET;
                scanCtl.startTime = time(NULL);
+               scanCtl.extraWait = 0;
                scanCtl.foundBi = FALSE;
                rescheduleMs = 50;
                scanCtl.signalFound = 0;
@@ -689,16 +724,24 @@ EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv,
                }
                else
                   sprintf(msgbuf, "Starting scan on %d known channels", scanCtl.provFreqCount);
-               scanCtl.MsgCallback(msgbuf);
+               scanCtl.MsgCallback(msgbuf, FALSE);
             }
             else
-               result = EPGSCAN_ACCESS_DEV_VIDEO;
+               result = EPGSCAN_NO_TUNER;
          }
          else
-            result = EPGSCAN_NO_TUNER;
+         {
+            #ifndef WIN32
+            const char * pErrStr = BtDriver_GetLastError();
+            sprintf(msgbuf, "\nFailed to open video input channel:\n%.200s.\n",
+                            ((pErrStr != NULL) ? pErrStr : "(unknown - internal error, please report)"));
+            scanCtl.MsgCallback(msgbuf, FALSE);
+            #endif
+            result = EPGSCAN_ACCESS_DEV_VIDEO;
+         }
       }
       else
-         result = EPGSCAN_ACCESS_DEV_VIDEO;
+         result = EPGSCAN_INTERNAL;
 
       if ((result != EPGSCAN_OK) && (scanCtl.acqWasEnabled == FALSE))
       {
@@ -716,7 +759,9 @@ EPGSCAN_START_RESULT EpgScan_Start( int inputSource, bool doSlow, bool useXawtv,
          scanCtl.pDbContext = NULL;
 
          if (scanCtl.acqWasEnabled == FALSE)
-            EpgDbAcqStop();
+         {
+            TtxDecode_StopAcq();
+         }
          else if (pAcqDbContext == NULL)
             EpgAcqCtl_Suspend(FALSE);
       }
@@ -749,8 +794,7 @@ void EpgScan_Stop( void )
 {
    if (scanCtl.state != SCAN_STATE_OFF)
    {
-      // do a reset to clear the db queue
-      EpgDbAcqReset(scanCtl.pDbContext, &scanCtl.dbQueue, EPG_ILLEGAL_PAGENO, EPG_ILLEGAL_APPID);
+      EpgStreamClear();
 
       if (scanCtl.provFreqTab != NULL)
       {
@@ -773,8 +817,9 @@ void EpgScan_Stop( void )
       BtDriver_CloseDevice();
       if (scanCtl.acqWasEnabled == FALSE)
       {
+         TtxDecode_StopAcq();
          BtDriver_StopAcq();
-         EpgDbAcqStop();
+         EpgStreamClear();
       }
       else
          EpgAcqCtl_Suspend(FALSE);
