@@ -20,7 +20,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgblock.c,v 1.41 2001/12/25 19:43:34 tom Exp tom $
+ *  $Id: epgblock.c,v 1.49 2003/03/09 19:29:53 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGDB
@@ -32,6 +32,7 @@
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
 #include "epgdb/epgblock.h"
+#include "epgdb/epgswap.h"
 
 
 static uchar netwopAlphabets[MAX_NETWOP_COUNT];
@@ -40,6 +41,13 @@ static uchar providerAlphabet;
 static time_t unixTimeBase1982;         // 1.1.1982 in UNIX time format
 #define JULIAN_DATE_1982  (45000-30)    // 1.1.1982 in Julian date format
 
+// when cross-compiling for WIN32 on Linux "timezone" is undefined
+#if !defined(__NetBSD__) && !defined(__FreeBSD__)
+# if defined(WIN32) && !defined(timezone)
+#  define timezone _timezone
+# endif
+#endif
+
 
 // ----------------------------------------------------------------------------
 // Save the alphabets of all networks for string decoding
@@ -100,10 +108,10 @@ sint EpgLtoGet( time_t when )
    struct tm *pTm;
    sint lto;
 
-   #ifndef __NetBSD__
+   #if !defined(__NetBSD__) && !defined(__FreeBSD__)
    pTm = localtime(&when);
    lto = 60*60 * pTm->tm_isdst - timezone;
-   #else
+   #else  // BSD
    pTm = gmtime(&when);
    pTm->tm_isdst = -1;
    lto = when - mktime(pTm);
@@ -136,10 +144,10 @@ void EpgLtoInit( void )
    unixTimeBase1982 = mktime(&tm);
 
    // undo the local->UTC conversion that mktime unwantedly always does
-   #ifndef __NetBSD__
+   #if !defined(__NetBSD__) && !defined(__FreeBSD__)
    pTm = localtime(&unixTimeBase1982);
    unixTimeBase1982 += 60*60 * pTm->tm_isdst - timezone;
-   #else
+   #else  // BSD
    pTm = gmtime(&unixTimeBase1982);
    pTm->tm_isdst = -1;
    unixTimeBase1982 += (unixTimeBase1982 - mktime(pTm));
@@ -346,7 +354,7 @@ static const char g2map_latin2[] =
 // ----------------------------------------------------------------------------
 // Return a de-nationalized character from the G0 set
 // - the G0 charset is basically ASCII, but at a few places the ASCII chars
-//   are replace by special chars, depending on the country
+//   are replaced by special chars, depending on the country
 // - in the following table -1 stands for ASCII chars, the other values
 //   are to be taken as index in the national options table
 // - note: declared as signed char to allow use of negative values; ANSI-C
@@ -385,9 +393,9 @@ static uchar GetG0Char(uchar val, uchar alphabeth)
 // ----------------------------------------------------------------------------
 // Apply escape sequence to a string
 // - only language-specific escapes and newlines are evaluated
-// - at the same time white space is compressed
-// - newline (explicit and implicit) is replaced by blank;
-//   line breaks are inserted at time of output, depending on window width
+// - at the same time white space (multiple blanks etc.) is compressed
+// - newline (explicit and implicit) is also replaced by blank, except for two
+//   subsequent newlines or a completely empty line (indicates paragraph break)
 //
 static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEscapes, uchar escCount, uchar netwop)
 {
@@ -395,6 +403,7 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
    uint  escIdx, nextEsc;
    uint  i, linePos, strLen;
    bool  lastWhite;
+   uchar newLine;
    uchar data, alphabet;
 
    // get default alphabet for the actual netwop
@@ -411,7 +420,9 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
    {
       po = pout;
       lastWhite = TRUE;  //suppress blank at start of line
+      newLine = 0;
       linePos = 0;
+
       escIdx = 0;
       if (escCount > 0)
          nextEsc = pEscapes[0] | ((pEscapes[1] & 0x03) << 8);
@@ -425,7 +436,7 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
          else
             *po = GetG0Char(pText[i], alphabet);
       
-         if (i == nextEsc)
+         while (i == nextEsc)
          {
             data = pEscapes[2];
             switch (pEscapes[1] >> 2)
@@ -446,23 +457,39 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
                      *po = ' ';
                   break;
                case 0x0A:  // CR/NL
-                  // the newline is inserted in front of the current character
-                  if (!lastWhite)
+                  if (newLine == 1)
+                  {  // current line empty -> insert paragraph break into output
+                     if (lastWhite && (po > pout))
+                     {  // overwrite blank at the end of the line
+                        *(po - 1) = '\n';
+                     }
+                     else if (strLen > 1)
+                     {
+                        po[1] = *po;
+                        *(po++) = '\n';
+                        strLen -= 1;
+                     }
+                     else
+                        debug0("Apply-Escapes: output string length exceeded");
+                     // set flag to avoid inserting multiple newlines (i.e. generating empty lines)
+                     newLine = 2;
+                  }
+                  else if (lastWhite == FALSE)
                   {
                      if (strLen > 1)
-                     {
+                     {  // the blank (instead of newline) is inserted in front of the current character
                         po[1] = *po;
                         *po++ = ' ';
                         strLen -= 1;
                      }
                      else
-                     {
                         debug0("Apply-Escapes: output string length exceeded");
-                        *po = ' ';
-                     }
                   }
                   lastWhite = TRUE;
                   linePos = 0;
+                  // set flag to indicate that the new line is still empty
+                  if (newLine == 0)
+                     newLine = 1;
                   break;
                case 0x0F:  // G2 character
                   if ((data >= 0x20) && (data < 0x80))
@@ -488,7 +515,7 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
                case 0x1f:
                {
                   uint c = (pEscapes[1]>>2)&0x0f;
-                  const char *pd = strchr(diacrits[c].g0, *po);
+                  const char *pd = strchr(diacrits[c].g0, pEscapes[2]);
                   if (pd != NULL)
                      *po = diacrits[c].latin1[pd - diacrits[c].g0];
                   break;
@@ -506,25 +533,43 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
 
          if (*po == ' ')
          {
-            if (!lastWhite)
+            if (lastWhite == FALSE)
             {
                po++;
                strLen -= 1;
+               lastWhite = TRUE;
             }
-            lastWhite = TRUE;
          }
          else
          {
             po++;
             strLen -= 1;
             lastWhite = FALSE;
+            // set flag to indicate this line is not empty
+            newLine = 0;
          }
          linePos += 1;
 
          if (linePos == 40)
-         {  // add blank after last character of line
-            if (lastWhite == FALSE)
-            {
+         {
+            if (newLine == 1)
+            {  // current line empty -> insert paragraph break into output
+               if (lastWhite && (po > pout))
+               {  // overwrite blank at the end of the line
+                  *(po - 1) = '\n';
+               }
+               else if (strLen > 0)
+               {
+                  *(po++) = '\n';
+                  strLen -= 1;
+               }
+               else
+                  debug0("Apply-Escapes: output string length exceeded");
+               // set flag to avoid inserting multiple newlines (i.e. generating empty lines)
+               newLine = 2;
+            }
+            else if (lastWhite == FALSE)
+            {  // add blank after last character of line
                if (strLen > 0)
                {
                   *(po++) = ' ';
@@ -533,12 +578,15 @@ static uchar * ApplyEscapes(const uchar *pText, uint textLen, const uchar *pEsca
                else
                   debug0("Apply-Escapes: output string length exceeded");
             }
+            if (newLine == 0)
+               newLine = 1;
             lastWhite = TRUE;
             linePos = 0;
          }
       }
 
       ifdebug1((strLen == 0) && (i < textLen), "Apply-Escapes: output string length exceeded: %d bytes remaining", textLen - i);
+      ifdebug6((escIdx < escCount), "Apply-Escapes: only %d of %d escapes applied: nextExc=%d textIdx=%d escType=0x%x data=0x%x", escIdx, escCount, nextEsc, i, pEscapes[1] >> 2, pEscapes[2]);
 
       if ( (po > pout) && (*(po - 1) == ' ') )
       {  // remove blank from end of line
@@ -592,8 +640,8 @@ static DESCRIPTOR * DecodeDescriptorLoop( const uchar *psd, uchar count, uchar b
                   //pDescriptors[i].eval = (psd[2] >> 2) | ((psd[3] & 0x3f)<<2);
                   break;
             }
+            psd   += (bitOff + 6 + 6 + 8) / 8;  // note: uses previous bitOff; psd must be set before bitOff is modified
             bitOff = (bitOff + 6 + 6 + 8) % 8;
-            psd   += (bitOff + 6 + 6 + 8) / 8;
          }
       }
    }
@@ -831,6 +879,32 @@ static bool EpgBlockCheckPi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all PI block struct elements > 1 byte for non-endian-matching databases
+// - the swap is done in-place
+//
+static bool EpgBlockSwapPi( EPGDB_BLOCK * pBlock )
+{
+   PI_BLOCK * pPi;
+
+   pPi = (PI_BLOCK *) &pBlock->blk.pi;
+
+   swap16(&pPi->block_no);
+   swap32(&pPi->start_time);
+   swap32(&pPi->stop_time);
+   swap32(&pPi->pil);
+   swap32(&pPi->series_code);
+   swap16(&pPi->feature_flags);
+   swap16(&pPi->background_ref);
+
+   swap16(&pPi->off_title);
+   swap16(&pPi->off_short_info);
+   swap16(&pPi->off_long_info);
+   swap16(&pPi->off_descriptors);
+
+   return TRUE;
+}
+
 // ----------------------------------------------------------------------------
 // Convert an AI block
 //
@@ -1000,6 +1074,50 @@ static bool EpgBlockCheckAi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all AI block struct elements > 1 byte for non-endian-matching databases
+// - called before consistancy check, hence used offsets and counters must be checked
+// - the swap is done in-place
+//
+static bool EpgBlockSwapAi( EPGDB_BLOCK * pBlock )
+{
+   AI_BLOCK  * pAi;
+   AI_NETWOP * pNetwop;
+   uchar netwop;
+   bool  result = FALSE;
+
+   pAi = (AI_BLOCK *) &pBlock->blk.ai;
+
+   swap16(&pAi->niCount);
+   swap16(&pAi->oiCount);
+   swap16(&pAi->miCount);
+   swap16(&pAi->niCountSwo);
+   swap16(&pAi->oiCountSwo);
+   swap16(&pAi->miCountSwo);
+
+   swap16(&pAi->off_serviceNameStr);
+   swap16(&pAi->off_netwops);
+
+   if ( (pAi->off_netwops == sizeof(AI_BLOCK)) &&
+        (pAi->netwopCount > 0) && (pAi->netwopCount <= MAX_NETWOP_COUNT) &&
+        (sizeof(AI_BLOCK) + (pAi->netwopCount * sizeof(AI_NETWOP)) <= pBlock->size) )
+   {
+      pNetwop = (AI_NETWOP *) AI_GET_NETWOPS(pAi);
+      for (netwop=0; netwop < pAi->netwopCount; netwop++, pNetwop++)
+      {
+         swap16(&pNetwop->cni);
+         swap16(&pNetwop->startNo);
+         swap16(&pNetwop->stopNo);
+         swap16(&pNetwop->stopNoSwo);
+         swap16(&pNetwop->addInfo);
+
+         swap16(&pNetwop->off_name);
+      }
+      result = TRUE;
+   }
+   return result;
+}
+
 // ----------------------------------------------------------------------------
 // Convert an OI block
 //
@@ -1123,6 +1241,25 @@ static bool EpgBlockCheckOi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all OI block struct elements > 1 byte for non-endian-matching databases
+// - the swap is done in-place
+//
+static bool EpgBlockSwapOi( EPGDB_BLOCK * pBlock )
+{
+   OI_BLOCK * pOi;
+
+   pOi = (OI_BLOCK *) &pBlock->blk.oi;
+
+   swap16(&pOi->block_no);
+
+   swap16(&pOi->off_header);
+   swap16(&pOi->off_message);
+   swap16(&pOi->off_descriptors);
+
+   return TRUE;
+}
+
 // ----------------------------------------------------------------------------
 // Convert a NI block
 //
@@ -1198,7 +1335,7 @@ EPGDB_BLOCK * EpgBlockConvertNi(const uchar *pCtrl, uint ctrlLen, uint strLen)
    // 1st step: sum up the length & compute the offsets of each element from the start
    blockLen = sizeof(NI_BLOCK);
    if (ni.no_events > 0)
-      ni.off_events = blockLen;
+      ni.off_events = ((blockLen + 3) & ~3);  // requires alignment for 32-bit element in struct
    else
       ni.off_events = 0;
    blockLen += ni.no_events * sizeof(EVENT_ATTRIB);
@@ -1319,6 +1456,61 @@ static bool EpgBlockCheckNi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all NI block struct elements > 1 byte for non-endian-matching databases
+// - called before consistancy check, hence used offsets and counters must be checked
+// - the swap is done in-place
+//
+static bool EpgBlockSwapNi( EPGDB_BLOCK * pBlock )
+{
+   NI_BLOCK       * pNi;
+   EVENT_ATTRIB   * pEv;
+   EV_ATTRIB_DATA * pEvUnit;
+   uint  ev_idx;
+   uint  att_idx;
+   bool  result = FALSE;
+
+   pNi = (NI_BLOCK *) &pBlock->blk.ni;
+
+   swap16(&pNi->block_no);
+   swap16(&pNi->msg_attrib);
+
+   swap16(&pNi->off_events);
+   swap16(&pNi->off_header);
+   swap16(&pNi->off_descriptors);
+
+   if (pNi->off_events != 0)
+   {
+      if ( (pNi->no_events <= NI_MAX_EVENT_COUNT) &&
+           (pNi->off_events + (pNi->no_events * sizeof(EVENT_ATTRIB)) <= pBlock->size) )
+      {
+         pEv = (EVENT_ATTRIB *) NI_GET_EVENTS(pNi);
+         result = TRUE;
+
+         for (ev_idx=0; ev_idx < pNi->no_events; ev_idx++, pEv++)
+         {
+            swap16(&pEv->next_id);
+            swap16(&pEv->off_evstr);
+
+            if (pEv->no_attribs <= NI_MAX_ATTRIB_COUNT)
+            {
+               pEvUnit = pEv->unit;
+               for (att_idx=0; att_idx < pEv->no_attribs; att_idx++, pEvUnit++)
+               {
+                  swap32(&pEvUnit->data);
+               }
+            }
+            else
+            {
+               result = FALSE;
+               break;
+            }
+         }
+      }
+   }
+   return result;
+}
+
 // ----------------------------------------------------------------------------
 // Convert a MI block
 //
@@ -1407,6 +1599,25 @@ static bool EpgBlockCheckMi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all MI block struct elements > 1 byte for non-endian-matching databases
+// - called before consistancy check, hence used offsets and counters must be checked
+// - the swap is done in-place
+//
+static bool EpgBlockSwapMi( EPGDB_BLOCK * pBlock )
+{
+   MI_BLOCK * pMi;
+
+   pMi = (MI_BLOCK *) &pBlock->blk.mi;
+
+   swap16(&pMi->block_no);
+
+   swap16(&pMi->off_message);
+   swap16(&pMi->off_descriptors);
+
+   return TRUE;
+}
+
 // ----------------------------------------------------------------------------
 // Convert a LI block
 //
@@ -1450,35 +1661,35 @@ EPGDB_BLOCK * EpgBlockConvertLi(const uchar *pCtrl, uint ctrlLen, uint strLen)
             ld[desc].lang_count = psd[1] >> 4;
             break;
       }
+      psd   += (bitOff + 6 + 4) / 8;  // note: uses previous bitOff; psd must be set before bitOff is modified
       bitOff = (bitOff + 6 + 4) % 8;
-      psd   += (bitOff + 6 + 4) / 8;
 
       for (lang=0; lang < ld[desc].lang_count; lang++)
       {
          switch (bitOff)
          {
             case 0:
-               ld[desc].lang[0][lang] = psd[0];
-               ld[desc].lang[1][lang] = psd[1];
-               ld[desc].lang[2][lang] = psd[2];
+               ld[desc].lang[lang][0] = psd[0];
+               ld[desc].lang[lang][1] = psd[1];
+               ld[desc].lang[lang][2] = psd[2];
                break;
 
             case 2:
-               ld[desc].lang[0][lang] = (psd[0] >> 2) | ((psd[1] & 0x03) << 6);
-               ld[desc].lang[1][lang] = (psd[1] >> 2) | ((psd[2] & 0x03) << 6);
-               ld[desc].lang[2][lang] = (psd[2] >> 2) | ((psd[3] & 0x03) << 6);
+               ld[desc].lang[lang][0] = (psd[0] >> 2) | ((psd[1] & 0x03) << 6);
+               ld[desc].lang[lang][1] = (psd[1] >> 2) | ((psd[2] & 0x03) << 6);
+               ld[desc].lang[lang][2] = (psd[2] >> 2) | ((psd[3] & 0x03) << 6);
                break;
 
             case 4:
-               ld[desc].lang[0][lang] = (psd[0] >> 4) | ((psd[1] & 0x0f) << 4);
-               ld[desc].lang[1][lang] = (psd[1] >> 4) | ((psd[2] & 0x0f) << 4);
-               ld[desc].lang[2][lang] = (psd[2] >> 4) | ((psd[3] & 0x0f) << 4);
+               ld[desc].lang[lang][0] = (psd[0] >> 4) | ((psd[1] & 0x0f) << 4);
+               ld[desc].lang[lang][1] = (psd[1] >> 4) | ((psd[2] & 0x0f) << 4);
+               ld[desc].lang[lang][2] = (psd[2] >> 4) | ((psd[3] & 0x0f) << 4);
                break;
 
             case 6:
-               ld[desc].lang[0][lang] = (psd[0] >> 6) | ((psd[1] & 0x3f) << 2);
-               ld[desc].lang[1][lang] = (psd[1] >> 6) | ((psd[2] & 0x3f) << 2);
-               ld[desc].lang[2][lang] = (psd[2] >> 6) | ((psd[3] & 0x3f) << 2);
+               ld[desc].lang[lang][0] = (psd[0] >> 6) | ((psd[1] & 0x3f) << 2);
+               ld[desc].lang[lang][1] = (psd[1] >> 6) | ((psd[2] & 0x3f) << 2);
+               ld[desc].lang[lang][2] = (psd[2] >> 6) | ((psd[3] & 0x3f) << 2);
                break;
          }
          psd += 3;
@@ -1549,6 +1760,23 @@ static bool EpgBlockCheckLi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all LI block struct elements > 1 byte for non-endian-matching databases
+// - called before consistancy check, hence used offsets and counters must be checked
+// - the swap is done in-place
+//
+static bool EpgBlockSwapLi( EPGDB_BLOCK * pBlock )
+{
+   LI_BLOCK * pLi;
+
+   pLi = (LI_BLOCK *) &pBlock->blk.li;
+
+   swap16(&pLi->block_no);
+   swap16(&pLi->off_desc);
+
+   return TRUE;
+}
+
 // ----------------------------------------------------------------------------
 // Convert a TI block
 //
@@ -1593,8 +1821,8 @@ EPGDB_BLOCK * EpgBlockConvertTi(const uchar *pCtrl, uint ctrlLen, uint strLen)
             std[desc].subt_count = psd[1] >> 4;
             break;
       }
+      psd   += (bitOff + 6 + 4) / 8;  // note: uses previous bitOff; psd must be set before bitOff is modified
       bitOff = (bitOff + 6 + 4) % 8;
-      psd   += (bitOff + 6 + 4) / 8;
 
       for (subt=0; subt < std[desc].subt_count; subt++)
       {
@@ -1639,7 +1867,7 @@ EPGDB_BLOCK * EpgBlockConvertTi(const uchar *pCtrl, uint ctrlLen, uint strLen)
          // decode TTX page reference according to ETS 300 707 chap. 11.3.2
          std[desc].subt[subt].page = (uint)buf[0] | ((uint)(buf[1] & 0x80) << 1) | ((uint)(buf[2] & 0xC0) << 3);
          std[desc].subt[subt].subpage = (uint)(buf[1] & 0x7F) |  ((uint)(buf[2] & 0x3F) << 8);
-         psd += 5;
+         psd += 6;
       }
    }
 
@@ -1707,6 +1935,54 @@ static bool EpgBlockCheckTi( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Swap all TI block struct elements > 1 byte for non-endian-matching databases
+// - called before consistancy check, hence used offsets and counters must be checked
+// - the swap is done in-place
+//
+static bool EpgBlockSwapTi( EPGDB_BLOCK * pBlock )
+{
+   TI_BLOCK * pTi;
+   TI_DESC  * pTd;
+   TI_SUBT  * pTs;
+   uint       desc;
+   uint       subt;
+   bool       result = FALSE;
+
+   pTi = (TI_BLOCK *) &pBlock->blk.ti;
+
+   swap16(&pTi->block_no);
+   swap16(&pTi->off_desc);
+
+   if (pTi->desc_no > 0)
+   {
+      if (pTi->off_desc == sizeof(TI_BLOCK))
+      {
+         pTd = (TI_DESC *) TI_GET_DESC(pTi);
+         result = TRUE;
+
+         for (desc=0; desc < pTi->desc_no; desc++, pTd++)
+         {
+            if (pTd->subt_count <= TI_MAX_LANG_COUNT)
+            {
+               pTs = pTd->subt;
+               for (subt = 0; subt < pTd->subt_count; subt++, pTs++)
+               {
+                  swap16(&pTs->page);
+                  swap16(&pTs->subpage);
+               }
+            }
+            else
+            {
+               result = FALSE;
+               break;
+            }
+         }
+      }
+   }
+   return result;
+}
+
 // ----------------------------------------------------------------------------
 // Convert a BI block
 //
@@ -1795,3 +2071,63 @@ bool EpgBlockCheckConsistancy( EPGDB_BLOCK * pBlock )
    return result;
 }
 
+// ---------------------------------------------------------------------------
+// Swap all EPG block struct elements > 1 byte for non-endian-matching databases
+//
+bool EpgBlockSwapEndian( EPGDB_BLOCK * pBlock )
+{
+   bool result;
+
+   if (pBlock != NULL)
+   {
+      swap32(&pBlock->size);
+      swap16(&pBlock->origBlkLen);
+      swap16(&pBlock->parityErrCnt);
+      swap32(&pBlock->updTimestamp);
+      swap32(&pBlock->acqTimestamp);
+      swap16(&pBlock->acqRepCount);
+      swap32(&pBlock->type);
+
+      switch (pBlock->type)
+      {
+         case BLOCK_TYPE_BI:
+            debug0("EpgBlock-SwapEndian: BI block encountered - should never be in the db");
+            result = FALSE;
+            break;
+         case BLOCK_TYPE_AI:
+            result = EpgBlockSwapAi(pBlock);
+            break;
+         case BLOCK_TYPE_NI:
+            result = EpgBlockSwapNi(pBlock);
+            break;
+         case BLOCK_TYPE_OI:
+            result = EpgBlockSwapOi(pBlock);
+            break;
+         case BLOCK_TYPE_MI:
+            result = EpgBlockSwapMi(pBlock);
+            break;
+         case BLOCK_TYPE_LI:
+            result = EpgBlockSwapLi(pBlock);
+            break;
+         case BLOCK_TYPE_TI:
+            result = EpgBlockSwapTi(pBlock);
+            result = TRUE;
+            break;
+         case BLOCK_TYPE_PI:
+            result = EpgBlockSwapPi(pBlock);
+            break;
+         default:
+            debug1("EpgBlock-SwapEndian: illegal block type %d", pBlock->type);
+            result = FALSE;
+            break;
+      }
+   }
+   else
+   {
+      debug0("EpgBlock-SwapEndian: illegal NULL ptr arg");
+      result = FALSE;
+   }
+
+   return result;
+}
+
