@@ -29,7 +29,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgdbsav.c,v 1.45 2002/02/13 21:02:28 tom Exp tom $
+ *  $Id: epgdbsav.c,v 1.51 2003/10/05 18:59:17 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGDB
@@ -56,8 +56,10 @@
 #include "epgctl/mytypes.h"
 #include "epgctl/epgversion.h"
 #include "epgctl/debug.h"
+#include "epgvbi/syserrmsg.h"
 
 #include "epgdb/epgblock.h"
+#include "epgdb/epgswap.h"
 #include "epgdb/epgdbmgmt.h"
 #include "epgdb/epgdbfil.h"
 #include "epgdb/epgdbif.h"
@@ -70,8 +72,12 @@ typedef const EPGDB_CONTEXT * CPDBC;
 typedef       EPGDB_CONTEXT * PDBC;
 
 
-const char * epgDbDirPath = NULL;
-const char * epgDemoDb    = NULL;
+static const char * epgDbDirPath = NULL;
+static const char * epgDemoDb    = NULL;
+
+// cache current time during database reload (optimization only)
+static time_t  epgDbReloadCurTime;
+static time_t  epgDbReloadExpireDelayPi = EPGDBSAV_DEFAULT_EXPIRE_TIME;
 
 
 // ---------------------------------------------------------------------------
@@ -181,8 +187,6 @@ bool EpgDbDump( PDBC dbc )
       fd = open(pFilename, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0666);
       if (fd >= 0)
       {
-         chmod(pFilename, 0666);
-
          result = EpgDbDumpHeader(dbc, fd) &&
                   EpgDbDumpAppendBlock(dbc->pAiBlock, fd);
 
@@ -347,9 +351,6 @@ static EPGDB_RELOAD_RESULT EpgDbReloadAddDefectPiBlock( PDBC dbc, EPGDB_BLOCK *
 {
    EPGDB_RELOAD_RESULT result;
 
-   // convert type to normal PI, since the "defect pi" type is known only inside this module
-   pBlock->type = BLOCK_TYPE_PI;
-
    if ( EpgBlockCheckConsistancy(pBlock) )
    {
       if (pBlock->version == ((pBlock->stream == 0) ? dbc->pAiBlock->blk.ai.version : dbc->pAiBlock->blk.ai.version_swo))
@@ -393,12 +394,21 @@ static EPGDB_RELOAD_RESULT EpgDbReloadAddPiBlock( PDBC dbc, EPGDB_BLOCK * pBlock
    if (EpgBlockCheckConsistancy(pBlock))
    {
       netwop = pBlock->blk.pi.netwop_no;
-      if ( EpgDbPiBlockNoValid(dbc, pBlock->blk.pi.block_no, netwop) )
+      pPrev  = pPrevNetwop[netwop];
+
+      if ( (netwop < dbc->pAiBlock->blk.ai.netwopCount) &&
+           (pBlock->blk.pi.start_time < pBlock->blk.pi.stop_time) )
       {
-         if (pBlock->blk.pi.start_time < pBlock->blk.pi.stop_time)
+         pBlock->acqRepCount = 0;
+         // check if the block still part of current stream, i.e. fits in AI block range
+         ((PI_BLOCK *)&pBlock->blk.pi)->block_no_in_ai =
+            EpgDbPiBlockNoValid(dbc, pBlock->blk.pi.block_no, netwop);
+
+         if ( (pBlock->blk.pi.block_no_in_ai) ||
+              ( (pBlock->blk.pi.stop_time <= epgDbReloadCurTime) &&
+                (pBlock->blk.pi.stop_time >= epgDbReloadCurTime - dbc->expireDelayPi) ))
          {
             //dprintf4("RELOAD PI ptr=%lx: netwop=%d, blockno=%d, start=%ld\n", (ulong)pBlock, netwop, pBlock->blk.pi.block_no, pBlock->blk.pi.start_time);
-            pBlock->acqRepCount = 0;
 
             if (dbc->pFirstPi == NULL)
             {  // there's no PI yet in the database -> just link with head pointers
@@ -417,8 +427,10 @@ static EPGDB_RELOAD_RESULT EpgDbReloadAddPiBlock( PDBC dbc, EPGDB_BLOCK * pBlock
             if ( ( (pBlock->blk.pi.start_time > dbc->pLastPi->blk.pi.start_time) ||
                    ( (pBlock->blk.pi.start_time == dbc->pLastPi->blk.pi.start_time) &&
                      (pBlock->blk.pi.netwop_no > dbc->pLastPi->blk.pi.netwop_no) )) &&
-                 ( ((pPrev = pPrevNetwop[netwop]) == NULL) ||
-                   ( EpgDbPiCmpBlockNoGt(dbc, pBlock->blk.pi.block_no, pPrev->blk.pi.block_no, netwop) &&
+                 ( (pPrev == NULL) ||
+                   ( ( (pBlock->blk.pi.block_no_in_ai == FALSE) ||
+                       (pPrev->blk.pi.block_no_in_ai == FALSE) ||
+                       (EpgDbPiCmpBlockNoGt(dbc, pBlock->blk.pi.block_no, pPrev->blk.pi.block_no, netwop)) ) &&
                      (pBlock->blk.pi.start_time >= pPrev->blk.pi.stop_time) )))
             {  // append the block after the last reloaded one
 
@@ -451,13 +463,13 @@ static EPGDB_RELOAD_RESULT EpgDbReloadAddPiBlock( PDBC dbc, EPGDB_BLOCK * pBlock
             //assert(EpgDbCheckChains());
          }
          else
-         {  // defective stop time value, PI cannot be added to regular list
-            //dprintf3("EXPIRED pi netwop=%d, blockno=%d start=%ld\n", pBlock->blk.pi.netwop_no, pBlock->blk.pi.block_no, pBlock->blk.pi.start_time);
-            result = EpgDbReloadAddDefectPiBlock(dbc, pBlock);
+         {  // block expired or removed from stream
+            xfree(pBlock);
          }
       }
       else
-      {  // invalid netwop or blockno
+      {  // invalid netwop or defective stop time: should never happen (defective blocks are filtered before saving)
+         //dprintf3("EXPIRED pi netwop=%d, blockno=%d start=%ld\n", pBlock->blk.pi.netwop_no, pBlock->blk.pi.block_no, pBlock->blk.pi.start_time);
          xfree(pBlock);
       }
       result = EPGDB_RELOAD_OK;
@@ -473,9 +485,10 @@ static EPGDB_RELOAD_RESULT EpgDbReloadAddPiBlock( PDBC dbc, EPGDB_BLOCK * pBlock
 // ---------------------------------------------------------------------------
 // Read and verify the file header
 //
-static EPGDB_RELOAD_RESULT EpgDbReloadHeader( uint cni, int fd, EPGDBSAV_HEADER * pHead )
+static EPGDB_RELOAD_RESULT EpgDbReloadHeader( uint cni, int fd, EPGDBSAV_HEADER * pHead, bool * pSwapEndian )
 {
    size_t size;
+   bool   swapEndian;
    EPGDB_RELOAD_RESULT result;
 
    size = read(fd, (uchar *)pHead, sizeof(*pHead));
@@ -483,13 +496,33 @@ static EPGDB_RELOAD_RESULT EpgDbReloadHeader( uint cni, int fd, EPGDBSAV_HEADER
    {
       if (strncmp(pHead->magic, MAGIC_STR, MAGIC_STR_LEN) == 0)
       {
-         if (pHead->endianMagic == ENDIAN_MAGIC)
+         if ( (pHead->endianMagic == ENDIAN_MAGIC) ||
+              (pHead->endianMagic == WRONG_ENDIAN) )
          {
+            if (pHead->endianMagic == WRONG_ENDIAN)
+            {
+               swapEndian = TRUE;
+
+               swap32(&pHead->compatVersion);
+               swap32(&pHead->swVersion);
+               swap32(&pHead->lastPiDate);
+               swap32(&pHead->firstPiDate);
+               swap32(&pHead->lastAiUpdate);
+               swap32(&pHead->cni);
+               swap32(&pHead->pageNo);
+               swap32(&pHead->tunerFreq);
+               swap32(&pHead->appId);
+            }
+            else
+               swapEndian = FALSE;
+
             if (pHead->compatVersion == DUMP_COMPAT)
             {
                if ((cni == RELOAD_ANY_CNI) || (pHead->cni == cni))
                {
                   result = EPGDB_RELOAD_OK;
+                  if (pSwapEndian != NULL)
+                     *pSwapEndian = swapEndian;
                }
                else
                {
@@ -499,11 +532,11 @@ static EPGDB_RELOAD_RESULT EpgDbReloadHeader( uint cni, int fd, EPGDBSAV_HEADER
             }
             else
             {
-               debug2("EpgDbReload-Header: reload db 0x%04X: incompatible version %06lx", cni, pHead->compatVersion);
+               debug2("EpgDbReload-Header: reload db 0x%04X: incompatible version %06x", cni, pHead->compatVersion);
                result = EPGDB_RELOAD_VERSION;
             }
          }
-         else if (pHead->endianMagic != WRONG_ENDIAN)
+         else
          {  // invalid endian code -> quick check if this is a header of a previous version
             OBSOLETE_EPGDBSAV_HEADER * pObsHead = (OBSOLETE_EPGDBSAV_HEADER *) pHead;
             if ( (pObsHead->dumpVersion >= OBSOLETE_DUMP_MIN_VERSION) &&
@@ -519,11 +552,6 @@ static EPGDB_RELOAD_RESULT EpgDbReloadHeader( uint cni, int fd, EPGDBSAV_HEADER
                result = EPGDB_RELOAD_CORRUPT;
             }
          }
-         else
-         {
-            debug2("EpgDbReload-Header: reload db 0x%04X: incompatible endian 0x%04x", cni, pHead->endianMagic);
-            result = EPGDB_RELOAD_ENDIAN;
-         }
       }
       else
       {
@@ -550,11 +578,13 @@ PDBC EpgDbReload( uint cni, EPGDB_RELOAD_RESULT * pResult )
    EPGDB_BLOCK *pPrevNetwop[MAX_NETWOP_COUNT];
    EPGDB_BLOCK *pPrevGeneric[BLOCK_TYPE_GENERIC_COUNT];
    EPGDB_BLOCK *pBlock;
-   uchar buffer[BLK_UNION_OFF];
-   size_t size;
-   int fd;
+   uchar   buffer[BLK_UNION_OFF];
+   uint32_t size;
+   size_t  readSize;
+   int     fd;
+   bool    swapEndian;
    uchar * pFilename;
-   time_t piStartOff;
+   time_t  piStartOff;
    BLOCK_TYPE type, lastType;
    EPGDB_RELOAD_RESULT result;
 
@@ -576,12 +606,13 @@ PDBC EpgDbReload( uint cni, EPGDB_RELOAD_RESULT * pResult )
    fd = open(pFilename, O_RDONLY|O_BINARY);
    if (fd >= 0)
    {
-      result = EpgDbReloadHeader(cni, fd, &head);
+      result = EpgDbReloadHeader(cni, fd, &head, &swapEndian);
       if (result == EPGDB_RELOAD_OK)
       {
          dbc->pageNo       = head.pageNo;
          dbc->tunerFreq    = head.tunerFreq;
          dbc->appId        = head.appId;
+         dbc->expireDelayPi = epgDbReloadExpireDelayPi;
 
          if (epgDemoDb != NULL)
          {  // demo mode: shift all PI in time to the current time and future
@@ -595,12 +626,15 @@ PDBC EpgDbReload( uint cni, EPGDB_RELOAD_RESULT * pResult )
          memset(pPrevNetwop, 0, sizeof(pPrevNetwop));
          memset(pPrevGeneric, 0, sizeof(pPrevGeneric));
          lastType = BLOCK_TYPE_INVALID;
+         epgDbReloadCurTime = time(NULL);
 
          // load the header of the next block from the file (including the block type)
          while ( (result == EPGDB_RELOAD_OK) &&
-                 ((size = read(fd, buffer, BLK_UNION_OFF)) == BLK_UNION_OFF) )
+                 (read(fd, buffer, BLK_UNION_OFF) == BLK_UNION_OFF) )
          {
             size = ((EPGDB_BLOCK *)buffer)->size;
+            if (swapEndian)
+               swap32(&size);
             // plausibility check for block size (avoid malloc failure, which reults in program abort)
             if (size <= EPGDBSAV_MAX_BLOCK_SIZE)
             {
@@ -608,13 +642,21 @@ PDBC EpgDbReload( uint cni, EPGDB_RELOAD_RESULT * pResult )
                pBlock = (EPGDB_BLOCK *) xmalloc(size + BLK_UNION_OFF);
                memcpy(pBlock, buffer, BLK_UNION_OFF);
                // load the rest of the block from the file
-               size = read(fd, (uchar *)pBlock + BLK_UNION_OFF, pBlock->size);
-               if (size == pBlock->size)
+               readSize = read(fd, (uchar *)pBlock + BLK_UNION_OFF, size);
+               if (readSize == size)
                {
                   // save the type in temp var in case the block gets freed
                   type = pBlock->type;
 
-                  switch (pBlock->type)
+                  if (swapEndian)
+                  {
+                     swap32(&type);
+                     if (type == BLOCK_TYPE_DEFECT_PI)
+                        pBlock->type = BLOCK_TYPE_PI << 24;
+                     EpgBlockSwapEndian(pBlock);
+                  }
+
+                  switch (type)
                   {
                      case BLOCK_TYPE_AI:
                         if (lastType == BLOCK_TYPE_INVALID)
@@ -627,13 +669,15 @@ PDBC EpgDbReload( uint cni, EPGDB_RELOAD_RESULT * pResult )
                         break;
 
                      case BLOCK_TYPE_PI:
-                        // in demo mode, add time offset to shift PI into presence or future
+                        // in demo mode, add time offset to shift PI into present or future
                         ((PI_BLOCK*)&pBlock->blk.pi)->start_time += piStartOff;
                         ((PI_BLOCK*)&pBlock->blk.pi)->stop_time  += piStartOff;
                         result = EpgDbReloadAddPiBlock(dbc, pBlock, pPrevNetwop);
                         break;
 
                      case BLOCK_TYPE_DEFECT_PI:
+                        // convert type to normal PI, since the "defect pi" type is known only inside this module
+                        pBlock->type = BLOCK_TYPE_PI;
                         result = EpgDbReloadAddDefectPiBlock(dbc, pBlock);
                         break;
 
@@ -711,31 +755,47 @@ PDBC EpgDbReload( uint cni, EPGDB_RELOAD_RESULT * pResult )
 // Scan database file for the OI block #0
 // - returns a pointer to the block (the caller must free the memory)
 //
-static EPGDB_BLOCK * EpgDbPeekOi( int fd )
+static EPGDB_BLOCK * EpgDbPeekOi( int fd, bool swapEndian )
 {
    EPGDB_BLOCK * pBlock;
    uchar buffer[BLK_UNION_OFF];
-   size_t size;
+   BLOCK_TYPE  type;
+   size_t      readSize;
+   uint32_t    size;
 
    pBlock = NULL;
 
    while (read(fd, buffer, BLK_UNION_OFF) == BLK_UNION_OFF)
    {
       size = ((EPGDB_BLOCK *)buffer)->size;
+      if (swapEndian)
+         swap32(&size);
       // plausibility check for block size (avoid malloc failure, which reults in program abort)
       if (size <= EPGDBSAV_MAX_BLOCK_SIZE)
       {
-         if ( ((EPGDB_BLOCK *)buffer)->type == BLOCK_TYPE_OI )
+         type = ((EPGDB_BLOCK *)buffer)->type;
+         if (swapEndian)
+            swap32(&type);
+
+         if (type == BLOCK_TYPE_OI)
          {
             pBlock = (EPGDB_BLOCK *) xmalloc(size + BLK_UNION_OFF);
             memcpy(pBlock, buffer, BLK_UNION_OFF);
             // load the rest of the block from the file
-            size = read(fd, (uchar *)pBlock + BLK_UNION_OFF, pBlock->size);
-            if (size == pBlock->size)
+            readSize = read(fd, (uchar *)pBlock + BLK_UNION_OFF, size);
+            if (readSize == size)
             {
+               if (swapEndian)
+                  EpgBlockSwapEndian(pBlock);
+
                if (pBlock->blk.oi.block_no == 0)
-               {  // found -> done
+               {  // found the block -> check it's consistancy
                   pBlock->pNextBlock = NULL;
+                  if (EpgBlockCheckConsistancy(pBlock) == FALSE)
+                  {
+                     xfree(pBlock);
+                     pBlock = NULL;
+                  }
                }
                else
                {  // OI block, but not #0 -> finished (blocks are sorted by increasing blockno)
@@ -743,24 +803,18 @@ static EPGDB_BLOCK * EpgDbPeekOi( int fd )
                   pBlock = NULL;
                }
 
-               // found the block -> check it's consistancy
-               if (EpgBlockCheckConsistancy(pBlock) == FALSE)
-               {
-                  xfree(pBlock);
-                  pBlock = NULL;
-               }
-
                // done: return the found block or NULL
                break;
             }
             else
             {
-               debug2("EpgDb-PeekOi: block read error: want %d, got %d", pBlock->size, size);
+               debug2("EpgDb-PeekOi: block read error: want %d, got %d", size, readSize);
                xfree(pBlock);
                pBlock = NULL;
+               break;
             }
          }
-         else if ( ((EPGDB_BLOCK *)buffer)->type == BLOCK_TYPE_PI )
+         else if ( ((EPGDB_BLOCK *)buffer)->type >= BLOCK_TYPE_PI )
          {  // read past all generic blocks -> finished
             break;
          }
@@ -791,9 +845,11 @@ EPGDB_CONTEXT * EpgDbPeek( uint cni, EPGDB_RELOAD_RESULT * pResult )
    EPGDB_CONTEXT * pDbContext;
    EPGDB_BLOCK   * pBlock;
    uchar buffer[BLK_UNION_OFF];
-   size_t size;
-   int fd;
-   uchar *pFilename;
+   uint32_t size;
+   size_t  readSize;
+   int     fd;
+   bool    swapEndian;
+   uchar * pFilename;
    EPGDB_RELOAD_RESULT result;
 
    pDbContext = NULL;
@@ -805,35 +861,48 @@ EPGDB_CONTEXT * EpgDbPeek( uint cni, EPGDB_RELOAD_RESULT * pResult )
       pDbContext = xmalloc(sizeof(EPGDB_CONTEXT));
       memset(pDbContext, 0, sizeof(EPGDB_CONTEXT));
 
-      result = EpgDbReloadHeader(cni, fd, &head);
+      result = EpgDbReloadHeader(cni, fd, &head, &swapEndian);
       if (result == EPGDB_RELOAD_OK)
       {
          pDbContext->pageNo       = head.pageNo;
          pDbContext->tunerFreq    = head.tunerFreq;
          pDbContext->appId        = head.appId;
+         pDbContext->expireDelayPi = epgDbReloadExpireDelayPi;
 
-         if ((size = read(fd, buffer, BLK_UNION_OFF)) == BLK_UNION_OFF)
+         if (read(fd, buffer, BLK_UNION_OFF) == BLK_UNION_OFF)
          {
             size = ((EPGDB_BLOCK *)buffer)->size;
+            if (swapEndian)
+               swap32(&size);
             // plausibility check for block size (avoid malloc failure, which reults in program abort)
             if (size <= EPGDBSAV_MAX_BLOCK_SIZE)
             {
                pBlock = (EPGDB_BLOCK *) xmalloc(size + BLK_UNION_OFF);
                memcpy(pBlock, buffer, BLK_UNION_OFF);
                // load the rest of the block from the file
-               size = read(fd, (uchar *)pBlock + BLK_UNION_OFF, pBlock->size);
-               // check if it's a valid AI block (the first block in the dumped db must be AI)
-               if ( (size == pBlock->size) &&
-                    (pBlock->type == BLOCK_TYPE_AI) &&
-                    EpgBlockCheckConsistancy(pBlock) )
+               readSize = read(fd, (uchar *)pBlock + BLK_UNION_OFF, size);
+               if (readSize == size)
                {
-                  pDbContext->pAiBlock = pBlock;
-                  pDbContext->pFirstGenericBlock[BLOCK_TYPE_OI] = EpgDbPeekOi(fd);
+                  if (swapEndian)
+                     EpgBlockSwapEndian(pBlock);
+
+                  // check if it's a valid AI block (the first block in the dumped db must be AI)
+                  if ( (pBlock->type == BLOCK_TYPE_AI) &&
+                       EpgBlockCheckConsistancy(pBlock) )
+                  {
+                     pDbContext->pAiBlock = pBlock;
+                     pDbContext->pFirstGenericBlock[BLOCK_TYPE_OI] = EpgDbPeekOi(fd, swapEndian);
+                  }
+                  else
+                  {
+                     debug0("EpgDb-Peek: no AI block found");
+                     result = EPGDB_RELOAD_CORRUPT;
+                     xfree(pBlock);
+                  }
                }
                else
                {
-                  debug0("EpgDb-Peek: no AI block found");
-                  result = EPGDB_RELOAD_CORRUPT;
+                  debug2("EpgDb-Peek: block read error: want %d, got %d", size, readSize);
                   xfree(pBlock);
                }
             }
@@ -1008,6 +1077,14 @@ const EPGDB_SCAN_BUF * EpgDbReloadScan( void )
 #endif  //WIN32
 }
 
+// ---------------------------------------------------------------------------
+// Initialize expire time for reloading PI
+//
+void EpgDbSavSetPiExpireDelay( time_t expireDelayPi )
+{
+   epgDbReloadExpireDelayPi = expireDelayPi;
+}
+
 // ---------------------------------------------------------------------------
 // Create database directory, if neccessary
 //
@@ -1026,34 +1103,24 @@ bool EpgDbSavSetupDir( const char * pDirPath, const char * pDemoDb )
    else if (pDirPath != NULL)
    {
       if (stat(pDirPath, &st) != 0)
-      {  // directory does no exist -> create it
+      {  // directory does not exist -> create it
          if ((errno != ENOENT) || (mkdir(pDirPath, 0777) != 0))
          {
-            fprintf(stderr, "cannot create dbdir %s: %s\n", pDirPath, strerror(errno));
+            fprintf(stderr, "cannot create database dir %s: %s\n", pDirPath, strerror(errno));
             result = FALSE;
          }
       }
       else if (S_ISDIR(st.st_mode) == FALSE)
       {  // target is not a directory -> warn
-         fprintf(stderr, "dbdir '%s' is not a directory\n", pDirPath);
+         fprintf(stderr, "database path '%s' exists but is not a directory\n", pDirPath);
          result = FALSE;
       }
-
-      if (result)
-      {  // set permissions of database directory: world-r/w-access
-         if (((st.st_mode & 07777) != 0777) && (chmod(pDirPath, 0777) != 0))
-         {
-            fprintf(stderr, "warning: cannot set permissions 0777 for dbdir %s: %s\n", pDirPath, strerror(errno));
-            // this is not a fatal error, result remains TRUE
-         }
-      }
    }
 
 #else  //WIN32
-   HANDLE hFind;
-   WIN32_FIND_DATA finddata;
    DWORD attr;
-   char msg[256];
+   DWORD errCode;
+   char  * pError = NULL;
 
    epgDbDirPath = pDirPath;
    if (pDemoDb != NULL)
@@ -1062,25 +1129,50 @@ bool EpgDbSavSetupDir( const char * pDirPath, const char * pDemoDb )
    }
    else if (pDirPath != NULL)
    {
-      hFind = FindFirstFile(pDirPath, &finddata);
-      if (hFind == (HANDLE) -1)
-      {  // directory does no exist -> create it
-         if (mkdir(epgDbDirPath) != 0)
-         {  // creation failed -> warn
-            sprintf(msg, "Cannot create database directory %s:\n%s\nCheck your -dbdir command line option", pDirPath, strerror(errno));
-            MessageBox(NULL, msg, "Nextview EPG", MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
+      attr = GetFileAttributes(pDirPath);
+      if (attr == INVALID_FILE_ATTRIBUTES)
+      {
+         errCode = GetLastError();
+         debug1("EpgDb-SavSetupDir: dbdir error %ld", errCode);
+
+         if (errCode == ERROR_FILE_NOT_FOUND)
+         {
+            // directory does no exist -> create it
+            if (CreateDirectory(epgDbDirPath, NULL) == 0)
+            {  // creation failed -> warn
+               SystemErrorMessage_Set(&pError, errCode, "Check option -dbdir ", pDirPath,
+                                      "\nPath does not exist and cannot be created: ", NULL);
+               if (pError != NULL)
+               {
+                  MessageBox(NULL, pError, "Nextview EPG", MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
+                  xfree(pError);
+               }
+               result = FALSE;
+            }
+         }
+         else
+         {
+            errCode = GetLastError();
+            SystemErrorMessage_Set(&pError, errCode, "Cannot access database directory:\n",
+                                                     pDirPath, "\n", NULL);
+            if (pError != NULL)
+            {
+               MessageBox(NULL, pError, "Nextview EPG", MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
+               xfree(pError);
+            }
             result = FALSE;
          }
       }
-      if (result)
-      {  // check if the file found is a directory
-         attr = GetFileAttributes(pDirPath);
-         if ((attr & FILE_ATTRIBUTE_DIRECTORY) == 0)
-         {  // target not a directory -> warn
-            sprintf(msg, "Invalid database directory specified: %s\nAlready exists, but is not a directory", pDirPath);
-            MessageBox(NULL, msg, "Nextview EPG", MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
-            result = FALSE;
+      else if ((attr & FILE_ATTRIBUTE_DIRECTORY) == 0)
+      {  // target not a directory -> warn
+         SystemErrorMessage_Set(&pError, 0, "Invalid database directory specified:\n",
+                                pDirPath, "\nAlready exists, but is not a directory", NULL);
+         if (pError != NULL)
+         {
+            MessageBox(NULL, pError, "Nextview EPG", MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
+            xfree(pError);
          }
+         result = FALSE;
       }
    }
 #endif
@@ -1150,13 +1242,15 @@ void EpgDbDumpGetDirAndCniFromArg( char * pArg, const char ** ppDirPath, uint *
 // ---------------------------------------------------------------------------
 // Update the tuner frequency in the dump file header
 //
-bool EpgDbDumpUpdateHeader( uint cni, ulong freq )
+bool EpgDbDumpUpdateHeader( uint cni, uint freq )
 {
    EPGDBSAV_HEADER head;
-   size_t size;
-   uchar * pFilename;
-   int fd;
-   bool result = FALSE;
+   size_t   size;
+   uchar  * pFilename;
+   int      fd;
+   uint32_t headCompatVersion;
+   uint32_t headCni;
+   bool     result = FALSE;
 
    pFilename = xmalloc(strlen(epgDbDirPath) + 1 + DUMP_NAME_MAX);
    sprintf(pFilename, "%s/" DUMP_NAME_FMT, epgDbDirPath, cni);
@@ -1167,28 +1261,45 @@ bool EpgDbDumpUpdateHeader( uint cni, ulong freq )
       if (size == sizeof(head))
       {
          if ( (strncmp(head.magic, MAGIC_STR, MAGIC_STR_LEN) == 0) &&
-              (head.compatVersion == DUMP_COMPAT) &&
-              (head.endianMagic == ENDIAN_MAGIC) &&
-              (head.cni == cni) )
+              ( (head.endianMagic == ENDIAN_MAGIC) ||
+                (head.endianMagic == WRONG_ENDIAN)) )
          {
-            head.tunerFreq = freq;
+            headCompatVersion = head.compatVersion;
+            headCni           = head.cni;
 
-            // rewind to the start of file and overwrite header
-            if (lseek(fd, 0, SEEK_SET) == 0)
+            if (head.endianMagic == WRONG_ENDIAN)
             {
-               size = write(fd, &head, sizeof(head));
-               if (size == sizeof(head))
+               swap32(&headCompatVersion);
+               swap32(&headCni);
+            }
+
+            if ( (headCompatVersion == DUMP_COMPAT) &&
+                 (headCni == cni) )
+            {
+               head.tunerFreq = freq;
+
+               if (head.endianMagic == WRONG_ENDIAN)
+                  swap32(&head.tunerFreq);
+
+               // rewind to the start of file and overwrite header
+               if (lseek(fd, 0, SEEK_SET) == 0)
                {
-                  result = TRUE;
+                  size = write(fd, &head, sizeof(head));
+                  if (size == sizeof(head))
+                  {
+                     result = TRUE;
+                  }
+                  else
+                     debug2("EpgDbDump-UpdateHeader: write failed with %d, errno %d\n", size, errno);
                }
                else
-                  debug2("EpgDbDump-UpdateHeader: write failed with %d, errno %d\n", size, errno);
+                  debug1("EpgDbDump-UpdateHeader: seek failed with errno %d", errno);
             }
             else
-               debug1("EpgDbDump-UpdateHeader: seek failed with errno %d", errno);
+               debug2("EpgDbDump-UpdateHeader: invalid CNI 0x%04X in db, != 0x%04X", head.cni, cni);
          }
          else
-            debug0("EpgDbDump-UpdateHeader: invalid magic or cni in db");
+            debug0("EpgDbDump-UpdateHeader: invalid magic in db");
       }
       close(fd);
    }
@@ -1216,11 +1327,21 @@ time_t EpgReadAiUpdateTime( uint cni )
       if (size == sizeof(head))
       {
          if ( (strncmp(head.magic, MAGIC_STR, MAGIC_STR_LEN) == 0) &&
-              (head.compatVersion == DUMP_COMPAT) &&
-              (head.endianMagic == ENDIAN_MAGIC) &&
-              (head.cni == cni) )
+              ( (head.endianMagic == ENDIAN_MAGIC) ||
+                (head.endianMagic == WRONG_ENDIAN)) )
          {
-            aiUpdateTime = head.lastAiUpdate;
+            if (head.endianMagic == WRONG_ENDIAN)
+            {
+               swap32(&head.compatVersion);
+               swap32(&head.cni);
+               swap32(&head.lastAiUpdate);
+            }
+
+            if ( (head.compatVersion == DUMP_COMPAT) &&
+                 (head.cni == cni) )
+            {
+               aiUpdateTime = head.lastAiUpdate;
+            }
          }
       }
       close(fd);
@@ -1233,13 +1354,13 @@ time_t EpgReadAiUpdateTime( uint cni )
 // ---------------------------------------------------------------------------
 // Reads tuner frequency from a database header, even for incompatible db versions
 //
-ulong EpgDbReadFreqFromDefective( uint cni )
+uint EpgDbReadFreqFromDefective( uint cni )
 {
    EPGDBSAV_HEADER head;
    size_t  size;
    uchar * pFilename;
    int     fd;
-   ulong   tunerFreq = 0L;
+   uint    tunerFreq = 0;
 
    pFilename = xmalloc(strlen(epgDbDirPath) + 1 + DUMP_NAME_MAX);
    sprintf(pFilename, "%s/" DUMP_NAME_FMT, epgDbDirPath, cni);
@@ -1250,10 +1371,19 @@ ulong EpgDbReadFreqFromDefective( uint cni )
       if (size == sizeof(head))
       {
          if ( (strncmp(head.magic, MAGIC_STR, MAGIC_STR_LEN) == 0) &&
-              (head.endianMagic == ENDIAN_MAGIC) &&
-              (head.cni == cni) )
+              ( (head.endianMagic == ENDIAN_MAGIC) ||
+                (head.endianMagic == WRONG_ENDIAN)) )
          {
-            tunerFreq = head.tunerFreq;
+            if (head.endianMagic == WRONG_ENDIAN)
+            {
+               swap32(&head.cni);
+               swap32(&head.tunerFreq);
+            }
+
+            if (head.cni == cni)
+            {
+               tunerFreq = head.tunerFreq;
+            }
          }
       }
       close(fd);
@@ -1263,3 +1393,28 @@ ulong EpgDbReadFreqFromDefective( uint cni )
    return tunerFreq;
 }
 
+// ---------------------------------------------------------------------------
+// Remove a provider database
+// - returns 0 on success, else an POSIX error code
+//
+uint EpgDbRemoveDatabaseFile( uint cni )
+{
+   uchar * pFilename;
+   uint result;
+
+   pFilename = xmalloc(strlen(epgDbDirPath) + 1 + DUMP_NAME_MAX);
+   sprintf(pFilename, "%s/" DUMP_NAME_FMT, epgDbDirPath, cni);
+
+   if ( unlink(pFilename) != 0 )
+   {
+      debug2("EpgDb-RemoveDatabaseFile: failed to remove db file '%s': %s", pFilename, strerror(errno));
+      result = errno;
+   }
+   else
+      result = 0;
+
+   xfree(pFilename);
+
+   return result;
+}
+
