@@ -22,12 +22,18 @@
  *  Author:
  *          Tom Zoerner
  *
- *  $Id: epgnetio.c,v 1.26 2002/03/02 14:48:15 tom Exp tom $
+ *  $Id: epgnetio.c,v 1.35 2003/10/05 19:03:28 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGDB
 #define DPRINTF_OFF
 
+#ifdef USE_DAEMON
+
+#ifdef WIN32
+#include <winsock2.h>
+#endif
+#include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
@@ -35,8 +41,10 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <time.h>
+#include <signal.h>
 #include <sys/stat.h>
 #ifndef WIN32
+#include <sys/signal.h>
 #include <sys/utsname.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -44,12 +52,11 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <syslog.h>
-#else
-#include <winsock2.h>
 #endif
 
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
+#include "epgvbi/syserrmsg.h"
 #include "epgdb/epgblock.h"
 #include "epgdb/epgqueue.h"
 #include "epgdb/epgtscqueue.h"
@@ -58,10 +65,19 @@
 #include "epgdb/epgdbsav.h"
 #include "epgdb/epgnetio.h"
 
-#if defined(linux) || defined(__NetBSD__)
+#if defined(linux) || defined(__NetBSD__) || defined(__FreeBSD__)
 #define HAVE_GETADDRINFO
 #endif
 
+#ifndef WIN32
+#define closesocket(FD)  close(FD)
+#define sockErrno        errno
+#define sockStrError(X)  strerror(X)
+#else
+#define sockErrno        WSAGetLastError()
+#define sockStrError(X)  WinSocketStrError(X)
+#endif
+
 
 typedef struct
 {
@@ -85,67 +101,79 @@ static EPGNETIO_LOGCF epgNetIoLogCf =
 #define SRV_LISTEN_BACKLOG_LEN  10
 #define SRV_CLNT_SOCK_PATH      "/tmp/nxtvepg.0"
 
-// ----------------------------------------------------------------------------
-// Save text describing network error cause
-// - argument list has to be terminated with NULL pointer
-// - to be displayed by the GUI to help the user fixing the problem
-//
-void EpgNetIo_SetErrorText( char ** ppErrorText, const char * pText, ... )
-{
-   va_list argl;
-   const char *argv[20];
-   uint argc, sumlen, off, idx;
+// settings & variables for WIN NT event log
+#ifdef WIN32
+#define EVLOG_APPNAME     "nxtvepg daemon"
+#define EVLOG_REGKEY_NAME "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\" EVLOG_APPNAME
+#define EVLOG_MSGFILE     "%SystemRoot%\\System32\\netmsg.dll"
+#define EVLOG_ERRNO       3299
+static const uchar * const pEvlogMsgFile = EVLOG_MSGFILE;
+static HANDLE hEventSource = NULL;
+#endif
 
-   // free the previous error text
-   if (*ppErrorText != NULL)
+// ---------------------------------------------------------------------------
+// Return socket system error string
+//
+#if defined(WIN32) && (DEBUG_SWITCH == ON)
+static const char * WinSocketStrError( DWORD errCode )
+{
+   static uchar msg[300];
+
+   // translate the error code into a human readable text
+   if (FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, errCode, LANG_USER_DEFAULT,
+                     msg, strlen(msg) - 1, NULL) == 0)
    {
-      xfree(*ppErrorText);
-      *ppErrorText = NULL;
+      if ((errCode < WSABASEERR) || (errCode > WSABASEERR + 2000))
+      {
+         debug1("WinSocketStrError: FormatMessage failed for errCode %ld", errCode);
+         sprintf(msg, "System error #%ld", errCode);
+      }
+      else
+         sprintf(msg, "WinSock error #%ld", errCode);
    }
 
-   // collect all given strings
-   if (pText != NULL)
+   return msg;
+}
+#endif  // WIN32 && DEBUG_SWITCH==ON
+
+#ifdef WIN32
+// ----------------------------------------------------------------------------
+// WIN NT syslog output
+//
+static void EpgNetIo_Win32SyslogOpen( void )
+{
+   DWORD  evMask;
+   HKEY   hKey;
+
+   if (hEventSource == NULL)
    {
-      argc    = 1;
-      argv[0] = pText;
-      sumlen  = strlen(pText);
-
-      va_start(argl, pText);
-      while (argc < 20 - 1)
+      if (RegCreateKey(HKEY_LOCAL_MACHINE, EVLOG_REGKEY_NAME, &hKey) == 0)
       {
-         argv[argc] = va_arg(argl, char *);
-         if (argv[argc] == NULL)
-            break;
+         RegSetValueEx(hKey, "EventMessageFile", 0, REG_EXPAND_SZ, (LPBYTE) pEvlogMsgFile, strlen(pEvlogMsgFile) + 1);
 
-         sumlen += strlen(argv[argc]);
-         argc += 1;
+         evMask = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;
+
+         RegSetValueEx(hKey, "TypesSupported", 0, REG_DWORD, (LPBYTE) &evMask, sizeof(evMask));
+         RegCloseKey(hKey);
       }
-      va_end(argl);
 
-      if (argc > 0)
-      {
-         // allocate memory for sum of all strings length
-         *ppErrorText = xmalloc(sumlen + 1);
-
-         // concatenate the strings
-         off = 0;
-         for (idx=0; idx < argc; idx++)
-         {
-            strcpy(*ppErrorText + off, argv[idx]);
-            off += strlen(argv[idx]);
-         }
-      }
+      hEventSource = RegisterEventSource(NULL, EVLOG_APPNAME);
    }
 }
+#endif
 
 // ----------------------------------------------------------------------------
 // Append entry to logfile
 //
-void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
+void EpgNetIo_Logger( int level, int clnt_fd, int errCode, const char * pText, ... )
 {
+   #ifdef WIN32
+   uchar   sysErrStr[160];
+   uint    evType;
+   #endif
    va_list argl;
    char timestamp[32], fdstr[20];
-   const char *argv[5];
+   const char *argv[10];
    uint argc, idx;
    sint fd;
    time_t now = time(NULL);
@@ -153,9 +181,10 @@ void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
    if (pText != NULL)
    {
       // open the logfile, if one is configured
-      if (level <= epgNetIoLogCf.fileloglev)
+      if ( (level <= epgNetIoLogCf.fileloglev) &&
+           (epgNetIoLogCf.pLogfileName != NULL) )
       {
-         fd = open("daemon.out", O_WRONLY|O_CREAT|O_APPEND, 0666);
+         fd = open(epgNetIoLogCf.pLogfileName, O_WRONLY|O_CREAT|O_APPEND, 0666);
          if (fd >= 0)
          {  // each line in the file starts with a timestamp
             strftime(timestamp, sizeof(timestamp) - 1, "[%d/%b/%Y:%H:%M:%S +0000] ", gmtime(&now));
@@ -169,16 +198,16 @@ void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
          fprintf(stderr, "nxtvepg: ");
 
       argc = 0;
+      memset(argv, 0, sizeof(argv));
       // add pointer to file descriptor (for client requests) or pid (for general infos)
       if (clnt_fd != -1)
          sprintf(fdstr, "fd %d: ", clnt_fd);
       else
       {
          #ifndef WIN32
-         sprintf(fdstr, "pid %d: ", getpid());
+         sprintf(fdstr, "pid %d: ", (int)getpid());
          #else
-         // XXX TODO: check how to query pid on windows
-         strcpy(fdstr, "daemon: ");
+         sprintf(fdstr, "pid %ld: ", GetCurrentProcessId());
          #endif
       }
       argv[argc++] = fdstr;
@@ -194,6 +223,17 @@ void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
       }
       va_end(argl);
 
+      // add system error message
+      if (errCode != 0)
+      {
+         #ifndef WIN32
+         argv[argc++] = strerror(errCode);
+         #else
+         FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, errCode, LANG_USER_DEFAULT, sysErrStr, sizeof(sysErrStr) - 1, NULL);
+         argv[argc++] = sysErrStr;
+         #endif
+      }
+
       // print the strings to the file and/or stderr
       for (idx=0; idx < argc; idx++)
       {
@@ -209,7 +249,6 @@ void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
          write(fd, "\n", 1);
          close(fd);
       }
-      #ifndef WIN32
       if (epgNetIoLogCf.do_logtty && (level <= LOG_WARNING))
       {
          fprintf(stderr, "\n");
@@ -219,6 +258,7 @@ void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
       // syslog output
       if (level <= epgNetIoLogCf.sysloglev)
       {
+         #ifndef WIN32
          switch (argc)
          {
             case 1: syslog(level, "%s", argv[0]); break;
@@ -226,11 +266,19 @@ void EpgNetIo_Logger( int level, int clnt_fd, const char * pText, ... )
             case 3: syslog(level, "%s%s%s", argv[0], argv[1],argv[2]); break;
             case 4: syslog(level, "%s%s%s%s", argv[0], argv[1],argv[2],argv[3]); break;
          }
+         #else
+         if (hEventSource != NULL)
+         {
+            if (level <= LOG_ERR)
+               evType = EVENTLOG_ERROR_TYPE;
+            else if (level <= LOG_WARNING)
+               evType = EVENTLOG_WARNING_TYPE;
+            else
+               evType = EVENTLOG_INFORMATION_TYPE;
+            ReportEvent(hEventSource, evType, 0, EVLOG_ERRNO, NULL, 9, 0, argv, NULL);
+         }
+         #endif
       }
-      #else
-      // on M$-Windows we don't have a terminal or console
-      epgNetIoLogCf.do_logtty = FALSE;
-      #endif
    }
 }
 
@@ -261,16 +309,20 @@ void EpgNetIo_SetLogging( int sysloglev, int fileloglev, const char * pLogfileNa
    else
       epgNetIoLogCf.fileloglev = -1;
 
-   #ifndef WIN32
    if (sysloglev && !epgNetIoLogCf.sysloglev)
    {
+      #ifndef WIN32
       openlog("nxtvepg", LOG_PID, LOG_DAEMON);
+      #else
+      EpgNetIo_Win32SyslogOpen();
+      #endif
    }
    else if (!sysloglev && epgNetIoLogCf.sysloglev)
    {
+      #ifndef WIN32
       closelog();
+      #endif
    }
-   #endif
 
    // convert GUI log-level setting to syslog enum value
    epgNetIoLogCf.sysloglev = ((sysloglev > 0) ? (sysloglev + LOG_ERR) : -1);
@@ -322,7 +374,7 @@ bool EpgNetIo_HandleIO( EPGNETIO_STATE * pIO, bool * pBlocked, bool closeOnZeroR
       assert(pIO->writeLen >= sizeof(EPGNETIO_MSG_HEADER));
       if (pIO->writeOff < sizeof(EPGNETIO_MSG_HEADER))
       {  // write message header
-         len = write(pIO->sock_fd, ((char *)&pIO->writeHeader) + pIO->writeOff, sizeof(EPGNETIO_MSG_HEADER) - pIO->writeOff);
+         len = send(pIO->sock_fd, ((char *)&pIO->writeHeader) + pIO->writeOff, sizeof(EPGNETIO_MSG_HEADER) - pIO->writeOff, 0);
          if (len >= 0)
          {
             pIO->lastIoTime = now;
@@ -335,7 +387,7 @@ bool EpgNetIo_HandleIO( EPGNETIO_STATE * pIO, bool * pBlocked, bool closeOnZeroR
       // write the message body, if the header is written
       if ((err == FALSE) && (pIO->writeOff >= sizeof(EPGNETIO_MSG_HEADER)) && (pIO->writeOff < pIO->writeLen))
       {
-         len = write(pIO->sock_fd, ((char *)pIO->pWriteBuf) + pIO->writeOff - sizeof(EPGNETIO_MSG_HEADER), pIO->writeLen - pIO->writeOff);
+         len = send(pIO->sock_fd, ((char *)pIO->pWriteBuf) + pIO->writeOff - sizeof(EPGNETIO_MSG_HEADER), pIO->writeLen - pIO->writeOff, 0);
          if (len > 0)
          {
             pIO->lastIoTime = now;
@@ -357,12 +409,20 @@ bool EpgNetIo_HandleIO( EPGNETIO_STATE * pIO, bool * pBlocked, bool closeOnZeroR
          else if (pIO->writeOff < pIO->writeLen)
             *pBlocked = TRUE;
       }
+      #ifndef WIN32
       else if ((errno != EAGAIN) && (errno != EINTR))
+      #else
+      else if ((WSAGetLastError() != WSAEWOULDBLOCK))
+      #endif
       {  // network error -> close the connection
-         debug2("EpgNetIo-HandleIO: write error on fd %d: %s", pIO->sock_fd, strerror(errno));
+         debug2("EpgNetIo-HandleIO: write error on fd %d: %s", pIO->sock_fd, sockStrError(sockErrno));
          result = FALSE;
       }
+      #ifndef WIN32
       else if (errno == EAGAIN)
+      #else
+      else if (WSAGetLastError() == WSAEWOULDBLOCK)
+      #endif
          *pBlocked = TRUE;
    }
    else if (pIO->waitRead || (pIO->readLen > 0))
@@ -371,7 +431,7 @@ bool EpgNetIo_HandleIO( EPGNETIO_STATE * pIO, bool * pBlocked, bool closeOnZeroR
       if (pIO->waitRead)
       {  // in read phase one: read the message length
          assert(pIO->readOff < sizeof(pIO->readHeader));
-         len = read(pIO->sock_fd, (char *)&pIO->readHeader + pIO->readOff, sizeof(pIO->readHeader) - pIO->readOff);
+         len = recv(pIO->sock_fd, (char *)&pIO->readHeader + pIO->readOff, sizeof(pIO->readHeader) - pIO->readOff, 0);
          if (len > 0)
          {
             closeOnZeroRead = FALSE;
@@ -407,7 +467,7 @@ bool EpgNetIo_HandleIO( EPGNETIO_STATE * pIO, bool * pBlocked, bool closeOnZeroR
       if ((err == FALSE) && (pIO->waitRead == FALSE) && (pIO->readLen > sizeof(EPGNETIO_MSG_HEADER)))
       {  // in read phase two: read the complete message into the allocated buffer
          assert(pIO->pReadBuf != NULL);
-         len = read(pIO->sock_fd, pIO->pReadBuf + pIO->readOff - sizeof(EPGNETIO_MSG_HEADER), pIO->readLen - pIO->readOff);
+         len = recv(pIO->sock_fd, pIO->pReadBuf + pIO->readOff - sizeof(EPGNETIO_MSG_HEADER), pIO->readLen - pIO->readOff, 0);
          if (len > 0)
          {
             pIO->lastIoTime = now;
@@ -431,12 +491,20 @@ bool EpgNetIo_HandleIO( EPGNETIO_STATE * pIO, bool * pBlocked, bool closeOnZeroR
             debug1("EpgNetIo-HandleIO: zero len read on fd %d", pIO->sock_fd);
             result = FALSE;
          }
+         #ifndef WIN32
          else if ((len < 0) && (errno != EAGAIN) && (errno != EINTR))
+         #else
+         else if ((len < 0) && (WSAGetLastError() != WSAEWOULDBLOCK))
+         #endif
          {  // network error -> close the connection
-            debug3("EpgNetIo-HandleIO: read error on fd %d: len=%d, %s", pIO->sock_fd, len, strerror(errno));
+            debug3("EpgNetIo-HandleIO: read error on fd %d: len=%d, %s", pIO->sock_fd, len, sockStrError(sockErrno));
             result = FALSE;
          }
+         #ifndef WIN32
          else if (errno == EAGAIN)
+         #else
+         else if (WSAGetLastError() == WSAEWOULDBLOCK)
+         #endif
             *pBlocked = TRUE;
       }
    }
@@ -451,7 +519,10 @@ void EpgNetIo_CloseIO( EPGNETIO_STATE * pIO )
 {
    if (pIO->sock_fd != -1)
    {
-      close(pIO->sock_fd);
+#ifdef WIN32
+      shutdown(pIO->sock_fd, SD_SEND);
+#endif
+      closesocket(pIO->sock_fd);
       pIO->sock_fd = -1;
    }
 
@@ -879,7 +950,7 @@ enum
    GAI_UNKNOWN_HOST    = -4,
 };
 
-int getaddrinfo( const char * pHostName, const char * pServiceName, const struct addrinfo * pInParams, struct addrinfo ** ppResult )
+static int getaddrinfo( const char * pHostName, const char * pServiceName, const struct addrinfo * pInParams, struct addrinfo ** ppResult )
 {
    struct servent  * pServiceEntry;
    struct hostent  * pHostEntry;
@@ -926,7 +997,7 @@ int getaddrinfo( const char * pHostName, const char * pServiceName, const struct
                res->ai_addrlen = sizeof(struct sockaddr_in);
 
                iad->sin_family      = AF_INET;
-               iad->sin_port        = port;
+               iad->sin_port        = htons(port);
                if (pHostName != NULL)
                   memcpy(&iad->sin_addr, (char *) pHostEntry->h_addr, pHostEntry->h_length);
                else
@@ -947,20 +1018,20 @@ int getaddrinfo( const char * pHostName, const char * pServiceName, const struct
 
    if (result != 0)
    {
-      xfree(res);
+      free(res);
       *ppResult = NULL;
    }
    return result;
 }
 
-void freeaddrinfo( struct addrinfo * res )
+static void freeaddrinfo( struct addrinfo * res )
 {
    if (res->ai_addr != NULL)
       free(res->ai_addr);
    free(res);
 }
 
-char * gai_strerror( int errCode )
+static char * gai_strerror( int errCode )
 {
    switch (errCode)
    {
@@ -985,7 +1056,7 @@ static int EpgNetIo_GetLocalSocketAddr( const char * pPathName, const struct add
    struct addrinfo * res;
    struct sockaddr_un * saddr;
 
-   assert((pInParams->ai_family == PF_LOCAL) && (pPathName != NULL));
+   assert((pInParams->ai_family == PF_UNIX) && (pPathName != NULL));
 
    // note: use regular malloc instead of xmalloc in case memory is freed by the libc internal freeaddrinfo
    res = malloc(sizeof(struct addrinfo));
@@ -1025,10 +1096,11 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
    ask.ai_flags    = AI_PASSIVE;
    ask.ai_socktype = SOCK_STREAM;
    sock_fd = -1;
+   res = NULL;
 
+   #ifdef PF_INET6
    if (is_tcp_ip)
    {  // try IP-v6: not supported everywhere yet, so errors must be silently ignored
-      res = NULL;
       ask.ai_family = PF_INET6;
       rc = getaddrinfo(listen_ip, listen_port, &ask, &res);
       if (rc == 0)
@@ -1044,6 +1116,7 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
       else
          debug1("EpgNetIo-Listen: getaddrinfo (ipv6): %s", gai_strerror(rc));
    }
+   #endif
 
    if (sock_fd == -1)
    {
@@ -1062,11 +1135,11 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
          sock_fd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
          if (sock_fd == -1)
          {
-            EpgNetIo_Logger(LOG_ERR, -1, "socket create failed: ", strerror(errno), NULL);
+            EpgNetIo_Logger(LOG_ERR, -1, sockErrno, "socket create failed: ", NULL);
          }
       }
       else
-         EpgNetIo_Logger(LOG_ERR, -1, "Invalid hostname or service/port: ", gai_strerror(rc), NULL);
+         EpgNetIo_Logger(LOG_ERR, -1, 0, "Invalid hostname or service/port: ", gai_strerror(rc), NULL);
    }
 
    if (sock_fd != -1)
@@ -1078,6 +1151,9 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
          // make the socket non-blocking
          #ifndef WIN32
          if (fcntl(sock_fd, F_SETFL, O_NONBLOCK) == 0)
+         #else
+         ulong ioArg = TRUE;
+         if (ioctlsocket(sock_fd, FIONBIO, &ioArg) == 0)
          #endif
          {
             // bind the socket
@@ -1087,7 +1163,7 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
                // set socket permissions: r/w allowed to everyone
                if ( (is_tcp_ip == FALSE) &&
                     (chmod(SRV_CLNT_SOCK_PATH, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH) != 0) )
-                  EpgNetIo_Logger(LOG_WARNING, -1, "chmod failed for named socket: ", strerror(errno), NULL);
+                  EpgNetIo_Logger(LOG_WARNING, -1, errno, "chmod failed for named socket: ", NULL);
                #endif
 
                // enable listening for new connections
@@ -1097,21 +1173,19 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
                }
                else
                {
-                  EpgNetIo_Logger(LOG_ERR, -1, "socket listen failed: ", strerror(errno), NULL);
+                  EpgNetIo_Logger(LOG_ERR, -1, sockErrno, "socket listen failed: ", NULL);
                   if (is_tcp_ip)
                      unlink(SRV_CLNT_SOCK_PATH);
                }
             }
             else
-               EpgNetIo_Logger(LOG_ERR, -1, "socket bind failed: ", strerror(errno), NULL);
+               EpgNetIo_Logger(LOG_ERR, -1, sockErrno, "socket bind failed: ", NULL);
          }
-         #ifndef WIN32
          else
-            EpgNetIo_Logger(LOG_ERR, -1, "socket fcntl(F_SETFL=O_NONBLOCK) failed: ", strerror(errno), NULL);
-         #endif
+            EpgNetIo_Logger(LOG_ERR, -1, sockErrno, "failed to set socket non-blocking: ", NULL);
       }
       else
-         EpgNetIo_Logger(LOG_ERR, -1, "socket setsockopt(SOL_SOCKET=SO_REUSEADDR) failed: ", strerror(errno), NULL);
+         EpgNetIo_Logger(LOG_ERR, -1, sockErrno, "socket setsockopt(SOL_SOCKET=SO_REUSEADDR) failed: ", NULL);
    }
 
    if (res != NULL)
@@ -1119,7 +1193,7 @@ int EpgNetIo_ListenSocket( bool is_tcp_ip, const char * listen_ip, const char *
 
    if ((result == FALSE) && (sock_fd != -1))
    {
-      close(sock_fd);
+      closesocket(sock_fd);
       sock_fd = -1;
    }
 
@@ -1136,7 +1210,7 @@ void EpgNetIo_StopListen( bool is_tcp_ip, int sock_fd )
       if (is_tcp_ip == FALSE)
          unlink(SRV_CLNT_SOCK_PATH);
 
-      close(sock_fd);
+      closesocket(sock_fd);
       sock_fd = -1;
    }
 }
@@ -1146,7 +1220,9 @@ void EpgNetIo_StopListen( bool is_tcp_ip, int sock_fd )
 //
 int EpgNetIo_AcceptConnection( int listen_fd )
 {
+   #ifndef WIN32
    struct hostent * hent;
+   #endif
    char  hname_buf[129];
    uint  length, maxLength;
    struct {  // allocate enough room for all possible types of socket address structs
@@ -1164,10 +1240,14 @@ int EpgNetIo_AcceptConnection( int listen_fd )
       {
          #ifndef WIN32
          if (fcntl(sock_fd, F_SETFL, O_NONBLOCK) == 0)
+         #else
+         ulong ioArg = TRUE;
+         if (ioctlsocket(sock_fd, FIONBIO, &ioArg) == 0)
          #endif
          {
             if (peerAddr.sa.sa_family == AF_INET)
             {
+               #ifndef WIN32  // XXX WIN32 is useless due to 5 secs timeout on 127.0.0.1
                hent = gethostbyaddr((void *) &peerAddr.sa, maxLength, AF_INET);
                if (hent != NULL)
                {
@@ -1175,9 +1255,10 @@ int EpgNetIo_AcceptConnection( int listen_fd )
                   hname_buf[sizeof(hname_buf) - 1] = 0;
                }
                else
+               #endif
                   sprintf(hname_buf, "%s, port %d", inet_ntoa(((struct sockaddr_in *) &peerAddr.sa)->sin_addr), ((struct sockaddr_in *) &peerAddr.sa)->sin_port);
 
-               EpgNetIo_Logger(LOG_INFO, sock_fd, "new connection from ", hname_buf, NULL);
+               EpgNetIo_Logger(LOG_INFO, sock_fd, 0, "new connection from ", hname_buf, NULL);
                result = TRUE;
             }
             #ifdef HAVE_GETADDRINFO
@@ -1185,62 +1266,107 @@ int EpgNetIo_AcceptConnection( int listen_fd )
             {
                if (getnameinfo(&peerAddr.sa, length, hname_buf, sizeof(hname_buf) - 1, NULL, 0, 0) == 0)
                {  // address could be resolved to hostname
-                  EpgNetIo_Logger(LOG_INFO, sock_fd, "new connection from ", hname_buf, NULL);
+                  EpgNetIo_Logger(LOG_INFO, sock_fd, 0, "new connection from ", hname_buf, NULL);
                   result = TRUE;
                }
                else if (getnameinfo(&peerAddr.sa, length, hname_buf, sizeof(hname_buf) - 1, NULL, 0,
                                     NI_NUMERICHOST | NI_NUMERICSERV) == 0)
                {  // resolver failed - but numeric conversion was successful
                   debug1("EpgNetIo-AcceptConnection: IPv6 resolver failed for %s", hname_buf);
-                  EpgNetIo_Logger(LOG_INFO, sock_fd, "new connection from ", hname_buf, NULL);
+                  EpgNetIo_Logger(LOG_INFO, sock_fd, 0, "new connection from ", hname_buf, NULL);
                   result = TRUE;
                }
                else
                {  // neither name looup nor numeric name output succeeded -> fatal error
-                  EpgNetIo_Logger(LOG_INFO, sock_fd, "new connection: failed to get IPv6 peer name or IP-addr: ", strerror(errno), NULL);
+                  EpgNetIo_Logger(LOG_INFO, sock_fd, sockErrno, "new connection: failed to get IPv6 peer name or IP-addr: ", NULL);
                   result = FALSE;
                }
             }
             #endif
             else if (peerAddr.sa.sa_family == AF_UNIX)
             {
-               EpgNetIo_Logger(LOG_INFO, sock_fd, "new connection from localhost via named socket", NULL);
+               EpgNetIo_Logger(LOG_INFO, sock_fd, 0, "new connection from localhost via named socket", NULL);
                result = TRUE;
             }
             else
             {  // neither INET nor named socket -> internal error
                sprintf(hname_buf, "%d", peerAddr.sa.sa_family);
-               EpgNetIo_Logger(LOG_WARNING, -1, "new connection via unexpected protocol family ", hname_buf, NULL);
+               EpgNetIo_Logger(LOG_WARNING, -1, 0, "new connection via unexpected protocol family ", hname_buf, NULL);
             }
          }
-         #ifndef WIN32
          else
          {  // fcntl failed: OS error (should never happen)
-            EpgNetIo_Logger(LOG_WARNING, -1, "new connection: failed to set socket to non-blocking: ", strerror(errno), NULL);
+            EpgNetIo_Logger(LOG_WARNING, -1, sockErrno, "new connection: failed to set socket to non-blocking: ", NULL);
          }
-         #endif
       }
       else
       {  // socket address buffer too small: internal error
          sprintf(hname_buf, "need %d, have %d", length, maxLength);
-         EpgNetIo_Logger(LOG_WARNING, -1, "new connection: saddr buffer too small: ", hname_buf, NULL);
+         EpgNetIo_Logger(LOG_WARNING, -1, 0, "new connection: saddr buffer too small: ", hname_buf, NULL);
       }
 
       if (result == FALSE)
       {  // error -> drop the connection
-         close(sock_fd);
+         closesocket(sock_fd);
          sock_fd = -1;
       }
    }
    else
    {  // connect accept failed: remote host may already have closed again
-      if (errno != EAGAIN)
-         EpgNetIo_Logger(LOG_INFO, -1, "accept failed: ", strerror(errno), NULL);
+      #ifndef WIN32
+      if (errno == EAGAIN)
+      #else
+      if (WSAGetLastError() == WSAEWOULDBLOCK)
+      #endif
+         EpgNetIo_Logger(LOG_INFO, -1, sockErrno, "accept failed: ", NULL);
    }
 
    return sock_fd;
 }
 
+// ----------------------------------------------------------------------------
+// Check if the given host name refers to the local host
+//
+bool EpgNetIo_IsLocalHost( char * pHostname )
+{
+   struct hostent  * pHostEntry;
+   struct in_addr  * pInAddr;
+   char local_name[100];
+   bool result;
+
+   if (gethostname(local_name, sizeof(local_name)-1) == 0)
+   {
+      if (strcmp(pHostname, local_name) != 0)
+      {
+         pHostEntry = gethostbyname(pHostname);
+         if (pHostEntry != NULL)
+         {
+            if (strcmp(pHostEntry->h_name, local_name) != 0)
+            {
+               pInAddr = (struct in_addr *)(pHostEntry->h_addr_list[0]);
+               if ( (pHostEntry->h_addrtype == AF_INET) &&
+                    (ntohl(pInAddr->s_addr) == INADDR_LOOPBACK) )
+               {
+                  result = TRUE;
+               }
+               else
+                  result = FALSE;
+            }
+            else
+               result = TRUE;
+         }
+         else
+            result = FALSE;
+      }
+      else
+         result = TRUE;
+   }
+   else
+      result = FALSE;
+
+   return result;
+}
+
 // ----------------------------------------------------------------------------
 // Attempt to connect to an already running server
 //
@@ -1266,7 +1392,7 @@ bool EpgNetIo_CheckConnect( void )
             result = TRUE;
          }
       }
-      close(fd);
+      closesocket(fd);
    }
 
    // if no server is listening, remove the socket from the file system
@@ -1299,6 +1425,7 @@ int EpgNetIo_ConnectToServer( bool use_tcp_ip, const char * pSrvHost, const char
    ask.ai_flags = 0;
    ask.ai_socktype = SOCK_STREAM;
 
+   #ifdef PF_INET6
    if (use_tcp_ip)
    {  // try IP-v6: not supported everywhere yet, so errors must be silently ignored
       ask.ai_family = PF_INET6;
@@ -1316,6 +1443,7 @@ int EpgNetIo_ConnectToServer( bool use_tcp_ip, const char * pSrvHost, const char
       else
          debug1("getaddrinfo (ipv6): %s", gai_strerror(rc));
    }
+   #endif
 
    if (sock_fd == -1)
    {
@@ -1334,14 +1462,14 @@ int EpgNetIo_ConnectToServer( bool use_tcp_ip, const char * pSrvHost, const char
          sock_fd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
          if (sock_fd == -1)
          {
-            debug1("socket (ipv4): %s", strerror(errno));
-            EpgNetIo_SetErrorText(ppErrorText, "Cannot create network socket: ", strerror(errno), NULL);
+            debug1("socket (ipv4): %s", sockStrError(sockErrno));
+            SystemErrorMessage_Set(ppErrorText, sockErrno, "Cannot create network socket: ", NULL);
          }
       }
       else
       {
          debug1("getaddrinfo (ipv4): %s", gai_strerror(rc));
-         EpgNetIo_SetErrorText(ppErrorText, "Invalid hostname or service/port: ", gai_strerror(rc), NULL);
+         SystemErrorMessage_Set(ppErrorText, 0, "Invalid hostname or service/port: ", gai_strerror(rc), NULL);
       }
    }
 
@@ -1349,12 +1477,17 @@ int EpgNetIo_ConnectToServer( bool use_tcp_ip, const char * pSrvHost, const char
    {
       #ifndef WIN32
       if (fcntl(sock_fd, F_SETFL, O_NONBLOCK) == 0)
+      #else
+      ulong ioArg = TRUE;
+      if (ioctlsocket(sock_fd, FIONBIO, &ioArg) == 0)
       #endif
       {
          // connect to the server socket
          if ( (connect(sock_fd, res->ai_addr, res->ai_addrlen) == 0)
               #ifndef WIN32
               || (errno == EINPROGRESS)
+              #else
+              || (WSAGetLastError() == WSAEWOULDBLOCK)
               #endif
               )
          {
@@ -1362,24 +1495,22 @@ int EpgNetIo_ConnectToServer( bool use_tcp_ip, const char * pSrvHost, const char
          }
          else
          {
-            debug1("connect: %s", strerror(errno));
+            debug1("connect: %s", sockStrError(sockErrno));
             if (use_tcp_ip)
-               EpgNetIo_SetErrorText(ppErrorText, "Server not running or not reachable: connect via TCP/IP failed: ", strerror(errno), NULL);
+               SystemErrorMessage_Set(ppErrorText, sockErrno, "Server not running or not reachable: connect via TCP/IP failed: ", NULL);
             else
-               EpgNetIo_SetErrorText(ppErrorText, "Server not running: connect via " SRV_CLNT_SOCK_PATH " failed: ", strerror(errno), NULL);
-            close(sock_fd);
+               SystemErrorMessage_Set(ppErrorText, sockErrno, "Server not running: connect via " SRV_CLNT_SOCK_PATH " failed: ", NULL);
+            closesocket(sock_fd);
             sock_fd = -1;
          }
       }
-      #ifndef WIN32
       else
       {
-         debug1("fcntl (F_SETFL=O_NONBLOCK): %s", strerror(errno));
-         EpgNetIo_SetErrorText(ppErrorText, "Failed to set socket non-blocking: ", strerror(errno), NULL);
-         close(sock_fd);
+         debug1("fcntl (F_SETFL=O_NONBLOCK): %s", sockStrError(sockErrno));
+         SystemErrorMessage_Set(ppErrorText, sockErrno, "Failed to set socket non-blocking: ", NULL);
+         closesocket(sock_fd);
          sock_fd = -1;
       }
-      #endif
    }
 
    if (res != NULL)
@@ -1390,12 +1521,15 @@ int EpgNetIo_ConnectToServer( bool use_tcp_ip, const char * pSrvHost, const char
 
 // ----------------------------------------------------------------------------
 // Check for the result of the connect syscall
-// - called when select() indicates writability
+// - UNIX: called when select() indicates writability
+// - Win32: called when select() indicates writablility (successful connected)
+//   or an exception (connect failed)
 //
 bool EpgNetIo_FinishConnect( int sock_fd, char ** ppErrorText )
 {
-   int  sockerr, sockerrlen;
    bool result = FALSE;
+#ifndef WIN32
+   int  sockerr, sockerrlen;
 
    sockerrlen = sizeof(sockerr);
    if (getsockopt(sock_fd, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &sockerrlen) == 0)
@@ -1407,16 +1541,65 @@ bool EpgNetIo_FinishConnect( int sock_fd, char ** ppErrorText )
       }
       else
       {  // failed to establish a connection to the server
-         debug1("EpgNetIo-FinishConnect: connect failed: %s", strerror(sockerr));
-         EpgNetIo_SetErrorText(ppErrorText, "Connect failed: ", strerror(sockerr), NULL);
+         debug1("EpgNetIo-FinishConnect: connect failed: %s", sockStrError(sockerr));
+         SystemErrorMessage_Set(ppErrorText, sockerr, "Connect failed: ", NULL);
       }
    }
    else
    {
-      debug1("EpgNetIo-FinishConnect: getsockopt: %s", strerror(errno));
-      EpgNetIo_SetErrorText(ppErrorText, "Failed to query socket connect result: ", strerror(errno), NULL);
+      debug1("EpgNetIo-FinishConnect: getsockopt: %s", sockStrError(sockErrno));
+      SystemErrorMessage_Set(ppErrorText, sockErrno, "Failed to query socket connect result: ", NULL);
    }
 
+#else  // WIN32
+   // on Windows the socket becomes writable if - and only if - the connect succeeded
+   dprintf0("EpgNetIo-FinishConnect: connect succeeded\n");
+   result = TRUE;
+#endif
+
    return result;
 }
 
+// ----------------------------------------------------------------------------
+// Initialize the module
+//
+bool EpgNetIo_Init( char ** ppErrorText )
+{
+#ifndef WIN32
+   struct sigaction  act;
+
+   // setup signal handler
+   memset(&act, 0, sizeof(act));
+   sigemptyset(&act.sa_mask);
+   act.sa_handler = SIG_IGN;
+   sigaction(SIGPIPE, &act, NULL);
+   return TRUE;
+
+#else // WIN32
+   WSADATA stWSAData;
+   bool   result;
+
+   result = (WSAStartup(0x202, &stWSAData) == 0);
+   if (result == FALSE)
+   {
+      if (ppErrorText == NULL)
+         EpgNetIo_Logger(LOG_ERR, -1, sockErrno, "winsock2 DLL init failed: ", NULL);
+      else
+         SystemErrorMessage_Set(ppErrorText, sockErrno, "Failed to initialize winsock2 DLL: ", NULL);
+   }
+
+   return result;
+#endif
+}
+
+// ----------------------------------------------------------------------------
+// Free module resources
+//
+void EpgNetIo_Destroy( void )
+{
+#ifdef WIN32
+   WSACleanup();
+#endif
+}
+
+#endif  // USE_DAEMON
