@@ -20,10 +20,10 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgstream.c,v 1.18 2001/12/21 20:06:07 tom Exp tom $
+ *  $Id: epgstream.c,v 1.28 2003/09/20 19:08:10 tom Exp tom $
  */
 
-#define DEBUG_SWITCH DEBUG_SWITCH_EPGDB
+#define DEBUG_SWITCH DEBUG_SWITCH_STREAM
 #define DPRINTF_OFF
 
 #include <stdio.h>
@@ -31,9 +31,12 @@
 
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
+
+#include "epgvbi/btdrv.h"
+#include "epgvbi/ttxdecode.h"
 #include "epgvbi/hamming.h"
 #include "epgdb/epgblock.h"
-#include "epgui/epgtxtdump.h"
+#include "epgui/dumpraw.h"
 #include "epgdb/epgqueue.h"
 #include "epgdb/epgstream.h"
 
@@ -59,17 +62,20 @@ typedef struct
    uchar   blockBuf[NXTV_BLOCK_MAXLEN + 4];
 } NXTV_STREAM;
 
-// max. number of pages that are allowed for EPG transmission: mFd & mdF: m[0..7],d[0..9]
-#define NXTV_VALID_PAGE_PER_MAG (1 + 10 + 10)
-#define NXTV_VALID_PAGE_COUNT   (8 * NXTV_VALID_PAGE_PER_MAG)
+// ---------------------------------------------------------------------------
+// struct that holds the state of the ttx decoder in the master process
+//
+#define HEADER_CHECK_LEN         12
+#define HEADER_CHECK_LEN_STR    "12"   // for debug output
+#define HEADER_CHECK_MAX_ERRORS   2
 
-typedef struct
+static struct
 {
-   uchar   pkgCount;
-   uchar   lastPkg;
-   uchar   okCount;
-} PAGE_SCAN_STATE;
-
+   uint     epgPageNo;
+   bool     isEpgPage;
+   bool     bHeaderCheckInit;
+   uchar    lastPageHeader[HEADER_CHECK_LEN];
+} ttxState;
 
 // ----------------------------------------------------------------------------
 // local variables
@@ -80,9 +86,6 @@ static EPGDB_QUEUE * pBlockQueue = NULL;
 static uint        epgStreamAppId;
 static bool        enableAllTypes;
 
-static PAGE_SCAN_STATE scanState[NXTV_VALID_PAGE_COUNT];
-static int             lastMagIdx[8];
-
 
 // ----------------------------------------------------------------------------
 // Take a pointer the control and string bit fields from the block decoder and
@@ -136,7 +139,7 @@ static void EpgStreamConvertBlock( const uchar *pBuffer, uint blockLen, uchar st
       pBlock->origChkSum    = pBuffer[2];
       pBlock->origBlkLen    = blockLen;
 
-      EpgTxtDump_Block(&pBlock->blk, pBlock->type, stream);
+      EpgDumpRaw_IncomingBlock(&pBlock->blk, pBlock->type, stream);
 
       if (enableAllTypes || (type <= EPGDBACQ_TYPE_AI))
       {
@@ -153,7 +156,7 @@ static void EpgStreamConvertBlock( const uchar *pBuffer, uint blockLen, uchar st
       }
    }
    else
-      EpgTxtDump_UnknownBlock(type, blockLen, stream);
+      EpgDumpRaw_IncomingUnknown(type, blockLen, stream);
 }
 
 // ---------------------------------------------------------------------------
@@ -259,7 +262,7 @@ static uint EpgStreamAddData( const uchar * dat, uint restLine, bool isNewBlock
    if (isNewBlock)
    {
       if (psd->haveBlock && (psd->recvLen > 0))
-         debug1("missing data for last block with recvLen=%d - discard block\n", psd->recvLen);
+         debug1("missing data for last block with recvLen=%d - discard block", psd->recvLen);
 
       psd->haveBlock  = TRUE;
       psd->haveHeader = FALSE;
@@ -296,7 +299,7 @@ static uint EpgStreamAddData( const uchar * dat, uint restLine, bool isNewBlock
             psd->blockLen = ((c2 >> 1) | (c3 << 3) | (c4 << 7)) + 4;  // + length of block len itself
             psd->haveHeader = TRUE;
 
-            dprintf4("               start block recv stream %d, appID=%d, len=%d, startlen=%d\n", /*packNo, blockPtr,*/ streamOfPage+1, psd->appID, psd->blockLen, restLine);
+            dprintf4("               start block recv stream %d, appID=%d, len=4+%d, startlen=4+%d\n", /*packNo, blockPtr,*/ streamOfPage+1, psd->appID, psd->blockLen - 4, restLine);
          }
          else
          {  // hamming error
@@ -329,22 +332,36 @@ static uint EpgStreamAddData( const uchar * dat, uint restLine, bool isNewBlock
 
 // ---------------------------------------------------------------------------
 // Process EPG data packet
-// - the function uses two separate internal states the two streams
-// - a single teletext packet can produce several blocks
+// - the function uses two separate internal states for the two streams
+// - a single teletext packet can produce several EPG blocks
 //
-void EpgStreamDecodePacket( uchar packNo, const uchar * dat )
+static void EpgStreamDecodePacket( uchar packNo, const uchar * dat )
 {
    NXTV_STREAM * const psd = &streamData[streamOfPage];
    schar c1;
    uint  blockPtr;
 
-   if ( (streamOfPage < NXTV_NO_STREAM) &&
-        (packNo <= streamData[streamOfPage].pkgCount) )
+   if (streamOfPage < NXTV_NO_STREAM)
+   {
+      if (packNo <= psd->lastPkg)
+      {  // packet sequence broken, probably due to missing page header
+         // don't know what stream this page belongs to -> discard it
+         debug2("missed page header: got pkg %d <= last pkg %d - discard page", packNo, psd->lastPkg);
+         streamOfPage = NXTV_NO_STREAM;
+      }
+      else if (packNo > streamData[streamOfPage].pkgCount)
+      {  // unexpected packet, probably from different page -> stop decoding current page
+         debug2("EpgStream-DecodePacket: invalid pkg no #%d > %d - discard page", packNo, streamData[streamOfPage].pkgCount);
+         streamOfPage = NXTV_NO_STREAM;
+      }
+   }
+
+   if (streamOfPage < NXTV_NO_STREAM)
    {
       if ( psd->haveBlock &&
           (packNo != psd->lastPkg + 1))
       {  // packet missing
-         debug2("missing packet %d (have %d) - discard block", psd->lastPkg + 1, packNo);
+         debug4("missing packet %d of %d (have %d) - discard block in stream %d", psd->lastPkg + 1, streamData[streamOfPage].pkgCount, packNo, streamOfPage);
          psd->haveBlock = FALSE;
       }
       psd->lastPkg = packNo;
@@ -355,43 +372,49 @@ void EpgStreamDecodePacket( uchar packNo, const uchar * dat )
 
          if (psd->haveBlock && (blockPtr > 1))
          {  // append data to a block
-            dprintf2("pkg=%2d, BP= 1: append %d bytes\n", packNo, blockPtr - 1);
-            EpgStreamAddData(dat + 1, blockPtr - 1, FALSE);
+            dprintf2("pkg=%2d, BP= 1: append up to %d bytes\n", packNo, blockPtr - 1);
+            blockPtr = 1 + EpgStreamAddData(dat + 1, blockPtr - 1, FALSE);
          }
-         else if (blockPtr == 40) dprintf1("BP=0xD -> no block start in this packet %d\n", packNo);
+         else if ((psd->haveBlock == FALSE) && (blockPtr == 40))
+            dprintf1("pkg=%2d: BP=0xD -> no block start in this packet\n", packNo);
 
          while (blockPtr < 40)
          {  // start of at least one new structure in this packet
-            if ( UnHam84Nibble(dat + blockPtr, &c1) && (c1 == 0x0c) )
+            if ( UnHam84Nibble(dat + blockPtr, &c1) )
             {
-               dprintf3("pkg=%2d, BP=%2d: start with %d bytes\n", packNo, blockPtr, 40 - 1 - blockPtr);
-               blockPtr += 1;
-               // write the data in the output buffer & process the EPG block when complete
-               blockPtr += EpgStreamAddData(dat + blockPtr, 40 - blockPtr, TRUE);
-
-               // skip filler bytes
-               while ((blockPtr < 40) && UnHam84Nibble(dat + blockPtr, &c1) && (c1 == 0x03))
+               if (c1 == 0x0c)
                {
+                  dprintf3("pkg=%2d, BP=%2d: start with %d bytes\n", packNo, blockPtr, 40 - 1 - blockPtr);
+                  blockPtr += 1;
+                  // write the data in the output buffer & process the EPG block when complete
+                  blockPtr += EpgStreamAddData(dat + blockPtr, 40 - blockPtr, TRUE);
+               }
+               else if (c1 == 0x03)
+               {  // filler byte -> skip one byte
                   dprintf2("pkg=%2d, BP=%2d: skipping filler byte\n", packNo, blockPtr);
                   blockPtr += 1;
                }
+               else
+               {  // unexpected content (lost sync) -> skip this packet
+                  debug3("pkg=%2d, BP=%2d: %02X: neither BS nor FB - skipping", packNo, blockPtr, c1);
+                  blockPtr = 40;
+               }
             }
             else
             {  // decoding error - skip this packet
-               debug4("struct header error: BP=%d appID=%x blockLen=%x bs=%x - skipping block", blockPtr, psd->appID, psd->blockLen, c1);
-               psd->haveBlock = FALSE;
+               debug4("pkg=%2d, BP=%2d: BS marker hamming error: BS=0x%02x; appID=%x", packNo, blockPtr, dat[blockPtr], psd->appID);
                blockPtr = 40;
             }
          }
       }
       else
       {
-         debug2("hamming error in BP=%x - discard packet %d", c1, packNo);
+         debug2("pkg=%2d: BP hamming error on value 0x%02x - discard pkg", packNo, dat[0]);
          psd->haveBlock = FALSE;
       }
    }
    else
-      debug3("EpgStream-DecodePacket: invalid stream=%d or packet=%d > %d", streamOfPage, packNo, streamData[streamOfPage].pkgCount);
+      dprintf1("pkg=%2d: discarded - not on a valid page\n", packNo);
 }
 
 // ---------------------------------------------------------------------------
@@ -399,7 +422,7 @@ void EpgStreamDecodePacket( uchar packNo, const uchar * dat )
 // - returns TRUE if this is an EPG page, i.e. if the rest of the packets
 //   should be passed into the decoder
 //
-bool EpgStreamNewPage( uint sub )
+static bool EpgStreamNewPage( uint sub )
 {
    uchar newCi, newPkgCount, firstPkg;
    bool  result = FALSE;
@@ -414,7 +437,7 @@ bool EpgStreamNewPage( uint sub )
          streamOfPage = NXTV_STREAM_NO_2;
          break;
       default:
-         debug1("EpgStream-NewPage: unexpected stream number %d - ignoring page", sub & 0xf00);
+         debug1("EpgStream-NewPage: unexpected stream number %d - ignoring page", (sub >> 8) & 0xf);
          streamOfPage = NXTV_NO_STREAM;
          break;
    }
@@ -448,7 +471,7 @@ bool EpgStreamNewPage( uint sub )
             }
          }
 
-         dprintf4("new page: CI=%d stream=%d pkgno=%d last=%d\n", newCi, streamOfPage, newPkgCount, firstPkg);
+         dprintf4("pkg= 0: ***NEW PAGE*** CI=%d stream=%d pkgno=%d last=%d\n", newCi, streamOfPage, newPkgCount, firstPkg);
          streamData[streamOfPage].ci = newCi;
          streamData[streamOfPage].pkgCount = newPkgCount;
          streamData[streamOfPage].lastPkg = firstPkg;
@@ -464,10 +487,125 @@ bool EpgStreamNewPage( uint sub )
    return result;
 }
 
+// ---------------------------------------------------------------------------
+// Check the page-header of the current page
+// - the acquisition control should immediately stop insertion of acquired
+//   blocks, until a new instance of an AI block has been received and
+//   its CNI evaluated
+// - the page header comparison fails if more than 3 characters are
+//   different; we need some tolerance here, sonce the page header
+//   is only parity protected
+//
+static bool EpgStreamPageHeaderCheck( const uchar * curPageHeader )
+{
+   uint  i, err;
+   schar dec;
+   bool  result = TRUE;
+
+   if (ttxState.bHeaderCheckInit)
+   {
+      err = 0;
+      for (i=0; i < HEADER_CHECK_LEN; i++)
+      {
+         dec = (schar)parityTab[curPageHeader[i]];
+         if ( (dec >= 0) && (dec != ttxState.lastPageHeader[i]) )
+         {  // count the number of differing characters
+            err += 1;
+         }
+      }
+
+      if (err > HEADER_CHECK_MAX_ERRORS)
+      {
+         debug2("EpgStream-PageHeaderCheck: %d diffs from header \"%" HEADER_CHECK_LEN_STR "s\" - stop acq", err, ttxState.lastPageHeader);
+         result = FALSE;
+      }
+   }
+   else
+   {  // first header after channel change -> copy header
+      for (i=0; i < HEADER_CHECK_LEN; i++)
+      {
+         dec = (schar)parityTab[ curPageHeader[i] ];
+         if (dec >= 0)
+         {
+            ttxState.lastPageHeader[i] = dec;
+         }
+         else
+            break;
+      }
+
+      // only OK if no parity decoding error
+      if (i >= HEADER_CHECK_LEN)
+      {
+         dprintf1("EpgStream-PageHeaderCheck: found header \"%" HEADER_CHECK_LEN_STR "s\"\n", ttxState.lastPageHeader);
+         ttxState.bHeaderCheckInit = TRUE;
+      }
+   }
+
+   return result;
+}
+
+// ---------------------------------------------------------------------------
+// Retrieve all available packets from VBI buffer and convert them to EPG blocks
+// - at this point the EPG database process/thread takes the data from the
+//   teletext acquisition process
+// - returns FALSE when an uncontrolled channel change was detected; the caller
+//   then should reset the acquisition
+//
+bool EpgStreamProcessPackets( void )
+{
+   const VBI_LINE * pVbl;
+   bool freePrevPkg   = FALSE;
+   bool channelChange = TRUE;
+
+   if ( (pVbiBuf != NULL) &&
+        (pVbiBuf->chanChangeReq == pVbiBuf->chanChangeCnf) )
+   {
+      // fetch the oldest packet from the teletext ring buffer
+      while ( (pVbl = TtxDecode_GetPacket(freePrevPkg)) != NULL )
+      {
+         freePrevPkg = TRUE;
+
+         //dprintf4("Process idx=%d: pkg=%d pg=%03X.%04X\n", pVbiBuf->reader_idx, pVbl->pkgno, pVbl->pageno, pVbl->sub);
+         if (pVbl->pkgno == 0)
+         {
+            // have to check page number again for the case of a page number change, in which
+            // the separate teletext thread would finish its current page with the wrong number
+            if (pVbl->pageno == ttxState.epgPageNo)
+            {
+               if ( EpgStreamPageHeaderCheck(pVbl->data + 13 - 5) == FALSE )
+               {
+                  ttxState.bHeaderCheckInit = FALSE;
+                  // abort processing remaining data
+                  channelChange = FALSE;
+                  break;
+               }
+               else
+                  ttxState.isEpgPage = EpgStreamNewPage(pVbl->sub);
+            }
+            else
+            {  // found an unexpected ttx page in the buffer
+               debug1("EpgStream-ProcessPackets: found non-EPG page %03X", pVbl->pageno);
+               ttxState.isEpgPage = FALSE;
+            }
+         }
+         else
+         {
+            if (ttxState.isEpgPage)
+            {
+               EpgStreamDecodePacket(pVbl->pkgno, pVbl->data);
+            }
+            else
+               debug1("EpgStream-ProcessPackets: discarding pkg %d on non-EPG page", pVbl->pkgno);
+         }
+      }
+   }
+   return channelChange;
+}
+
 // ---------------------------------------------------------------------------
 // Initialize the internal decoder state
 //
-void EpgStreamInit( EPGDB_QUEUE *pDbQueue, bool bWaitForBiAi, uint appId )
+void EpgStreamInit( EPGDB_QUEUE *pDbQueue, bool bWaitForBiAi, uint appId, uint pageNo )
 {
    memset(&streamData, 0, sizeof(streamData));
    streamOfPage   = NXTV_NO_STREAM;
@@ -475,12 +613,16 @@ void EpgStreamInit( EPGDB_QUEUE *pDbQueue, bool bWaitForBiAi, uint appId )
    enableAllTypes = !bWaitForBiAi;
    pBlockQueue    = pDbQueue;
 
+   // initialize the decoder state in the master process
+   ttxState.epgPageNo = pageNo;
+   ttxState.isEpgPage = FALSE;
+   ttxState.bHeaderCheckInit = FALSE;
+
    // note: intentionally using clear func instead of overwriting the pointer with NULL
    // (simplifies upper layers' state machines: start acq may be called while acq is already running)
    EpgDbQueue_Clear(pBlockQueue);
 }
 
-
 // ---------------------------------------------------------------------------
 // Free resources
 //
@@ -489,127 +631,3 @@ void EpgStreamClear( void )
    EpgDbQueue_Clear(pBlockQueue);
 }
 
-// ---------------------------------------------------------------------------
-//                              E P G   S C A N
-// ---------------------------------------------------------------------------
-
-// ---------------------------------------------------------------------------
-// Initialize state of EPG syntax scan
-//
-void EpgStreamSyntaxScanInit( void )
-{
-   // set all indices to -1
-   memset(lastMagIdx, 0xff, sizeof(lastMagIdx));
-   // set all ok counters to 0
-   memset(scanState, 0, sizeof(scanState));
-}
-
-// ---------------------------------------------------------------------------
-// Scan EPG page header syntax
-// - checks if page number is suggested EPG page number 0x1DF, mdF or mFd
-//   (where <m> stands for magazine number, <d> for 0..9 and F for 0x0F)
-//   and stream number in page subcode is 0 or 1.
-//
-void EpgStreamSyntaxScanHeader( uint page, uint sub )
-{
-   uint d1, d2;
-   int idx;
-
-   if ( ((sub & 0xf00) >> 8) < 2 )
-   {
-      // Compute an index into the page state array for a potential EPG page
-      d1 = page & 0x0f;
-      d2 = (page >> 4) & 0x0f;
-
-      if ((page & 0xff) == 0xdf)
-         idx = 0;
-      else if ( (d1 == 0xf) && (d2 < 0xa) )
-         idx = 1 + d2;
-      else if ( (d2 == 0xf) && (d1 < 0xa) )
-         idx = 11 + d1;
-      else
-         idx = -1;
-
-      if (idx >= 0)
-      {
-         idx += ((page >> 8) & 0x07) * NXTV_VALID_PAGE_PER_MAG;
-         dprintf2("EpgStream-ScanPageHeader: found possible EPG page %03X, idx=%d\n", page, idx);
-
-         scanState[idx].pkgCount = ((sub & 0x3000) >> (12-3)) | ((sub & 0x70) >> 4);
-         scanState[idx].lastPkg = 0;
-      }
-   }
-   else
-   {  // invalid stream number
-      idx = -1;
-   }
-
-   lastMagIdx[(page >> 8) & 0x07] = idx;
-}
-
-// ---------------------------------------------------------------------------
-// Scan EPG data packet syntax
-// - this is done in a very limited and simple way: only BP and BS are
-//   checked, plus SH for hamming errors
-// - a page is considered ok if at least two third of the packets are
-//   syntactically correct
-//
-bool EpgStreamSyntaxScanPacket( uchar mag, uchar packNo, const uchar * dat )
-{
-   PAGE_SCAN_STATE *psc;
-   schar bs, bp, c1, c2, c3, c4;
-   bool result = FALSE;
-
-   if ((mag < 8) && (lastMagIdx[mag] >= 0))
-   {
-      psc = &scanState[lastMagIdx[mag]];
-      if (packNo > psc->lastPkg)
-      {
-         if (packNo <= psc->pkgCount)
-         {
-            if ( UnHam84Nibble(dat, &bp) )
-            {
-               if (bp < 0x0c)
-               {
-                  bp = 1 + 3 * bp;
-
-                  if ( UnHam84Nibble(dat + bp, &bs) && (bs == 0x0c) &&
-                       UnHam84Nibble(dat + bp + 1, &c1) &&
-                       UnHam84Nibble(dat + bp + 2, &c2) &&
-                       UnHam84Nibble(dat + bp + 3, &c3) &&
-                       UnHam84Nibble(dat + bp + 4, &c4) )
-                  {
-                     psc->okCount++;
-                  }
-               }
-               else if (bp == 0x0c)
-               {
-                  bp = 1 + 3 * bp;
-
-                  if ( UnHam84Nibble(dat + bp, &bs) && (bs == 0x0c) &&
-                       UnHam84Nibble(dat + bp + 1, &c1) &&
-                       UnHam84Nibble(dat + bp + 2, &c2) )
-                  {
-                     psc->okCount++;
-                  }
-               }
-               else if (bp == 0x0d)
-               {
-                  psc->okCount++;
-               }
-
-               if (psc->okCount >= 16)
-               {  // this page has had enough syntactically correct packages
-                  dprintf2("EpgStream-ScanPacketSyntax: syntax ok: mag=%d, idx=%d\n", mag, lastMagIdx[mag]);
-                  lastMagIdx[mag] = -1;
-                  result = TRUE;
-               }
-            }
-         }
-      }
-      else
-         lastMagIdx[mag] = -1;
-   }
-   return result;
-}
-
