@@ -21,13 +21,14 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: uictrl.c,v 1.23 2002/02/28 19:22:16 tom Exp tom $
+ *  $Id: uictrl.c,v 1.39 2003/10/05 19:36:50 tom Exp $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGUI
 #define DPRINTF_OFF
 
 #include <stdio.h>
+#include <time.h>
 #include <string.h>
 
 #include <tcl.h>
@@ -43,18 +44,38 @@
 #include "epgctl/epgctxmerge.h"
 #include "epgctl/epgacqctl.h"
 #include "epgctl/epgscan.h"
+#include "epgctl/epgacqsrv.h"
 #include "epgctl/epgacqclnt.h"
 #include "epgui/epgmain.h"
 #include "epgui/pifilter.h"
-#include "epgui/pilistbox.h"
+#include "epgui/pibox.h"
 #include "epgui/uictrl.h"
 #include "epgui/menucmd.h"
 #include "epgui/statswin.h"
 #include "epgui/timescale.h"
-#include "epgui/xawtv.h"
 #include "epgctl/epgctxctl.h"
 
 
+// describing the state of the ui db
+typedef enum
+{
+   EPGDB_NOT_INIT,
+   EPGDB_WAIT_SCAN,
+   EPGDB_PROV_SCAN,
+   EPGDB_PROV_WAIT,
+   EPGDB_PROV_SEL,
+   EPGDB_ACQ_NO_FREQ,
+   EPGDB_ACQ_NO_TUNER,
+   EPGDB_ACQ_ACCESS_DEVICE,
+   EPGDB_ACQ_PASSIVE,
+   EPGDB_ACQ_WAIT,
+   EPGDB_ACQ_WAIT_DAEMON,
+   EPGDB_ACQ_OTHER_PROV,
+   EPGDB_EMPTY,
+   EPGDB_PREFILTERED_EMPTY,
+   EPGDB_OK
+} EPGDB_STATE;
+
 typedef struct
 {
    uint                     cni;
@@ -67,6 +88,119 @@ typedef struct
 static bool uiControlInitialized = FALSE;
 
 
+// ---------------------------------------------------------------------------
+// Generate a help message for an db error state
+//
+static const uchar * UiControl_GetDbStateMsg( EPGDB_STATE state )
+{
+   const uchar * pMsg;
+
+   switch (state)
+   {
+      case EPGDB_WAIT_SCAN:
+         pMsg = "Please wait until the provider scan has finished.";
+         break;
+
+      case EPGDB_PROV_SCAN:
+         pMsg = "There are no providers for Nextview data known yet. "
+                "Please start a provider scan from the Configure menu "
+                "(you only have to do this once). "
+                "During the scan all TV channels are checked for Nextview "
+                "transmissions. Among the found ones you then can select your "
+                "favorite provider and start reading in its TV programme database.";
+         break;
+
+      case EPGDB_PROV_WAIT:
+         pMsg = "There are no providers for Nextview data known yet. "
+                "Since you do not use the TV tuner as video input source, "
+                "you have to select a provider's channel at the external "
+                "video source by yourself. For a list of possible channels "
+                "see the README file or the nxtvepg Internet Homepage.";
+         break;
+
+      case EPGDB_PROV_SEL:
+         pMsg = "Please select your favorite provider from the Configure menu.";
+         break;
+
+      case EPGDB_ACQ_NO_FREQ:
+         pMsg = "The database of the currently selected provider is empty "
+                "and the provider's TV channel is unknown. Please do start "
+                "a provider scan from the Configure menu to find out the frequency "
+                "(this has to be done only this one time). Else, please make sure "
+                "you have tuned the TV channel of the selected Nextview provider "
+                "or select a different provider from the Configure menu.";
+         break;
+
+      case EPGDB_ACQ_NO_TUNER:
+         pMsg = "The database of the currently selected provider is empty. "
+                "Since you have not selected the internal TV tuner as input source, "
+                "you have to make sure yourself that "
+                "you have tuned the TV channel of the this provider "
+                "or select a different provider from the Configure menu.";
+         break;
+
+      case EPGDB_ACQ_ACCESS_DEVICE:
+         pMsg = "The database of the currently selected provider is empty. "
+                "The TV channel could not be changed because the video device is "
+                "kept busy by another application. Therefore you have to make sure "
+                "you have tuned the TV channel of the selected Nextview provider "
+                "or select a different provider from the Configure menu.";
+         break;
+
+      case EPGDB_ACQ_PASSIVE:
+         pMsg = "The database of the currently selected provider is empty. "
+                "You have configured acquisition mode to passive, so data for "
+                "this provider can only be acquired if you use another application "
+                "to tune to it's TV channel.";
+         break;
+
+      case EPGDB_ACQ_WAIT:
+         pMsg = "The database of the currently selected provider is empty. "
+                "Please wait a few seconds until Nextview data is received "
+                "or select a different provider from the Configure menu.";
+         break;
+
+      case EPGDB_ACQ_WAIT_DAEMON:
+         pMsg = "The database of the currently selected provider is empty. "
+                "The acquisition daemon is currently working on a different provider. "
+                "Either change the daemon's acquisition mode to \"Follow-UI\" "
+                "or select a different provider from the Configure menu.";
+         break;
+
+      case EPGDB_ACQ_OTHER_PROV:
+         pMsg = "The database of the currently selected provider is empty. "
+                "This provider is not in your selection for acquisition! "
+                "Please choose a different provider, or change the "
+                "acquisition mode from the Configure menu.";
+         break;
+
+      case EPGDB_EMPTY:
+         pMsg = "The database of the currently selected provider is empty. "
+                "Start the acquisition from the Control menu or select a "
+                "different provider from the Configure menu.";
+         break;
+
+      case EPGDB_PREFILTERED_EMPTY:
+         pMsg = "None of the programmes in this database match your network "
+                "preselection. Either add more networks for this provider or "
+                "select a different one in the Configure menus. Starting the "
+                "acquisition might also help.";
+         break;
+
+      case EPGDB_OK:
+         // db is now ready -> clear message display
+         pMsg = NULL;
+         break;
+
+      default:
+         fatal1("UiControl-GetDbStateMsg: unknown state %d", state);
+         pMsg = "Internal error";
+         break;
+   }
+
+   return pMsg;
+}
+
 // ---------------------------------------------------------------------------
 // Determine state of UI database and acquisition
 // - this is used to display a helpful message in the PI listbox as long as
@@ -104,7 +238,12 @@ static EPGDB_STATE UiControl_GetDbState( void )
    {  // AI present, but no PI in database
 
       // check if acquisition is working for the browser database
-      if ( EpgDbContextIsMerged(pUiDbContext) )
+      if ( (acqState.state == ACQDESCR_NET_CONNECT) ||
+           (acqState.state == ACQDESCR_DISABLED) ||
+           (acqState.state == ACQDESCR_SCAN) ||
+           (acqState.dbCni == 0) )
+         acqWorksOnUi = FALSE;
+      else if ( EpgDbContextIsMerged(pUiDbContext) )
          acqWorksOnUi = EpgContextMergeCheckForCni(pUiDbContext, acqState.dbCni);
       else
          acqWorksOnUi = (acqState.dbCni == uiCni);
@@ -199,9 +338,10 @@ void UiControl_CheckDbState( void )
       else
       {  // provider present -> check for PI
          pPreFilterContext = EpgDbFilterCopyContext(pPiFilterContext);
-         filterMask = pPreFilterContext->enabledFilters;
+         filterMask = pPreFilterContext->enabledPreFilters;
          // disable all filters except the expire time
-         EpgDbFilterDisable(pPreFilterContext, FILTER_ALL & ~FILTER_EXPIRE_TIME);
+         EpgDbFilterDisable(pPreFilterContext, FILTER_NONPERM);
+         EpgDbPreFilterDisable(pPreFilterContext, FILTER_PERM & ~FILTER_EXPIRE_TIME);
 
          pPiBlock = EpgDbSearchFirstPi(pUiDbContext, pPreFilterContext);
          if (pPiBlock == NULL)
@@ -212,7 +352,7 @@ void UiControl_CheckDbState( void )
          {
             if (filterMask & FILTER_NETWOP_PRE)
             {  // re-enable the network pre-filter
-               EpgDbFilterEnable(pPreFilterContext, FILTER_NETWOP_PRE);
+               EpgDbPreFilterEnable(pPreFilterContext, FILTER_NETWOP_PRE);
             }
 
             pPiBlock = EpgDbSearchFirstPi(pUiDbContext, pPreFilterContext);
@@ -232,7 +372,7 @@ void UiControl_CheckDbState( void )
    else
       state = EPGDB_NOT_INIT;
 
-   PiListBox_UpdateState(state);
+   PiBox_ErrorMessage(UiControl_GetDbStateMsg(state));
 }
 
 // ----------------------------------------------------------------------------
@@ -243,7 +383,7 @@ void UiControl_CheckDbState( void )
 //
 void UiControl_AiStateChange( ClientData clientData )
 {
-   uint target = (uint) clientData;
+   uint target = PVOID2UINT(clientData);
    const AI_BLOCK *pAiBlock;
 
    if ( (EpgDbContextGetCni(pUiDbContext) == 0) &&
@@ -256,9 +396,9 @@ void UiControl_AiStateChange( ClientData clientData )
       pUiDbContext = EpgContextCtl_Open(EpgDbContextGetCni(pAcqDbContext), CTX_FAIL_RET_DUMMY, CTX_RELOAD_ERR_ANY);
 
       // update acq CNI list in case follow-ui acq mode is selected
-      SetAcquisitionMode();
+      SetAcquisitionMode(NETACQ_KEEP);
       // display the data from the new db
-      PiListBox_Reset();
+      PiBox_Reset();
    }
 
    // check if the message relates to the browser db
@@ -273,13 +413,28 @@ void UiControl_AiStateChange( ClientData clientData )
          sprintf(comm, "wm title . {Nextview EPG: %s}\n", AI_GET_SERVICENAME(pAiBlock));
          eval_check(interp, comm);
 
-         // update the netwop filter bar and the netwop prefilter
-         PiFilter_UpdateNetwopList();
+         // generate the netwop mapping tables and update the netwop filter bar
+         sprintf(comm, "UpdateProvCniTable 0x%04X\n"
+                       "UpdateNetwopFilterBar\n",
+                       EpgDbContextGetCni(pUiDbContext));
+         eval_check(interp, comm);
+
+         // update network prefilters (restrict to user-selected networks)
+         PiFilter_SetNetwopPrefilter();
+
+         // update the language (if in automatic mode)
+         SetUserLanguage(interp);
+
+         // update the netwop prefilter and refresh the display
+         PiBox_AiStateChange();
       }
       else
       {  // no AI block in db -> reset window title to empty
          sprintf(comm, "wm title . {Nextview EPG}\n");
          eval_check(interp, comm);
+
+         // clear the programme display
+         PiBox_Reset();
       }
       EpgDbLockDatabase(pUiDbContext, FALSE);
    }
@@ -298,9 +453,9 @@ void UiControl_AiStateChange( ClientData clientData )
 // Add or update an EPG provider channel frequency in the rc/ini file
 // - called by acq control when the first AI is received after a provider change
 //
-void UiControlMsg_NewProvFreq( uint cni, ulong freq )
+void UiControlMsg_NewProvFreq( uint cni, uint freq )
 {
-   sprintf(comm, "UpdateProvFrequency {0x%04X %ld}\n", cni, freq);
+   sprintf(comm, "UpdateProvFrequency {0x%04X %d}\n", cni, freq);
    eval_check(interp, comm);
 }
 
@@ -308,7 +463,7 @@ void UiControlMsg_NewProvFreq( uint cni, ulong freq )
 // Query the frequency for the given provider stored in the rc/ini file
 // - used by acq control upon provider changes
 //
-ulong UiControlMsg_QueryProvFreq( uint cni )
+uint UiControlMsg_QueryProvFreq( uint cni )
 {
    return GetProvFreqForCni(cni);
 }
@@ -416,7 +571,8 @@ void UiControl_ReloadError( ClientData clientData )
 //
 void UiControlMsg_ReloadError( uint cni, EPGDB_RELOAD_RESULT dberr, CONTEXT_RELOAD_ERR_HAND errHand, bool isNewDb )
 {
-   uchar * pTmpStr, * pSavedResult;
+   const uchar * pTmpStr;
+   uchar       * pSavedResult;
    MSG_RELOAD_ERR * pMsg;
 
    if (errHand != CTX_RELOAD_ERR_NONE)
@@ -482,7 +638,7 @@ void UiControlMsg_ReloadError( uint cni, EPGDB_RELOAD_RESULT dberr, CONTEXT_RELO
 static void UiControl_MissingTunerFreq( ClientData clientData )
 {
    char * comm2 = xmalloc(2048);
-   uint cni = (uint) clientData;
+   uint cni = PVOID2UINT(clientData);
 
    sprintf(comm2, "tk_messageBox -type ok -icon warning -parent . "
                   "-message {Found a database (CNI %X) with unknown tuner frequency. "
@@ -500,7 +656,7 @@ static void UiControl_MissingTunerFreq( ClientData clientData )
 void UiControlMsg_MissingTunerFreq( uint cni )
 {
    if (uiControlInitialized)
-      AddMainIdleEvent(UiControl_MissingTunerFreq, (ClientData) cni, FALSE);
+      AddMainIdleEvent(UiControl_MissingTunerFreq, UINT2PVOID(cni), FALSE);
    else
       fprintf(stderr, "nxtvepg: warning: cannot tune channel for provider 0x%04X: frequency unknown\n", cni);
 }
@@ -520,7 +676,7 @@ static void UiControl_NetAcqError( ClientData clientData )
       comm2 = xmalloc(2048);
       sprintf(comm2, "tk_messageBox -type ok -icon warning -parent . "
                      "-message {An error occurred in the network connection to the acquisition "
-                               "server: %s."
+                               "server: %s"
                                "}\n", netState.cause);
       eval_check(interp, comm2);
       xfree(comm2);
@@ -598,21 +754,26 @@ void UiControlMsg_AcqEvent( ACQ_EVENT acqEvent )
          case ACQ_EVENT_STATS_UPDATE:
             StatsWin_StatsUpdate(DB_TARGET_ACQ);
             TimeScale_AcqStatsUpdate();
+            MenuCmd_AcqStatsUpdate();
             break;
 
          case ACQ_EVENT_PI_ADDED:
             TimeScale_AcqPiAdded();
             break;
 
+         case ACQ_EVENT_PI_EXPIRED:
+            TimeScale_ProvChange(DB_TARGET_UI);
+            TimeScale_ProvChange(DB_TARGET_ACQ);
+            StatsWin_StatsUpdate(DB_TARGET_UI);
+            StatsWin_StatsUpdate(DB_TARGET_ACQ);
+            break;
+
          case ACQ_EVENT_PI_MERGED:
             TimeScale_AcqPiMerged();
             break;
 
          case ACQ_EVENT_VPS_PDC:
             StatsWin_StatsUpdate(DB_TARGET_ACQ);
-            #ifndef WIN32
-            AddMainIdleEvent(Xawtv_PollVpsPil, NULL, TRUE);
-            #endif
             break;
       }
    }
@@ -652,7 +813,7 @@ bool UiControlMsg_AcqQueueOverflow( bool prepare )
       {
          if (EpgDbContextIsMerged(pUiDbContext))
             EpgContextMergeLockInsert(pUiDbContext, TRUE);
-         PiListBox_Lock(TRUE);
+         PiBox_Lock(TRUE);
       }
    }
    else
@@ -661,7 +822,7 @@ bool UiControlMsg_AcqQueueOverflow( bool prepare )
       {
          eval_check(interp, "C_ProvMerge_Start");
       }
-      PiListBox_Lock(FALSE);
+      PiBox_Lock(FALSE);
    }
 #endif
 
