@@ -1,5 +1,5 @@
 /*
- *  VBI capture driver for the Booktree 848/849/878/879 family
+ *  Tuner channel to frequency translation
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -16,7 +16,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: tvchan.h,v 1.4 2001/04/09 20:47:26 tom Exp tom $
+ *  $Id: tvchan.h,v 1.6 2002/05/11 15:42:50 tom Exp tom $
  */
 
 #ifndef __TVCHAN_H
@@ -26,11 +26,11 @@
 // ---------------------------------------------------------------------------
 // declaration of service interface functions
 //
-bool TvChannels_GetNext( uint *pChan, ulong *pFreq );
+bool TvChannels_GetNext( uint *pChan, uint *pFreq );
 int  TvChannels_GetCount( void );
 void TvChannels_GetName( uint channel, uchar * pName, uint maxNameLen );
 void TvChannels_SelectFreqTable( uint tableIdx );
-ulong TvChannels_NameToFreq( const char * pName );
+uint TvChannels_NameToFreq( const char * pName );
 
 
 #endif  // __TVCHAN_H
